/*
 * Copyright 2006 Nuova Systems, Inc.  All rights reserved.
 * Copyright 2013 Cisco Systems, Inc.  All rights reserved.
 */
#ident "$Id: paloint.h 6610 2007-10-15 19:13:01Z bsmith $"

#ifndef _PALO_INT_H
#define _PALO_INT_H

/*
 * MIPS CPU interrupt controller owns IRQs 0 - 7.
 *         MIPS IP 7 => Linux IRQ 23 (timer)
 *         MIPS IP 2 => Linux IRQ 18 (UART for now)
 *         MIPS SW 1 => Linux IRQ 17
 *         MIPS SW 0 => Linux IRQ 16
 */


/*
 * Hardware interrupts begin at IP 2.
 */
#define MIPSCPU_INT_HW0         2

/*
 * Palo Interrupt controller covers IRQ numbers from 32 - 192:
 *
 * HW 4 => MIPS IP 6 => Linux IRQ 160 - 191
 * HW 3 => MIPS IP 5 => Linux IRQ 128 - 159
 * HW 2 => MIPS IP 4 => Linux IRQ  96 - 127
 * HW 1 => MIPS IP 3 => Linux IRQ  64 -  95
 * HW 0 => MIPS IP 2 => Linux IRQ  32 -  63
 */
#define PALO_NUM_INTS           5
#define PALO_IRQS_PER_INT       32
#define PALO_NUM_IRQS           (PALO_IRQS_PER_INT * PALO_NUM_INTS)
#define PALO_IRQ_BASE           32

static void inline palo_int_from_irq(int irq, int *lvl, int *src)
{
    irq -= PALO_IRQ_BASE;
    *lvl = irq >> 5;
    *src = irq & 0x1f;
}

static int inline palo_int_to_irq(const int lvl, const int src)
{
    BUG_ON(src >= PALO_IRQS_PER_INT);
    return PALO_IRQ_BASE + (lvl << 5) + (src & 0x1f);
}

#endif
