/*
 * Copyright 2006 Nuova Systems, Inc.  All rights reserved.
 * Copyright 2009 Cisco Systems, Inc.  All rights reserved.
 */
#ident "$Id: palo_bus.c 43 2016-12-03 22:59:19Z buildsa $"

#include <linux/init.h>
#include <linux/platform_device.h>
#include <asm/io.h>
#include <palo_memsync.h>

extern void palo_net_setup(void);
extern void palo_rq2wq_setup(void);
static int palo_bus_setup(void);

static struct platform_device paloints_dev[] = {
	{ .name = "paloints", .id = 0,},
	{ .name = "paloints", .id = 1,},
	{ .name = "paloints", .id = 2,},
	{ .name = "paloints", .id = 3,},
	{ .name = "paloints", .id = 4,},
};

static struct platform_device palopcie_dev = {
	.name = "palopcie", .id = 0,
};

static struct platform_device paloi2c_dev[] = {
	{ .name = "paloi2c", .id = 0},
	{ .name = "paloi2c", .id = 1},
};

static struct platform_device paloecpu_dev[] = {
	{ .name = "paloecpu", .id = 0 },
	{ .name = "paloecpu", .id = 1 },
	{ .name = "paloecpu", .id = 2 },
#ifdef CONFIG_BASE_CRUZ
	{ .name = "paloecpu", .id = 3 },
	{ .name = "paloecpu", .id = 4 },
#endif
};

static struct platform_device palonic_dev = {
	.name = "palonic", .id = 0,
};

#ifdef CONFIG_BASE_CRUZ
static struct platform_device rc_dev = {
	.name = "rc", .id = 0,
};
#endif

static struct platform_device *devices[] = {
	&paloints_dev[0], /* [22-25]GPIOs */
	&paloints_dev[1], /* [0-1]enic0/1 [28-31]rnaxi0-3 INTs */
	&paloints_dev[2],
	// Enable these if find user-level consumers for them
	&paloints_dev[3],
	//&paloints_dev[4],
	&palopcie_dev,
	&paloi2c_dev[0],
	&paloi2c_dev[1],
	&paloecpu_dev[0],
	&paloecpu_dev[1],
	&paloecpu_dev[2],
#ifdef CONFIG_BASE_CRUZ
	&paloecpu_dev[3],
	&paloecpu_dev[4],
	&rc_dev,
#endif
	&palonic_dev,
};

#ifdef CONFIG_DMA_NONCOHERENT

static void (*arch_cache_wback_inv)(unsigned long start, unsigned long size);
static void (*arch_cache_wback)(unsigned long start, unsigned long size);

static void
palo_dma_cache_wback(unsigned long addr, unsigned long size)
{
	arch_cache_wback(addr, size);
	palo_memsync();
}

static void
palo_dma_cache_wback_inv(unsigned long addr, unsigned long size)
{
	arch_cache_wback_inv(addr, size);
	palo_memsync();
}

static void
palo_cache_init(void)
{
	/* install our own hooks */
	arch_cache_wback_inv = _dma_cache_wback_inv;
	arch_cache_wback = _dma_cache_wback;
	_dma_cache_wback_inv = palo_dma_cache_wback_inv;
	_dma_cache_wback = palo_dma_cache_wback;

	/* Make sure we were called after cpu_cache_init inits handlers */
	BUG_ON(arch_cache_wback_inv == NULL);
	BUG_ON(arch_cache_wback == NULL);
}

#endif /* CONFIG_DMA_NONCOHERENT */

int coherentio = 0;
int hw_coherentio = 0;

/*
 * Initialize the devices on the platform bus.
 */
static int __init
palo_bus_setup(void)
{
#ifdef CONFIG_DMA_NONCOHERENT
	palo_cache_init();
#endif /* CONFIG_DMA_NONCOHERENT */

	/* register all platform devices */
	platform_add_devices(devices, ARRAY_SIZE(devices));

#ifdef CONFIG_PALO_MNIC
	palo_net_setup();
#endif
#ifdef CONFIG_PALO_RQ2WQ
	palo_rq2wq_setup();
#endif
	return 0;
}

arch_initcall(palo_bus_setup);
