/*
 * Copyright (C) 2005 MIPS Technologies, Inc.  All rights reserved.
 *
 *  This program is free software; you can distribute it and/or modify it
 *  under the terms of the GNU General Public License (Version 2) as
 *  published by the Free Software Foundation.
 *
 *  This program is distributed in the hope it will be useful, but WITHOUT
 *  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 *  FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 *  for more details.
 *
 *  You should have received a copy of the GNU General Public License along
 *  with this program; if not, write to the Free Software Foundation, Inc.,
 *  59 Temple Place - Suite 330, Boston MA 02111-1307, USA.
 *
 */
#ident "$Id: palo_cmdline.c 21 2016-12-02 21:25:59Z buildsa $"
#include <linux/init.h>
#include <linux/string.h>
#include <linux/kernel.h>
#include <linux/proc_fs.h>
#include <asm/bootinfo.h>

#define PROM_ENVSIZE        1024

extern char arcs_cmdline[];

static char *prom_env[PROM_ENVSIZE / sizeof(char *)];

char *prom_getenv(const char *s)
{
	char *var, *p;
	int i, len;

	for (i = 0; prom_env[i] != NULL; i++) {
		var = prom_env[i];
		p = strchr(var, '=');
		if (p == NULL && strcmp(s, var) == 0)
			return var + strlen(var);
		len = p - var;
		if (len == strlen(s) && strncmp(var, s, len) == 0)
			return p + 1;
	}

	// XXX until we get prom env everywhere
	if (strcmp(s, "macaddr0") == 0) {
		return "000a0b0c0d00";
	}

	return NULL;
}

char * __init prom_getcmdline(void)
{
	return arcs_cmdline;
}

void __init prom_init_cmdline(void)
{
	int argc = fw_arg0;
	char **argv = (char **) fw_arg1;
	char **envp = (char **) fw_arg2;
	char *empty_env[1] = {NULL};
	char *d;
	int i, n, l;

	envp = (char **) fw_arg2;
	if (envp == NULL)
		envp = empty_env;

	l = sizeof(char *);
	for (i = 0; envp[i] != NULL; i++) {
		n = sizeof(char *) +strlen(envp[i]) + 1;
		if (l + n > PROM_ENVSIZE)
			break;
		l += n;
	}
	n = i;

	d = (char *) prom_env + (n + 1) * sizeof(char *);
	for (i = 0; i < n; i++) {
		prom_env[i] = d;
		strcpy(d, envp[i]);
		d += strlen(d) + 1;
	}
	prom_env[i] = NULL;

	strcpy(arcs_cmdline, "console=ttyS0,38400 rhash_entries=64");
	d = arcs_cmdline + strlen(arcs_cmdline);
	for (i = 1; i < argc; i++)
		d += sprintf(d, " %s", argv[i]);
}

#ifdef CONFIG_PROC_FS

static int prom_readproc(char *page, char **start, off_t off,
	int count, int *eof, void *data)
{
	int idx, llen = 0;
	off_t pos = 0;
	char *s;

	for (idx = 0; (s = prom_env[idx]) != NULL; idx++) {
		llen = strlen(s);
		if (pos + (llen + 1) > off)
			break;
		pos += llen + 1;
	}
	if (s != NULL) {
		s += (off - pos);
		llen -= (off - pos);
	}

	*start = page;

	while (count > 0 && s != NULL) {
		if (*s == '\0') {
			*page++ = '\n';
			--count;
			s = prom_env[++idx];
			if (s != NULL)
				llen = strlen(s);
		} else {
			if (count < llen)
				llen = count;
			memcpy(page, s, llen);
			page += llen;
			count -= llen;
			s += llen;
		}
	}

	*eof = (s == NULL);

	return(int) (page - *start);
}

static int __init prom_install_procfs(void)
{
	create_proc_read_entry("promenv", 0, NULL, prom_readproc, NULL);
	return 0;
}

static void __exit prom_remove_procfs(void)
{
	remove_proc_entry("promenv", NULL);
}

module_init(prom_install_procfs)
module_exit(prom_remove_procfs)
#endif

