/*
 * Copyright 2006 Nuova Systems, Inc.  All rights reserved.
 * Copyright 2009 Cisco Systems, Inc.  All rights reserved.
 */
#ident "$Id: palo_int.c 46 2016-12-03 23:17:24Z buildsa $"

#include <linux/init.h>
#include <linux/interrupt.h>
#include <asm/irq_cpu.h>
#include <asm/irq.h>
#include <asm/hardirq.h>
#include <paloint.h>
#include <prom.h>
#include <palo.h>
#include <palo/include/intrc_regs.h>
#include <palo/include/l2_mregs.h>
#include <palo/include/pcie_mregs.h>
#include <palo/include/net_mregs.h>
#include <palo/include/bmc_mregs.h>
#include <palo/include/dce_mregs.h>
#include <palo/include/mc_mregs.h>
#include <palo/include/ec_mregs.h>
#include <palo/include/wq_mregs_gbl.h>
#include <palo/include/ig_mregs_gbl.h>
#include <palo/include/wq_mregs_sched.h>
#include <palo/include/palo/iommu_mregs.h>
#include <palo/include/xlge_mregs.h>
#include <palo/include/sereno/xlge_mregs.h>
#include <palo/fw/mcpu/include/palo_conf.h>
#include "regio.h"

#include <linux/types.h>
#include <linux/init.h>
#include <linux/kernel_stat.h>
#include <linux/sched.h>
#include <linux/spinlock.h>
#include <linux/interrupt.h>
#include <linux/mc146818rtc.h>
#include <linux/smp.h>
#include <linux/timex.h>

#include <asm/hardirq.h>
#include <asm/div64.h>
#include <asm/cpu.h>
#include <asm/setup.h>
#include <asm/time.h>
#include <asm/irq.h>
#include <asm/mc146818-time.h>
#include <asm/msc01_ic.h>

#include <asm/mips-boards/generic.h>
#include <asm/mips-boards/prom.h>


#define INTRC_STATUS(lvl)       CKSEG1ADDR( INTRC_MREGS_STATUS(0, lvl) )
#define INTRC_MASKSET(lvl)      CKSEG1ADDR( INTRC_MREGS_MASK_SET(0, lvl) )
#define INTRC_MASKCLEAR(lvl)    CKSEG1ADDR( INTRC_MREGS_MASK_CLEAR(0, lvl) )

extern int palo_mipssim;

static inline int
clz(unsigned long x)
{
	__asm__ (
	"	.set	push					\n"
	"	.set	mips32					\n"
	"	clz	%0, %1					\n"
	"	.set	pop					\n"
	: "=r" (x)
	: "r" (x));

	return x;
}

/*
 * Version of ffs that only looks at bits 12..15.
 */
static inline unsigned int
irq_ffs(unsigned int pending)
{
	return -clz(pending) + 31 - CAUSEB_IP;
}

/*
 * Default interrupt dispatch function.
 */
asmlinkage void
plat_irq_dispatch(void)
{
	unsigned int pending = read_c0_cause() & read_c0_status() & ST0_IM;
	int irq;

	irq = irq_ffs(pending);

	if (irq >= 0)
		do_IRQ(MIPS_CPU_IRQ_BASE + irq);
	else
		spurious_interrupt();
}

static inline void
mask_palo_irq(unsigned int irq)
{
	int lvl, src;

	palo_int_from_irq(irq, &lvl, &src);
	writel(1 << src, (uint32_t *) INTRC_MASKSET(lvl));
	(void) readl((void *) INTRC_MASKSET(lvl));
}

static inline void
unmask_palo_irq(unsigned int irq)
{
	int lvl, src;

	palo_int_from_irq(irq, &lvl, &src);
	writel(1 << src, (uint32_t *) INTRC_MASKCLEAR(lvl));
	(void) readl((void *) INTRC_MASKCLEAR(lvl));
}

static inline void
clear_palo_irq(unsigned int irq)
{
	int lvl, src;

	palo_int_from_irq(irq, &lvl, &src);
	writel(1 << src, (uint32_t *) INTRC_STATUS(lvl));
	(void) readl((void *) INTRC_STATUS(lvl));
}

#define unmask_mips_hwintr(hw) \
	do {								      \
		set_c0_status(IE_IRQ0 << (hw));			      	      \
		irq_enable_hazard();					      \
	} while (0)

#define mask_mips_hwintr(hw) \
	do {								      \
		clear_c0_status(IE_IRQ0 << (hw));			      \
		irq_disable_hazard();					      \
	} while (0)

asmlinkage static void
palo_hw4_irqdispatch(void)
{
	uint32_t intrc_mask, intrc_status, irq;
	static uint32_t next_irq = 0;

	mask_mips_hwintr(4);
	intrc_mask = readl((void *) INTRC_MASKCLEAR(4));
	intrc_status = ~intrc_mask & readl((void *) INTRC_STATUS(4));
#define RR_IRQ
#ifdef RR_IRQ
	for (irq = next_irq; intrc_status; irq = (irq + 1) & 0x1f) {
		unsigned int irqbit = (1 << irq);
		if (intrc_status & irqbit) {
			intrc_status &= ~(irqbit);
			do_IRQ(PALO_IRQ_BASE + ((4) << 5) + irq);
		}
	}
#else
	while (intrc_status != 0) {
		irq = 31 - clz(intrc_status);
		intrc_status &= ~(1 << irq);
		do_IRQ(PALO_IRQ_BASE + ((4) << 5) + irq);
	}
#endif
	next_irq = irq;
	/* read mask reg to flush outstanding writes */
	(void) readl((void *) INTRC_MASKCLEAR(4));
	unmask_mips_hwintr(4);
}

/*
 * Each hardware interrupt line shared an almost identical dispatch
 * function - differing only in the Palo status register read, and the
 * corresponding IRQ block.  For minimum latency, instantiate parameterized
 * functions for hardware levels 4 down to 0.
 */
#define DECLARE_HW_IRQDISPATCH(hw) \
	asmlinkage static void						      \
	palo_hw##hw##_irqdispatch(void)					      \
	{								      \
		uint32_t intrc_mask, intrc_status, irq;			      \
									      \
		mask_mips_hwintr(hw);					      \
		intrc_mask = readl((void *)INTRC_MASKCLEAR(hw));	      \
		intrc_status = ~intrc_mask & readl((void *)INTRC_STATUS(hw)); \
		while (intrc_status != 0) {				      \
			irq = 31 - clz(intrc_status);			      \
			intrc_status &= ~(1 << irq);			      \
			do_IRQ(PALO_IRQ_BASE + ((hw) << 5) + irq);	      \
		}							      \
		/* read mask reg to flush outstanding writes */		      \
		(void)readl((void *)INTRC_MASKCLEAR(hw));		      \
		unmask_mips_hwintr(hw);					      \
	}
//DECLARE_HW_IRQDISPATCH(4)
DECLARE_HW_IRQDISPATCH(3)
DECLARE_HW_IRQDISPATCH(2)
DECLARE_HW_IRQDISPATCH(1)
DECLARE_HW_IRQDISPATCH(0)

static struct irqaction paloirq = {
	.handler = no_action,
	.flags	 = IRQF_DISABLED | IRQF_PERCPU | IRQF_NO_THREAD,
	.name	 = "Palo cascade"
};

static void
enable_palo_irq(struct irq_data *d)
{
	unsigned int irq = d->irq - MIPS_CPU_IRQ_BASE;

	unmask_palo_irq(irq);
}

static void
disable_palo_irq(struct irq_data *d)
{
	unsigned int irq = d->irq - MIPS_CPU_IRQ_BASE;

	mask_palo_irq(irq);
}

static void
ack_palo_irq(struct irq_data *d)
{
	unsigned int irq = d->irq - MIPS_CPU_IRQ_BASE;

	mask_palo_irq(irq);
	clear_palo_irq(irq);
}

static void
end_palo_irq(struct irq_data *d)
{
	unsigned int irq = d->irq - MIPS_CPU_IRQ_BASE;

	if (!(d->state_use_accessors &
		(IRQD_IRQ_INPROGRESS | IRQD_IRQ_DISABLED)))
		unmask_palo_irq(irq);
}

#if defined CONFIG_BASE_PALO || defined CONFIG_BASE_SERENO

static void __init
set_l2_msks(u_int8_t masked)
{
	u_int32_t msk = masked ? ~0 : 0;
	writereg(L2_MREGS_INT_L2_DATA_ECC_ERROR_MSK, msk);
	writereg(L2_MREGS_INT_L2_TAG_PARITY_ERROR_MSK, msk);
	writereg(L2_MREGS_INT_L2_REBUF_ECC_ERROR_MSK, msk);
#ifdef CONFIG_BASE_PALO
	writereg(L2_MREGS_INT_MEMCD_MSK, msk);
#endif /* CONFIG_BASE_PALO */
}

static void __init
set_iommu_msks(u_int8_t masked)
{
#ifdef CONFIG_BASE_PALO
	u_int32_t msk = masked ? ~0 : 0;
	if (palo_conf_has_iommu()) {
		writereg(IOMMU_MREGS_INT_IOMMU_ATC_TAG_ECC_ERROR_MSK, msk);
		writereg(IOMMU_MREGS_INT_IOMMU_ATC_DATA_ECC_ERROR_MSK, msk);
		writereg(IOMMU_MREGS_INT_IOMMU_WS_ECC_ERROR_MSK, msk);
		writereg(IOMMU_MREGS_INT_IOMMU_DT_ECC_ERROR_MSK, msk);
		writereg(IOMMU_MREGS_INT_IOMMU_LRU_ECC_ERROR_MSK, msk);
		writereg(IOMMU_MREGS_INT_IOMMU_REBUF_ECC_ERROR_MSK, msk);
		writereg(IOMMU_MREGS_INT_MISC_ERROR_MSK, msk);
	}
#endif /* CONFIG_BASE_PALO */
}

static void __init
set_pciec_msks(u_int8_t masked)
{
	u_int32_t msk = masked ? ~0 : 0;
	writereg(PCIEC_MREGS_INT_MAC_MSK, msk);
	writereg(PCIEC_MREGS_INT_SER_MSK, msk);
}

static void __init
set_misc_msks(u_int8_t masked)
{
	u_int32_t msk = masked ? ~0 : 0;
	writereg(MISC_MREGS_AER_MSK, msk);
	writereg(MISC_MREGS_INT_PIN_SPARE_MSK, msk);
#ifdef CONFIG_BASE_SERENO
	writereg(MISC_MREGS_BE_INT_ECC_ERROR_MSK, msk);
#endif /* CONFIG_BASE_SERENO */
}

static void __init
set_net_msks(u_int8_t masked)
{
	u_int64_t msk = masked ? ~0ULL : 0ULL;
	writereg64(NET_MREGS_INT_VEC_MSK, msk);
	writereg64(NET_MREGS_INT_VEC_EG0_NSPI_FRAMING_ERROR_MSK, msk);
	writereg64(NET_MREGS_INT_VEC_EG1_NSPI_FRAMING_ERROR_MSK, msk);
	writereg64(NET_MREGS_INT_VEC_IG0_NSPI_FRAMING_ERROR_MSK, msk);
	writereg64(NET_MREGS_INT_VEC_IG1_NSPI_FRAMING_ERROR_MSK, msk);
#ifdef CONFIG_BASE_SERENO
	writereg(NET_MREGS_BMC_INT_MSK, msk);
	writereg(NET_MREGS_BMC_INT_PCS_RX_MSK, msk);
#endif /* CONFIG_BASE_SERENO */
}

static void __init
set_bmc_msks(u_int8_t masked)
{
#ifdef CONFIG_BASE_PALO
	u_int32_t msk = masked ? ~0 : 0;
	writereg(BMC_MREGS_INT_MSK, msk);
#endif /* CONFIG_BASE_PALO */
}

static void __init
set_dce_msks(u_int8_t masked)
{
#ifdef CONFIG_BASE_PALO
	u_int32_t msk = masked ? ~0 : 0;
	writereg(DCE_MREGS_PORT0_INTR_MSK, msk);
	writereg(DCE_MREGS_PORT1_INTR_MSK, msk);
	writereg(DCE_MREGS_PORT0_SER_INTERRUPT_MSK, msk);
	writereg(DCE_MREGS_PORT1_SER_INTERRUPT_MSK, msk);
	writereg(DCE_MREGS_RLP0_INT_PTP_MSK, msk);
	writereg(DCE_MREGS_RLP1_INT_PTP_MSK, msk);
	writereg(DCE_MREGS_RLP0_INT_ECC_MSK, msk);
	writereg(DCE_MREGS_RLP1_INT_ECC_MSK, msk);
#endif /* CONFIG_BASE_PALO */
}

static void __init
set_mc_msks(u_int8_t masked)
{
#ifdef CONFIG_BASE_SERENO
	u_int32_t msk = masked ? ~0 : 0;

#define MC_MREGS_(reg, uif) \
    ((uif) == 0 ? \
     MC_MREGS0_BASE_ADDR + MC_MREGS_##reg##_OFFSET : \
     MC_MREGS1_BASE_ADDR + MC_MREGS_##reg##_OFFSET)

	if (palo_conf_is_asic()) {
		u_int8_t uif;

		for (uif = 0; uif < palo_conf_uif_count(); uif++) {
			writereg(MC_MREGS_(MX_SD0_INT_MSK, uif), msk);
			writereg(MC_MREGS_(MX_SD1_INT_MSK, uif), msk);
			writereg(MC_MREGS_(MX_SD2_INT_MSK, uif), msk);
			writereg(MC_MREGS_(MX_SD3_INT_MSK, uif), msk);

			if (readreg(MISC_MREGS_VERSION) >= 0xb0) {
				writereg(MC_MREGS_(BMC_INT_MSK, uif), msk);
				writereg(MC_MREGS_(BMC_INT_PCS_RX_MSK, uif),
					msk);
				writereg(MC_MREGS_(INT_BMC_RX_EBUF_PPM_ERR_MSK,
					uif), msk);
				writereg(MC_MREGS_(INT_BMC_TX_EBUF_PPM_ERR_MSK,
					uif), msk);
			}
		}
	}
#endif /* CONFIG_BASE_SERENO */
}

static void __init
set_ec_msks(u_int8_t masked)
{
#ifdef CONFIG_BASE_SERENO
	u_int32_t msk = masked ? ~0 : 0;
	u_int8_t uif;

#define EC_MREGS_(reg, uif) \
    ((uif) == 0 ? \
     EC_MREGS0_BASE_ADDR + EC_MREGS_##reg##_OFFSET : \
     EC_MREGS1_BASE_ADDR + EC_MREGS_##reg##_OFFSET)

	for (uif = 0; uif < palo_conf_uif_count(); uif++) {
		writereg(EC_MREGS_(INT_VEC_MSK, uif), msk);
	}
#endif /* CONFIG_BASE_SERENO */
}

static void __init
set_xlge_msks(u_int8_t masked)
{
#ifdef CONFIG_BASE_SERENO
	u_int32_t msk = masked ? ~0 : 0;

#define XLGE_MREGS_(reg, uif) \
    ((uif) == 0 ? \
     XLGE_MREGS0_BASE_ADDR + XLGE_MREGS_##reg##_OFFSET : \
     XLGE_MREGS1_BASE_ADDR + XLGE_MREGS_##reg##_OFFSET)

	if (palo_conf_is_asic()) {
		u_int8_t uif;

		for (uif = 0; uif < palo_conf_uif_count(); uif++) {
			writereg(XLGE_MREGS_(XLGMAC_INT_MSK, uif), msk);
			writereg(XLGE_MREGS_(XLGMAC_INT_EDGE_MSK, uif), msk);
			writereg(XLGE_MREGS_(XLGMAC_INT_ERR_MSK, uif), msk);
			writereg(XLGE_MREGS_(XLGMAC_INT_RX_BIP_MSK, uif), msk);
			writereg(XLGE_MREGS_(XGPCS0_INT_MSK, uif), msk);
			writereg(XLGE_MREGS_(XGPCS0_INT_EDGE_MSK, uif), msk);
			writereg(XLGE_MREGS_(XGPCS1_INT_MSK, uif), msk);
			writereg(XLGE_MREGS_(XGPCS1_INT_EDGE_MSK, uif), msk);
			writereg(XLGE_MREGS_(XGPCS2_INT_MSK, uif), msk);
			writereg(XLGE_MREGS_(XGPCS2_INT_EDGE_MSK, uif), msk);
			writereg(XLGE_MREGS_(XGPCS3_INT_MSK, uif), msk);
			writereg(XLGE_MREGS_(XGPCS3_INT_EDGE_MSK, uif), msk);
			writereg(XLGE_MREGS_(XG0_INT_XG_MSK, uif), msk);
			writereg(XLGE_MREGS_(CT0_INT_MUX_MSK, uif), msk);
			writereg(XLGE_MREGS_(XG1_INT_XG_MSK, uif), msk);
			writereg(XLGE_MREGS_(CT1_INT_MUX_MSK, uif), msk);
			writereg(XLGE_MREGS_(XG2_INT_XG_MSK, uif), msk);
			writereg(XLGE_MREGS_(CT2_INT_MUX_MSK, uif), msk);
			writereg(XLGE_MREGS_(XG3_INT_XG_MSK, uif), msk);
			writereg(XLGE_MREGS_(CT3_INT_MUX_MSK, uif), msk);
		}
	}
#endif /* CONFIG_BASE_SERENO */
}

static void __init
set_ig_msks(u_int8_t masked)
{
	u_int32_t msk = masked ? ~0 : 0;
	writereg(IG_MREGS_GBL_INT_ECC_ERROR_MSK, msk);
	writereg(IG_MREGS_GBL_INT_IG_ERR_MSK, msk);
}

static void __init
set_eg_msks(u_int8_t masked)
{
	u_int64_t msk = masked ? ~0 : 0;
	writereg(WQ_MREGS_GBL_INT_SCHED_MISMATCH_MSK, msk);
	writereg(WQ_MREGS_GBL_INT_ECC_ERROR_MSK, msk);
	writereg64(WQ_MREGS_SCHED_INT_SCHED_ERROR_MSK, msk);
}

static void __init
set_rnaxi0_msks(u_int8_t masked)
{
	set_l2_msks(masked);
	set_iommu_msks(masked);
	set_pciec_msks(masked);
	set_misc_msks(masked);
	set_net_msks(masked);
	set_bmc_msks(masked);
	set_dce_msks(masked);
	set_mc_msks(masked);
	set_ec_msks(masked);
	set_xlge_msks(masked);
}

static void __init
set_rnaxi1_msks(u_int8_t masked)
{
	set_ig_msks(masked);
	set_eg_msks(masked);
}

static void __init
set_rnaxi2_msks(u_int8_t masked)
{
	// no cpu/proc intr msks
}

static void __init
set_rnaxi3_msks(u_int8_t masked)
{
	// don't touch pciep msks here, barrier might close
}

static void __init
set_intr_msks(void)
{
	//
	// Note that _MSKs behave like ENAble registers.  A "1" set in the
	// _MSK register allows the interrupt to propogate; a "0" in _MSK
	// prevents the interrupt from propogating.
	//
	// We block rnaxi0 interrupt sources because we will be enabling the
	// rnaxi0 interrupt to the cpu so we can receive the PCIEC_MREGS_INT_MAC
	// interrupt on pcie link changes and secondary bus changes.  If we
	// leave these other rnaxi0 sources enabled and one of them interrupts
	// due to an error then we would have to unravel the interrupt chain in
	// some common kernel interrupt handler, or run the risk of linux
	// disabling our pcie mac interrupt because we don't have a handler for
	// one of the many rnaxi0 sources.
	//
	// We can unblock the other rnaxi interrupt sources because we do not
	// (currently) enable any of those interrupts to the cpu.  paloerrd will
	// find, report, and acknowledge these sources.
	//
	set_rnaxi0_msks(0);
	set_rnaxi1_msks(1);
	set_rnaxi2_msks(1);
	set_rnaxi3_msks(1);
}

#else /* CONFIG_BASE */

static void __init
set_l2_msks(u_int8_t masked)
{
	u_int32_t msk = masked ? ~0 : 0; 

	writereg(L2M_MREGS_INT_L2_TAG_PARITY_ERROR_MSK, msk);
	writereg(L2M_MREGS_INT_L2_DATA_ECC_ERROR_MSK, msk);
	writereg(L2M_MREGS_INT_L2_REBUF_ECC_ERROR_MSK, msk);
	writereg(L2M_MREGS_MEMCD_INT_MSK, msk);
}

static void __init
set_net_msks(u_int8_t masked)
{
	u_int32_t msk = masked ? ~0 : 0; 
	
	writereg(NET_MREGS_INT_VEC_MSK, msk);
	writereg(NET_MREGS_INT_VEC_EG0_NSPI_FRAMING_ERROR_MSK, msk);
	writereg(NET_MREGS_INT_VEC_IG0_NSPI_FRAMING_ERROR_MSK, msk);
	writereg(NET_MREGS_INT_VEC_EG1_NSPI_FRAMING_ERROR_MSK, msk);
	writereg(NET_MREGS_INT_VEC_IG1_NSPI_FRAMING_ERROR_MSK, msk);
	writereg(NET_MREGS_BMC_INT_MSK, msk);
	writereg(NET_MREGS_BMC_INT_PCS_RX_MSK, msk);

}

static void __init
set_ec_msks(u_int8_t masked)
{
	u_int32_t msk = masked ? ~0 : 0; 
	
	writereg(EC_MREGS_INT_VEC_MSK, msk);
}

static void __init
set_pc_msks(u_int8_t masked)
{ 
	u_int32_t msk = masked ? ~0 : 0;

	writereg(PC_MREGS_INT_ECC_ERROR_MSK, msk);
	writereg(PC_MREGS_INT_ERR_MSK, msk);
}

static void __init
set_misc_msks(u_int8_t masked)
{
	u_int32_t msk = masked ? ~0 : 0;

	writereg(MISC_MREGS_INT_PIN_SPARE_MSK, msk);
}

static void __init
set_rnaxi0_msks(u_int8_t masked)
{
	set_l2_msks(masked);
	set_net_msks(masked);
	set_pc_msks(masked);
	set_ec_msks(masked);
	set_misc_msks(masked);
	// XLGE handled in libdcem
}

static void __init
set_intr_msks(void)
{
    set_rnaxi0_msks(0);
}

#endif /* CONFIG_BASE */

static struct irq_chip palo_irq_type = {
	.name		= "Palo",
	.irq_unmask	= enable_palo_irq,
	.irq_mask	= disable_palo_irq,
	.irq_ack	= ack_palo_irq,
	.irq_eoi	= end_palo_irq,

};

void __init
arch_init_irq(void)
{
	static void (*vtab[])(void) = {
		palo_hw0_irqdispatch,
		palo_hw1_irqdispatch,
		palo_hw2_irqdispatch,
		palo_hw3_irqdispatch,
		palo_hw4_irqdispatch,
	};
	int i, is_mipssim = 0;

	if (!cpu_has_vint)
		panic("palo requires vectored interrupts");

	/*
	 * Setup MIPS default processing.
	 */
	mips_cpu_irq_init();

#ifdef MISC_EMULATION_SIM
	is_mipssim = (palo_conf_em_type() == MISC_EMULATION_SIM);
	if (is_mipssim) {
		/*
		 * Install a vectored handler for HW 0 => MIPS IP 2.  This
		 * interrupt line is used for the APB peripherals such as the
		 * UART and for now is handled in MIPSSim rather than by the
		 * Palo model.
		 */
		set_vi_handler(MIPSCPU_INT_HW0, plat_irq_dispatch);
	}
#endif /* MISC_EMULATION_SIM */

	set_intr_msks();

	/*
	 * Point the Palo IRQs to the controller.
	 * The vectored interrupt handler is installed on-demand.
	 */
	for (i = PALO_IRQ_BASE; i < PALO_IRQ_BASE + PALO_NUM_IRQS; i++) {
		irq_set_chip_and_handler(i, &palo_irq_type, handle_level_irq);
		/*		irq_desc[i].chip = &palo_irq_type;
				irq_desc[i].depth = 2; */
	}

	/*
	 * Claim the IRQs we control.
	 */
	for (i = 0; i < ARRAY_SIZE(vtab); i++) {
		if (i == 0 && is_mipssim)
			continue;
		setup_irq(MIPS_CPU_IRQ_BASE + MIPSCPU_INT_HW0 + i, &paloirq);
		set_vi_handler(MIPSCPU_INT_HW0 + i, vtab[i]);
		writel(~0, (uint32_t *) INTRC_MASKSET(i));
		writel(~0, (uint32_t *) INTRC_STATUS(i));
	}
	/* read mask reg to flush outstanding writes */
	(void) readl((void *) INTRC_MASKCLEAR(0));
}
