/*
 * Carsten Langgaard, carstenl@mips.com
 * Copyright (C) 1999,2000 MIPS Technologies, Inc.  All rights reserved.
 *
 *  This program is free software; you can distribute it and/or modify it
 *  under the terms of the GNU General Public License (Version 2) as
 *  published by the Free Software Foundation.
 *
 *  This program is distributed in the hope it will be useful, but WITHOUT
 *  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 *  FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 *  for more details.
 *
 *  You should have received a copy of the GNU General Public License along
 *  with this program; if not, write to the Free Software Foundation, Inc.,
 *  59 Temple Place - Suite 330, Boston MA 02111-1307, USA.
 *
 * Putting things on the screen/serial line using YAMONs facilities.
 */
#ident "$Id: palo_printf.c 43 2016-12-03 22:59:19Z buildsa $"
#include <linux/init.h>
#include <linux/kernel.h>
#include <linux/serial_reg.h>
#include <asm/io.h>
#include <asm/mach-palo/palo.h>

// from kernel/printk.c
extern int console_printk_enabled;

static inline void __iomem *port_addr(int offset)
{
	return (void __iomem *)CKSEG1ADDR(PALO_SERIAL1_BASE + (offset << 2));
}

static inline unsigned int serial_in(int offset)
{
	return readl(port_addr(offset));
}

static inline void serial_out(int offset, int value)
{
	writel(value, port_addr(offset));
}

void prom_putchar(char c)
{
	if (console_printk_enabled) {
		while ((serial_in(UART_LSR) & UART_LSR_THRE) == 0)
			;
		serial_out(UART_TX, c);
	}
}

void __init prom_putchar_init(void)
{
	extern char *prom_getenv(const char *s);
	char *val;

	val = prom_getenv("console_printk_enabled");
	if (val) {
		console_printk_enabled = *val - '0';
		printk("console_printk_enabled = %d\n",
			console_printk_enabled);
	}
}
