/*
 * This file is subject to the terms and conditions of the GNU General Public
 * License.  See the file "COPYING" in the main directory of this archive
 * for more details.
 *
 * Copyright 2006-2007 Nuova Systems, Inc.  All rights reserved.
 * Copyright 2009 Cisco Systems, Inc.  All rights reserved.
 */
#ident "$Id: palo_serial.c 43 2016-12-03 22:59:19Z buildsa $"
#include <linux/init.h>
#include <linux/ioport.h>
#include <linux/serial_8250.h>
#include <linux/serial_core.h>
#include <linux/serial_reg.h>
#include <paloint.h>
#include <palo.h>
#include <palo/fw/mcpu/include/palo_conf.h>

#define PALO_UART_LEN   0x100
#define PALO_UART_USR   0x7c

static int last_lcr;

static void palo_serial_out(struct uart_port *p, int offset, int value)
{
	if (offset == UART_LCR)
		last_lcr = value;

	offset <<= p->regshift;
	writeb(value, p->membase + offset);
}

static unsigned int palo_serial_in(struct uart_port *p, int offset)
{
	offset <<= p->regshift;

	return readb(p->membase + offset);
}

static int palo_serial_isr(struct uart_port *p)
{
	unsigned int iir = p->serial_in(p, UART_IIR);
	if (serial8250_handle_irq(p, iir)) {
		return 1;
	} else if ((iir & UART_IIR_BUSY) == UART_IIR_BUSY) {
		(void)readb(p->membase + PALO_UART_USR);
		p->serial_out(p, UART_LCR, last_lcr);
		return 1;
	}
	return 0;
}

static struct uart_port palo_uart = {
	.mapbase        = PALO_SERIAL1_BASE,
	.iotype         = UPIO_MEM,
	.regshift       = 2,
	.flags          = UPF_BOOT_AUTOCONF | UPF_SKIP_TEST,
};

void __init palo_serial_setup(void)
{
	struct uart_port *up = &palo_uart;

	up->membase         = ioremap_nocache(up->mapbase, PALO_UART_LEN);
	up->private_data    = (char *)up->membase + PALO_UART_USR;

	up->uartclk         = palo_conf_uart_clk();
	up->irq             = palo_conf_uart_irq();
	up->handle_irq      = palo_serial_isr;
	up->serial_in       = palo_serial_in;
	up->serial_out      = palo_serial_out;

	if (early_serial_setup(up))
		printk(KERN_ERR "Early serial init failed\n");
}
