/*
 * Copyright (C) 2005 MIPS Technologies, Inc.  All rights reserved.
 *
 *  This program is free software; you can distribute it and/or modify it
 *  under the terms of the GNU General Public License (Version 2) as
 *  published by the Free Software Foundation.
 *
 *  This program is distributed in the hope it will be useful, but WITHOUT
 *  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 *  FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 *  for more details.
 *
 *  You should have received a copy of the GNU General Public License along
 *  with this program; if not, write to the Free Software Foundation, Inc.,
 *  59 Temple Place - Suite 330, Boston MA 02111-1307, USA.
 *
 */

/*
 * Copyright 2006 Nuova Systems, Inc.  All rights reserved.
 * Copyright 2009 Cisco Systems, Inc.  All rights reserved.
 */
#ident "$Id: palo_setup.c 153 2017-03-24 23:09:53Z sborikar $"


#include <linux/init.h>
#include <linux/string.h>
#include <linux/kernel.h>
#include <linux/ioport.h>
#include <linux/tty.h>
#include <linux/serial.h>
#include <linux/serial_core.h>
#include <linux/serial_8250.h>
#include <linux/utsname.h>
#include <linux/delay.h>

#include <asm/cpu.h>
#include <asm/bootinfo.h>
#include <asm/irq.h>
#include <asm/mach-palo/prom.h>
#include <asm/serial.h>
#include <asm/io.h>
#include <asm/time.h>
#include <asm/reboot.h>
#include <asm/traps.h>
#include <linux/pm.h>
#include <palo/fw/mcpu/include/ecpuctx.h>
#include <palo/include/intrc_mregs.h>
#include <palo/include/pcie_mregs.h>
#include <palo/include/misc_regs_defs.h>
#include <palo/include/bmc_mregs.h>
#include <palo/include/misc_mregs.h>
#include <palo/include/palo_gpio.h>
#include "regio.h"
#include <common/include/newjersey_defs.h>
#include <palo/fw/mcpu/include/palo_conf.h>
#include <palo/fw/mcpu/include/board_conf.h>
#ifdef CONFIG_BASE_CRUZ
#include <palo/include/rc_mregs.h>
#include <palo/include/cruz/l2m_mregs.h>
#include <asm/mach-palo/rc.h>
#endif
#include "libgpio.h"

extern void palo_serial_setup(void);
extern void prom_putchar_init(void);

const char *get_system_type(void)
{
	return "Nuova Palo";
}

static void
nocando(char *what)
{
	printk("%s not implemented - spinning.\n", what);
	local_irq_disable();
	for (;;);
}

static void
palo_machine_restart(char *command)
{
	nocando("Restart");
}

static void
palo_machine_halt(void)
{
	nocando("Halt");
}

static void
palo_machine_poweroff(void)
{
	nocando("Power Off");
}

static void
palo_nmi_setup(void)
{
	extern void nmi_handler(void);

	ecpupage_t *ecp = ioremap_nocache(ECPUCTX_MEMBASE, ECPUCTX_MEMSIZE);
	ecpuctx_t *ctx = &ecp->ctx[0];

	ctx->nmivec = (u_int32_t) nmi_handler;
	ctx->nmimagic = ECPUCTX_NMIMAGIC;
}

#if defined CONFIG_BASE_PALO

static void
resp_err_get(u8 *validp, u8 *idp, u8 *addrerrp, u8 *dataerrp)
{
	u_int32_t base = (u_int32_t) KSEG1ADDR(INTRC_MREGS_BASE_ADDR);
	INTRC_MREGS_RESP_ERR_GET(base, validp, idp, addrerrp, dataerrp);
}

static void
resp_err_reset(void)
{
	u_int32_t base = (u_int32_t) KSEG1ADDR(INTRC_MREGS_BASE_ADDR);
	INTRC_MREGS_RESP_ERR_MAKE(base, 1, 0, 0, 0);
}

static void
resp_info_get(int idx, u32 *addrp, u8 *pendingp, u8 *sizep)
{
	u_int32_t base = (u_int32_t) KSEG1ADDR(INTRC_MREGS_BASE_ADDR);
	INTRC_MREGS_PENDING_RESP_INFO_GET(base, idx, addrp, pendingp, sizep);
}

static int
resp_info_maxcnt(void)
{
	return INTRC_MREGS_PENDING_RESP_INFO_MAXCNT;
}

#elif defined CONFIG_BASE_SERENO || defined CONFIG_BASE_CRUZ

static void
resp_err_get(u8 *validp, u8 *idp, u8 *addrerrp, u8 *dataerrp)
{
	u_int32_t base = (u_int32_t) KSEG1ADDR(INTRC_MREGS_BASE_ADDR);
	INTRC_MREGS_RESP_ERR0_GET(base, validp, idp, addrerrp, dataerrp);
}

static void
resp_err_reset(void)
{
	u_int32_t base = (u_int32_t) KSEG1ADDR(INTRC_MREGS_BASE_ADDR);
	INTRC_MREGS_RESP_ERR0_MAKE(base, 1, 0, 0, 0);
}

static void
resp_info_get(int idx, u32 *addrp, u8 *pendingp, u8 *sizep)
{
	u_int32_t base = (u_int32_t) KSEG1ADDR(INTRC_MREGS_BASE_ADDR);
	INTRC_MREGS_PENDING_RESP_INFO0_GET(base, idx, addrp, pendingp, sizep);
}

static int
resp_info_maxcnt(void)
{
	return INTRC_MREGS_PENDING_RESP_INFO0_MAXCNT;
}

#endif /* CONFIG_BASE */

typedef struct pending_resp_info_s {
	u_int8_t valid;
	u_int8_t errflags;
#define ERRF_ADDR (1<<0)
#define ERRF_DATA (1<<1)
	u_int32_t addr;
} pending_resp_info_t;

static int
palo_get_pending_resp_info(pending_resp_info_t *p)
{
	struct {
		u_int32_t address;
		u_int8_t pending;
		u_int8_t size;
	} pri;
	struct {
		u_int8_t valid;
		u_int8_t id;
		u_int8_t addrerr;
		u_int8_t dataerr;
	} e;
	u_int32_t i;

	memset(p, 0, sizeof(*p));
	resp_err_get(&e.valid, &e.id, &e.addrerr, &e.dataerr);
	if (e.valid) {
		p->valid = 1;
		if (e.addrerr) p->errflags |= ERRF_ADDR;
		if (e.dataerr) p->errflags |= ERRF_DATA;
		for (i = 0; i < resp_info_maxcnt(); i++) {
			resp_info_get(i, &pri.address, &pri.pending, &pri.size);
			if (i == e.id)
				p->addr = pri.address;
		}
	}
	return p->valid;
}

static int
palo_report_pending_resp_info(void)
{
	struct {
		u_int32_t address;
		u_int8_t pending;
		u_int8_t size;
	} pri;
	struct {
		u_int8_t valid;
		u_int8_t id;
		u_int8_t addrerr;
		u_int8_t dataerr;
	} e;
	static const char *prisize[] = {
		"LINE", "WORD", "SUB-WORD", "OTHER"
	};
	u_int32_t i;

	resp_err_get(&e.valid, &e.id, &e.addrerr, &e.dataerr);
	if (e.valid) {
		printk("NAXI error:");
		if (e.addrerr) printk(" ADDRESS");
		if (e.dataerr) printk(" DATA");
		printk("\n");
		for (i = 0; i < resp_info_maxcnt(); i++) {
			resp_info_get(i, &pri.address, &pri.pending, &pri.size);
			printk("%c %2d: %08x %c %-9s",
				(i == e.id) ? '*' : ' ',
				i,
				pri.address,
				pri.pending ? 'P' : ' ',
				prisize[pri.size]);
			if ((i & 1) != 0)
				printk("\n");
		}
	}
	return e.valid;
}

static int
palo_pciep_access(pending_resp_info_t *p)
{
#if defined CONFIG_BASE_PALO || defined CONFIG_BASE_SERENO
#define PCIEP_START PCIEP_MREGS_BASE_ADDR
#define PCIEP_SIZE  0x00400000
#define PCIEP_END   (PCIEP_START + PCIEP_SIZE)
#elif defined CONFIG_BASE_CRUZ
#define PCIEP_START PPSP_MREGS0_BASE_ADDR
#define PCIEP_SLICE (PPSP_MREGS1_BASE_ADDR - PPSP_MREGS0_BASE_ADDR)
#define PCIEP_SIZE  (PCIEP_SLICE * PPSP_MREGS_MAXCNT)
#define PCIEP_END   (PCIEP_START + PCIEP_SIZE)
#endif /* CONFIG_BASE */

	if (p->valid && (p->errflags & ERRF_ADDR)) {
		if (p->addr >= PCIEP_START && p->addr < PCIEP_END) {
			return(1);
		}
	}
	return(0);
}

#ifdef CONFIG_BASE_CRUZ
static int
cruz_rc_access(pending_resp_info_t *p)
{
#define RC_START	RC_MREGS_BASE_ADDR
#define RC_END		RC_MREGS_STA_SPARE7

    if (p->valid && (p->errflags & ERRF_ADDR)) {
        if (p->addr >= RC_START && p->addr < RC_END) {
            return (1);
        }
    }
    if (p->valid && (p->errflags & ERRF_ADDR)) {
        if (p->addr >= PALOMEM_RC_BASE &&
                p->addr < (PALOMEM_RC_BASE + PALOMEM_RC_SIZE)) {
            printk(KERN_ERR "RC Address space access without remap window"
                    "configuration\n");
            return (1);
        }
    }
	return (0);
}

#define PRINT_REG(reg) \
    printk("%s:0x%x\n", #reg, readreg(reg));

#define DENALI_ECC_DATAOUT_CORRECTED   (1 << 0)
#define DENALI_ECC_DATAOUT_UNCORRECTED (1 << 1)
static void
dump_ddr_ecc_stat(void)
{
	unsigned int val, val1;

	PRINT_REG(L2M_MREGS_INT_L2_TAG_PARITY_ERROR);
	PRINT_REG(L2M_MREGS_INT_L2_DATA_ECC_ERROR);
	PRINT_REG(L2M_MREGS_INT_L2_REBUF_ECC_ERROR);
	PRINT_REG(L2M_MREGS_MEMCD_INT);
	val1 = readreg(L2M_MREGS_MEMCD_INT);
	if (val1 & DENALI_ECC_DATAOUT_CORRECTED) {
		printk("Detected DENALI_ECC_DATAOUT_CORRECTED\n");

		val = readreg(L2M_MREGS_DBN_DENALI_CTL_41);
		printk("L2M_MREGS_DBN_DENALI_CTL_41 ADDR: %ld\n",
				val & L2M_MREGS_DBN_DENALI_CTL_41_ECC_C_ADDR);

		val = readreg(L2M_MREGS_DBN_DENALI_CTL_42);
		printk("L2M_MREGS_DBN_DENALI_CTL_42 SYND: %ld\n",
				val & L2M_MREGS_DBN_DENALI_CTL_42_ECC_C_SYND);

		val = readreg(L2M_MREGS_DBN_DENALI_CTL_43);
		printk("L2M_MREGS_DBN_DENALI_CTL_43 DATA: %ld\n",
				val & L2M_MREGS_DBN_DENALI_CTL_43_ECC_C_DATA);

		val = readreg(L2M_MREGS_DBN_DENALI_CTL_44);
		printk("L2M_MREGS_DBN_DENALI_CTL_44 DATA: %ld\n",
				val & L2M_MREGS_DBN_DENALI_CTL_44_ECC_C_DATA);

		val = readreg(L2M_MREGS_DBN_DENALI_CTL_45);
		printk("L2M_MREGS_DBN_DENALI_CTL_45 ID: %ld\n",
		(val & L2M_MREGS_DBN_DENALI_CTL_45_ECC_C_ID) >>
		L2M_MREGS_DBN_DENALI_CTL_45_ECC_C_ID_SHIFT);
	}
	if (val1 & DENALI_ECC_DATAOUT_UNCORRECTED) {
		printk("Detected DENALI_ECC_DATAOUT_UNCORRECTED\n");

		val = readreg(L2M_MREGS_DBN_DENALI_CTL_37);
		printk("L2M_MREGS_DBN_DENALI_CTL_37 ADDR: %ld\n",
				val & L2M_MREGS_DBN_DENALI_CTL_37_ECC_U_ADDR);

		val = readreg(L2M_MREGS_DBN_DENALI_CTL_38);
		printk("L2M_MREGS_DBN_DENALI_CTL_38 SYND: %ld\n",
				val & L2M_MREGS_DBN_DENALI_CTL_38_ECC_U_SYND);

		val = readreg(L2M_MREGS_DBN_DENALI_CTL_39);
		printk("L2M_MREGS_DBN_DENALI_CTL_39 DATA: %ld\n",
				val & L2M_MREGS_DBN_DENALI_CTL_39_ECC_U_DATA);

		val = readreg(L2M_MREGS_DBN_DENALI_CTL_40);
		printk("L2M_MREGS_DBN_DENALI_CTL_40 DATA: %ld\n",
				val & L2M_MREGS_DBN_DENALI_CTL_40_ECC_U_DATA);

		val = readreg(L2M_MREGS_DBN_DENALI_CTL_45);
		printk("L2M_MREGS_DBN_DENALI_CTL_45 ID: %ld\n",
		(val & L2M_MREGS_DBN_DENALI_CTL_45_ECC_U_ID) >>
		L2M_MREGS_DBN_DENALI_CTL_45_ECC_U_ID_SHIFT);
	}
	writereg(L2M_MREGS_MEMCD_INT, val1);
}
#endif

static int
palo_be_handler(struct pt_regs *regs, int is_fixup)
{
	pending_resp_info_t p;

	palo_get_pending_resp_info(&p);
	if (palo_pciep_access(&p)) {
		printk("Discarding bus error accessing pciep addr 0x%x\n",
			p.addr);
		resp_err_reset();
		return MIPS_BE_DISCARD;
	}
#ifdef CONFIG_BASE_CRUZ
	if (cruz_rc_access(&p)) {
		printk("Discarding bus error accessing rc addr 0x%x\n", p.addr);
		return MIPS_BE_DISCARD;
	}
#endif
	palo_report_pending_resp_info();
#ifdef CONFIG_BASE_CRUZ
	dump_ddr_ecc_stat();
#endif
	resp_err_reset();
	return is_fixup ? MIPS_BE_FIXUP : MIPS_BE_FATAL;
}

static void
palo_be_init(void)
{
	resp_err_reset();
	board_be_handler = palo_be_handler;
}

#if defined CONFIG_BASE_CRUZ

static void
palo_montclair_writegpio(int val)
{
	int b0, b1;

	b0 = (val >> 0) & 0x1;
	b1 = (val >> 1) & 0x1;

	gpio_output(PL_MC_GPIO_POST_STATUS_0, b0);
	gpio_output(PL_MC_GPIO_POST_STATUS_1, b1);
}
#endif

static int
palo_panic_handler(struct notifier_block *this, unsigned long event, void *ptr)
{
#if defined CONFIG_BASE_CRUZ
	if (board_conf_is_montclair()) {
		/*
		 * Report fatal error.
		 */
		palo_montclair_writegpio(NJ_CRUZPOST_FAILED);
	}
#endif
	return NOTIFY_DONE;
}

static struct notifier_block panic_block = {
	.notifier_call = palo_panic_handler,
};

void __init plat_mem_setup(void)
{
	printk("Linux started...\n");

	_machine_restart = palo_machine_restart;
	_machine_halt = palo_machine_halt;
	pm_power_off = palo_machine_poweroff;
	board_nmi_handler_setup = palo_nmi_setup;
	board_be_init = palo_be_init;
	atomic_notifier_chain_register(&panic_notifier_list, &panic_block);
}

/*
 * ARM the pin-reset FPGA to ensure FLASH is reset when we reset.
 */
static void __init arm_pin_reset(void)
{
	if (board_conf_has_armflashreset()) {
		board_conf_arm_flashreset();
	}
}

/*
 * Set initial hostname.
 */
static void __init palo_sethostname(void)
{
	struct new_utsname *uts = init_utsname();
	strcpy(uts->nodename, "palo");
}

/*
 * Set initial core pattern.
 */
static void __init palo_setcorepattern(void)
{
	extern char core_pattern[];
	strcpy(core_pattern, "/cores/%t_%h_BCxx_MEZZxxxx_%e_core.%p");
}

#if defined CONFIG_BASE_SERENO

static void
enable_plls(void)
{
	// Enable Sereno CPLLs, Comet PLLs and SGMII PLL

	// 312.5Mhz PLL: Set to non-bypass and start bit to correct state
	// sro_misc_mipl_pll_mac word 0, word 1
	writereg(0x12c00080, 0x10e4);
	writereg(0x12c00084, 0x4f1);

	// 644Mhz PLL: Set to non-bypass and start bit to correct state
	// sro_misc_mipl_pll_lc word 0, word 1
	writereg(0x12c00078, 0x401f4);
	writereg(0x12c0007c, 0x7f1);

	// 312.5Mhz PLL: Set clear reset and start (H->L) transition enables VCO
	// sro_misc_mipl_pll_mac word 0, word 1
	writereg(0x12c00080, 0x10e8);
	writereg(0x12c00084, 0x4f1);

	// 644Mhz PLL: Set clear reset and start (H->L) transition enables VCO
	// sro_misc_mipl_pll_lc word 0, word 1
	writereg(0x12c00078, 0x401f8);
	writereg(0x12c0007c, 0x7f1);

	// wait 100us?
	udelay(100);

	// 312.5Mhz PLL: Enable the PLL
	// sro_misc_mipl_pll_mac word 0, word 1
	writereg(0x12c00080, 0x10e9);
	writereg(0x12c00084, 0x4f1);

	// 625Mhz PLL: Enable the PLL
	// sro_misc_mipl_pll_lc word 0, word 1
	writereg(0x12c00078, 0x401f9);
	writereg(0x12c0007c, 0x7f1);

	// Enable SGMII PLL: Enable PLL, PLL multiplier 8X
	writereg(0x13100c18, 0x9);

	// Reset NE as PLLS have changed the frequency across async boundary
	writereg(0x12c00050, 0x81);
	(void) readreg(0x12c00050);
	writereg(0x12c00050, 0x1);

	// Reset MA as PLLS have changed the frequency across async boundary
	writereg(0x12c00050, 0x7);
	(void) readreg(0x12c00050);
	writereg(0x12c00050, 0x1);
}

static void
enable_minislaves(void)
{
	// serdes_cfg_10g
	// SERDES_CFG_10G: Minisalve reset
	//nword	$sro_mc0_sd_minislave_sw_rst	0x00000000
	writereg(0x13800014, 0);
	//nword	$sro_mc1_sd_minislave_sw_rst	0x00000000
	writereg(0x13c00014, 0);
	//nword	$sro_xlge0_xlg_minislave_sw_rst	0x00000000
	writereg(0x14800400, 0);
	//nword	$sro_xlge1_xlg_minislave_sw_rst	0x00000000
	writereg(0x14c00400, 0);

	// uif 0
	writereg(0x14800c00, 0x521); // port 0
	writereg(0x14801600, 0x521); // port 1
	writereg(0x14801e00, 0x521); // port 2
	writereg(0x14802600, 0x521); // port 3
	// uif 1
	writereg(0x14c00c00, 0x521); // port 0
	writereg(0x14c01600, 0x521); // port 1
	writereg(0x14c01e00, 0x521); // port 2
	writereg(0x14c02600, 0x521); // port 3
}
#endif /* CONFIG_BASE_SERENO */

void __init prom_init(void)
{
	extern char *prom_getenv(const char *s);

	board_conf_init();
	arm_pin_reset();
	palo_serial_setup();
	prom_init_cmdline();
	prom_putchar_init();
	printk("LINUX started... (%s on %s)\n",
		prom_getenv("fwbuild"), board_conf_brd_type_name());
	prom_meminit();
	palo_sethostname();
	palo_setcorepattern();
#if defined CONFIG_BASE_SERENO
	if (palo_conf_is_sereno()) {
		enable_plls();
		enable_minislaves();
	}
#endif /* CONFIG_BASE_SERENO */
}
