#ident "$Id: palo_time.c 43 2016-12-03 22:59:19Z buildsa $"
#include <linux/clocksource.h>
#include <linux/init.h>
#include <linux/clockchips.h>
#include <linux/interrupt.h>
#include <linux/irq.h>
#include <asm/time.h>
#include <palo/fw/mcpu/include/palo_conf.h>
#include <asm/setup.h>

#include <asm/time.h>

static unsigned int __init get_freq_hz(void)
{
	unsigned int hz = palo_conf_cpu_clk();

	mips_hpt_frequency = hz/2;
	return(hz);
}

static cycle_t c0_hpt_read(struct clocksource *cs)
{
	return read_c0_count();
}

static struct clocksource palo_cs = {
	.name		= "Palo_clock",
	.read		= c0_hpt_read,
	.mask		= CLOCKSOURCE_MASK(32),
	.flags		= CLOCK_SOURCE_IS_CONTINUOUS,
};

int __init init_palo_clocksource(void)
{
	palo_cs.rating = 300 + mips_hpt_frequency / 10000000;
	clocksource_register_hz(&palo_cs, mips_hpt_frequency);

	return 0;
}

static int mips_next_event(unsigned long delta,
                           struct clock_event_device *evt)
{
	unsigned int cnt;
	int res;

	cnt = read_c0_count();
	cnt += delta;
	write_c0_compare(cnt);
	res = ((int)(read_c0_count() - cnt) >= 0) ? -ETIME : 0;
	return res;
}

void mips_set_clock_mode(enum clock_event_mode mode,
				struct clock_event_device *evt)
{
	/* Nothing to do ...  */
}

static struct clock_event_device palo_cd = {
	.name           = "Palo_timer",
	.features       = CLOCK_EVT_FEAT_ONESHOT,
	.rating         = 300,
	.set_next_event = mips_next_event,
	.set_mode       = mips_set_clock_mode,
};

irqreturn_t c0_compare_interrupt(int irq, void *dev_id)
{
	struct clock_event_device *cd;

	if (read_c0_cause() & (1 << 30)) {
		/* Clear Count/Compare Interrupt */
		write_c0_compare(read_c0_compare());
		cd = &palo_cd;
		cd->event_handler(cd);
	}
	return IRQ_HANDLED;
}

static void mips_timer_dispatch(void)
{
	do_IRQ(palo_cd.irq);
}


struct irqaction c0_compare_irqaction = {
	.handler = c0_compare_interrupt,
	.flags = IRQF_TIMER,
	.name = "timer",
};

int __init init_palo_clockevent(void)
{
	clockevent_set_clock(&palo_cd, mips_hpt_frequency);
	palo_cd.irq  =   MIPS_CPU_IRQ_BASE + cp0_compare_irq;
	palo_cd.max_delta_ns	= clockevent_delta2ns(0x7fffffff, &palo_cd);
	palo_cd.min_delta_ns	= clockevent_delta2ns(0x300, &palo_cd);
	palo_cd.cpumask		= cpumask_of(smp_processor_id());

	clockevents_register_device(&palo_cd);
	setup_irq(palo_cd.irq, &c0_compare_irqaction);
	set_vi_handler(cp0_compare_irq, mips_timer_dispatch);

	return 0;
}

void __init plat_time_init(void)
{
	get_freq_hz();
	init_palo_clockevent();
	init_palo_clocksource();
}
