/*
 * Copyright 2013 Cisco Systems, Inc.  All rights reserved.
 */
#ident "$Id: ioctl-rc.c 43 2016-12-03 22:59:19Z buildsa $"

#include <linux/device.h>
#include <linux/module.h>
#include <linux/kernel.h>
#include <linux/errno.h>
#include <linux/types.h>
#include <linux/ioctl.h>
#include <linux/pci.h>
#include <linux/sched.h>
#include <linux/slab.h>
#include <linux/init.h>
#include <linux/interrupt.h>
#include <asm/mach-palo/rc.h>

#include "rc.h"
#include <palo/fw/mcpu/include/rc_dev.h>

static void add_rcdev_info(struct pci_dev *dev, struct rc_inventory *inv)
{
	inv->rc_devices[inv->ndev].bdf = make_bdf(dev->bus->number,
				PCI_SLOT(dev->devfn), PCI_FUNC(dev->devfn));
	inv->rc_devices[inv->ndev].slot = dev->phys_slot;
	inv->rc_devices[inv->ndev].vendor_id = dev->vendor;
	inv->rc_devices[inv->ndev].device_id = dev->device;
	inv->rc_devices[inv->ndev].subvendor_id = dev->subsystem_vendor;
	inv->rc_devices[inv->ndev].subdevice_id = dev->subsystem_device;
	inv->rc_devices[inv->ndev].pciclass = dev->class >> 16;
	inv->ndev++;
}

static int rc_ioctl_get_inventory(struct rc_inventory __user *uarg)
{
	struct rc_inventory inv;
	int r = 0;
	struct pci_dev *dev = NULL;
	int phys_slot = 0;
	struct pci_controller *hose = NULL;

	memset(&inv, 0, sizeof(struct rc_inventory));
	for_each_pci_dev(dev) {
		if (inv.ndev >= RC_MAX_DEVICES)
			break;
		hose = (struct pci_controller *)dev->sysdata;
		if (dev->bus->number == (hose->primary + 2)) {
			dev->phys_slot = ++phys_slot;
		} else {
			if (dev->bus && dev->bus->self) {
				dev->phys_slot = dev->bus->self->phys_slot;
			}
		}
		if ((dev->class >> 16) != PCI_BASE_CLASS_BRIDGE) {
			add_rcdev_info(dev, &inv);
		}
	}
	r = copy_to_user(uarg, &inv, sizeof(struct rc_inventory));
	return r;
}

static int rc_ioctl_get_events(struct rc_event __user *ev)
{
	return rc_copy_events_to_user(ev);
}

long rc_ioctl(struct file *filep, unsigned int cmd, unsigned long arg)
{
	void __user *uarg = (void __user *)arg;
	int r;

	switch (cmd) {
		case RC_IOC_GET_EVENTS:
			r = rc_ioctl_get_events(uarg);
			break;
		case RC_IOC_GET_INVENTORY:
			r = rc_ioctl_get_inventory(uarg);
			break;
		default:
			r = -ENOTTY;
			break;
	}
	return r;
}
