/*
 * Copyright 2012 Cisco Systems, Inc.  All rights reserved.
 */
#ident "$Id: ops-rc.c 43 2016-12-03 22:59:19Z buildsa $"
#include <linux/pci.h>
#include <linux/interrupt.h>
#include <linux/kthread.h>
#include <linux/sched.h>
#include <linux/notifier.h>
#include <linux/module.h>

#include <asm/io.h>
#include <asm/mach-palo/rc.h>
#include <asm/mach-palo/paloint.h>

#include <palo/include/pcie_mregs.h>
#include <palo/include/cruz/rc_mregs.h>
#include <palo/include/cruz/proc_mregs.h>
#include <palo/include/cruz/intrc_mregs.h>
#include <palo/include/intrc_regs_defs.h>
#include <palo/fw/mcpu/include/rc_dev.h>

#include "ecpu_res.h"
#include "rc.h"

#define RING_DEPTH  64
#define RING_SIZE   512
#define SPARE       0
#define ECC         0

ATOMIC_NOTIFIER_HEAD(rc_notify_list);
EXPORT_SYMBOL(rc_notify_list);

rc_vnic_info_t rc_vnic[MAX_VNIC_COUNT];
int error_stats[RC_NUM_PORTS][RC_ERROR_STAT_COUNT];
int key_index;

static int err_int_reg[] = {
	RC_MREGS_PPS0_INT_ERR,
	RC_MREGS_PPS2_INT_ERR
};

static int rc_mac[] = {
	RC_MREGS_PPS0_INT_MAC,
	RC_MREGS_PPS2_INT_MAC
};

static void process_ind_info(u8 process_index, int index)
{
	char *tlp_base, *info_base;
	int ind_rsp = 0, dest_vnic, arq_id;
	u8 cause, lbdf2vnic_match_index = 0;
	u8 resp_action = 1; // Decide based on TLP
	u32 rsp_status = 0;
	struct rc_notifier_param rc_param;

	memset(&rc_param, 0, sizeof(struct rc_notifier_param));

	info_base = (char *)rc_vnic[index].ind_base;
	tlp_base = &info_base[process_index * RING_SIZE];
	cause = tlp_base[0] & 0x1F;

	rc_param.bdf = rc_vnic[index].bdf;
	rc_param.ind_cause = cause;

	switch (cause) {
		case  RC_UP_BDF_MISS:
			printk(KERN_ERR "UP BDF lookup miss\n");
			break;
		case RC_POISONED:
			printk(KERN_ERR "poisoned packet\n");
			break;
		case RC_UNSUPPORTED:
			printk(KERN_ERR "unsupported request\n");
			break;
		case RC_ECRC_ERROR:
			printk(KERN_ERR "ecrc error\n");
			break;
		case RC_BDF_CAM_TRAP:
			printk(KERN_ERR "bdf cam trap\n");
			break;
		case RC_ACL_TRAP:
			printk(KERN_ERR "acl trap\n");
			break;
		case RC_ACL_RAM_ECC:
			printk(KERN_ERR "acl ram ecc error\n");
			break;
		case RC_MSG_REQ:
			do_IRQ(palo_int_to_irq(RC_DEVICE_LEGACY_INT_LVL,
                        RC_DEVICE_LEGACY_INT_BASE));
			break;
		default:
            printk(KERN_ERR "default: Cause:%d index:%d\n",
                    cause, process_index);
			break;
	}
	dest_vnic = ((*(int *)&tlp_base[0]) >> 25) & 0x7F;
	arq_id = (tlp_base[1] >> 1) & 0x7F;
	lbdf2vnic_match_index = (tlp_base[0] >> 16) * 0x7F;
	ind_rsp =  process_index  |  (resp_action <<  24) |
				(lbdf2vnic_match_index << 6) | (arq_id << 13) |
				((rsp_status & 7) << 20);

	if (cause != RC_MSG_REQ) {
		rc_evt_ind(rc_vnic[index].bdf, cause);
		atomic_notifier_call_chain(&rc_notify_list, 0, &rc_param);
    }
	writereg(RC_MREGS_DHS_UP_IND_RSP, ind_rsp);
	(void) readreg(RC_MREGS_DHS_UP_IND_RSP);

	memset(tlp_base, 0, RING_SIZE);
}

static irqreturn_t rc_intr_handler(int irq, void *arg)
{
	u8 i;
	u8 posted_id = 0;
	u32 ring_index;

	for (i = 0; i < key_index; i++) {
		if (rc_vnic[i].valid) {
			ring_index =
			  readreg(RC_MREGS_DHS_UP_INDIR_RING_INDEX + (i * 4));
			posted_id = ring_index  & 0x3F;
			posted_id += 1;

			while (rc_vnic[i].cpl_id != posted_id) {
				rc_vnic[i].cpl_id %= RING_DEPTH;
				process_ind_info(rc_vnic[i].cpl_id++, i);
			}
		}
	}
	return IRQ_HANDLED;
}

void get_vnic_cfg_ram(int index, int *buf)
{
	if (buf)
		readregs(RC_MREGS_DHS_UP_VNIC_CFG +
				(16 * index), &buf[0], VNIC_CFG_LENGTH);
}

static void rc_up_vnic_cfg_entry_make(u32 * reg_base,
                            u32 indirect_info_phys,
                            u8 ring_size,
                            u32 int_address,
                            u8 remote,
                            u8 spare,
                            u8 ecc)
{
	memset(reg_base, 0, 3 * sizeof (int));

	reg_base[0] = (indirect_info_phys >> 15)  |
				   (ring_size << 19) |
				   ((int_address << 22) & 0xFFC00000);
	reg_base[1] = (int_address >> 10) |
				   (remote << 21) |
				   (spare << 22);
	reg_base[2] = ecc & 0xFF;
}

void setup_rcindirect_table(int index, unsigned int intr_address,
		unsigned int *ind_base)
{
	// set vnic cfg table
	// assign interrupt vector
	// interrupt handler
	// Program the upstream cfg main mode
	// program vnic number basea
	static int set_int_addr;
	int phys_addr;
	int vnic_cfg_ram[3];
	int count = 0;
	int remote = 0;

up:
	rc_vnic[index].ind_base =
				kzalloc(RING_DEPTH * RING_SIZE, GFP_KERNEL);

	if (!rc_vnic[index].ind_base) {
		panic("No memory for indirect info base\n");
	} else if (((int)rc_vnic[index].ind_base % (32 * 1024))) {
		kfree(rc_vnic[index].ind_base);
		if (count > 5)
			panic("memory allocation failure RC vnic:"
					"%x\n", index);
		count++;
		goto up;
	}
	phys_addr = (int)virt_to_phys(rc_vnic[index].ind_base);
	if (!intr_address) {
		intrc_set_int_level_addr_enc(&set_int_addr,
				INTRC_MREGS_BASE_ADDR,
				0,
				RC_INDIRECT_INT_LVL,
				RC_INDIRECT_INT_SRC);
	} else {
		set_int_addr = intr_address;
		remote = 1;
		if (ind_base)
			*ind_base = (unsigned int)phys_addr;
	}
	rc_up_vnic_cfg_entry_make(&vnic_cfg_ram[0], phys_addr,
			6, set_int_addr / 4, remote, SPARE, ECC);

	writeregs(RC_MREGS_DHS_UP_VNIC_CFG +
				(16 * index), &vnic_cfg_ram[0],
				VNIC_CFG_LENGTH);
}

static void update_err_stats(int err_status, int port)
{
	int i = 0;
	for (i = 0; i < RC_ERROR_STAT_COUNT; i++) {
		if (err_status & (1 << i))
			error_stats[port][i]++;
	}
}

static void rc_pps_monitor(void)
{
	int err_status, i;

	for (i = 0; i < 2; i++) {
		err_status = readreg(err_int_reg[i]);

		if (err_status & MAC_RADM_RCVD_REQ_UR) {
			printk(KERN_ERR "pps%d: MAC received a request TLP that"
					"the device does not support\n", i);
		}
		if (err_status & MAC_RADM_MLF_TLP_ERR) {
			printk(KERN_ERR "pps%d: MAC received Malformed TLP\n", i);
		}
		if (err_status & MAC_RADM_ECRC_ERR) {
			printk(KERN_ERR "pps%d: MAC received TLP with ECRC"
					" Error\n", i);
		}
		if (err_status & UP_RESPERR_CA_UR) {
			printk(KERN_ERR "pps%d: Naxi response addr error"
					" for an Upstream request\n", i);
		}
		if (err_status & UP_RESPERR_EP) {
			printk(KERN_ERR "pps%d:Naxi response data error for an"
					"Upstream request\n", i);
		}
		if (err_status & MAC_PME_MSI) {
			printk(KERN_ERR "pps%d:RC MAC cfg_mac_pme_msi"
					" indication\n", i);
		}
		if (err_status & MAC_PME_INT) {
			printk(KERN_ERR "pps%d:RC MAC cfg_mac_pme_int"
					" indication\n", i);
		}
		if (err_status & MAC_SYS_ERR_RC) {
			printk(KERN_ERR "pps%d:RC MAC cfg_sys_err_rc"
					" indication\n", i);
		}
		if (err_status & MAC_AER_RC_ERR_MSI) {
			printk(KERN_ERR "pps%d:RC MAC aer_rc_err_msi"
					" indication\n", i);
		}
		if (err_status & MAC_AER_RC_ERR_INT) {
			printk(KERN_ERR "pps%d:RC MAC aer_rc_err_int"
					" indication\n", i);
		}
		if (err_status & DN_CPL_EOT_ERR) {
			printk(KERN_ERR "pps%d:Downstream CPL EOT "
					"unexpected err\n", i);
		}
		if (err_status & DN_CPL_LOW_ADDR_ERR) {
			printk(KERN_ERR "pps%d:Downstream CPL lower_addr"
					" mismatch err\n", i);
		}
		if (err_status & DN_CPL_BYTE_CNT_ERR) {
			printk(KERN_ERR "pps%d:Downstream CPL bytecnt "
					"mismatch err\n", i);
		}
		if (err_status & DN_REQ_DRF_ERR_UNC) {
			printk(KERN_ERR "pps%d:Downstream DRF uncorrectable"
					" ecc error\n", i);
		}
		if (err_status & DN_REQ_CRF_ERR_UNC) {
			printk(KERN_ERR "pps%d:Downstream CRF uncorrectable"
					" ecc error\n", i);
		}
		if (err_status & DN_CPU_REQ_ERR) {
			printk(KERN_ERR "pps%d:Downstream CPU request"
					" error\n", i);
		}
		if (err_status & DN_RSP_DERR) {
			printk(KERN_ERR "pps%d:Downstream response data"
					" error\n", i);
		}
		if (err_status & DN_RSP_AERR) {
			printk(KERN_ERR "pps%d:Downstream response address"
					" error\n", i);
		}
		if (err_status & DN_CPL_TIMEOUT) {
			printk(KERN_ERR "pps%d:Downstream completion"
					" timeout\n", i);
		}
		if (err_status & DN_CPL_STAT_CRS_ERR) {
			printk(KERN_INFO "pps%d:Downstream completion"
					" CFG_RETRY\n", i);
		}
		if (err_status & DN_CPL_STAT_UR_ERR) {
			printk(KERN_INFO "pps%d:Downstream completion "
					"status=UR\n", i);
		}
		if (err_status & DN_CPL_STAT_CA_ERR) {
			printk(KERN_ERR "pps%d:=Downstream completion"
					" status=CA error\n", i);
		}
		if (err_status & DN_CPL_ECRC_ERR) {
			printk(KERN_ERR "pps%d:Downstream completion"
					" ECRC error\n", i);
		}
		if (err_status & DN_CPL_TLP_POISONED) {
			printk(KERN_ERR "pps%d:Downstream completion"
					" TLP poisoned\n", i);
		}
		if (err_status & DN_CPL_TLP_ABORT) {
			printk(KERN_ERR "pps%d:Downstream completion"
					" TLP abort\n", i);
		}
		if (err_status & DN_CPL_DLLP_ABORT) {
			printk(KERN_ERR "pps%d:Downstream completion"
					" DLLP abort\n", i);
		}
		if (err_status & DN_CPL_UNEXP) {
			printk(KERN_ERR "pps%d:Downstream completion"
					" unexpected\n", i);
		}
		if (err_status & DN_REQ_UNS) {
			printk(KERN_ERR "pps%d:Downstream request"
					" unsupported\n", i);
		}
		// Clear all registers whatever set
		update_err_stats(err_status, i);
		writereg(err_int_reg[i], err_status);
	}
}

static void rc_mac_monitor(void)
{
	int status, i;

	for (i = 0; i < 2; i++) {
		status = readreg(rc_mac[i]);
		// Some bits will always be set no matter what the mask is.
		if (!(status & 0x5A00)) {
			writereg(rc_mac[i], status);
			readreg(rc_mac[i]);
			printk(KERN_WARNING "rc pps%d mac"
					" interrupt: %x\n", i, status);
		}
	}
}

static void rc_ecc_monitor(void)
{
	int status = 0;
	struct rc_notifier_param rc_param;
	int ecc_cause = 0, ecc_status = 0;

	memset(&rc_param, 0, sizeof(struct rc_notifier_param));

	status = readreg(RC_MREGS_INT_ERR);
	if (status) {
		printk(KERN_WARNING "rc_int_error: %x\n", status);
		writereg(RC_MREGS_INT_ERR, status);
	}
	if (!status)
		return;
	status = readreg(RC_MREGS_INT_ECC_ERROR);
	if (status) {
		printk(KERN_ERR "rc_int_ecc_error: %x\n", status);
		writereg(RC_MREGS_INT_ECC_ERROR, status);
		rc_param.ecc_cause = RC_INT_ECC_ERROR;
		rc_param.ecc_status = status;
		ecc_cause = RC_INT_ECC_ERROR;
		ecc_status = status;
	}

	status = readreg(RC_MREGS_PPS0_INT_ECC_ERROR);
	if (status) {
		printk(KERN_ERR "rc_pps0_int_ecc_error: %x\n", status);
		writereg(RC_MREGS_PPS0_INT_ECC_ERROR, status);
		rc_param.ecc_cause = RC_PPS0_INT_ECC_ERROR;
		rc_param.ecc_status = status;
		ecc_cause = RC_PPS0_INT_ECC_ERROR;
		ecc_status = status;
	}

	status = readreg(RC_MREGS_PPS2_INT_ECC_ERROR);
	if (status) {
		printk(KERN_ERR "rc_pps2_int_ecc_error: %x\n", status);
		writereg(RC_MREGS_PPS2_INT_ECC_ERROR, status);
		rc_param.ecc_cause = RC_PPS2_INT_ECC_ERROR;
		rc_param.ecc_status = status;
		ecc_cause = RC_PPS2_INT_ECC_ERROR;
		ecc_status = status;
	}

	status = readreg(RC_MREGS_STA_UP_ARQ_ECC_ERROR);
	if (status) {
		printk(KERN_ERR "rc_sta_up_arq_ecc_error: %x\n", status);
		writereg(RC_MREGS_STA_UP_ARQ_ECC_ERROR, status);
		rc_param.ecc_cause = RC_STA_UP_ARQ_ECC_ERROR;
		rc_param.ecc_status = status;
		ecc_cause = RC_STA_UP_ARQ_ECC_ERROR;
		ecc_status = status;
	}
	status = readreg(RC_MREGS_STA_UP_ACL_ECC_ERROR);
	if (status) {
		printk(KERN_ERR "rc_sta_up_acl_ecc_error: %x\n", status);
		writereg(RC_MREGS_STA_UP_ACL_ECC_ERROR, status);
		rc_param.ecc_cause = RC_STA_UP_ACL_ECC_ERROR;
		rc_param.ecc_status = status;
		ecc_cause = RC_STA_UP_ACL_ECC_ERROR;
		ecc_status = status;
	}

	status = readreg(RC_MREGS_STA_UP_VNIC_ECC_ERROR);
	if (status) {
		printk(KERN_ERR "rc_sta_up_vnic_ecc_error: %x\n", status);
		writereg(RC_MREGS_STA_UP_VNIC_ECC_ERROR, status);
		rc_param.ecc_cause = RC_STA_UP_VNIC_ECC_ERROR;
		rc_param.ecc_status = status;
		ecc_cause = RC_STA_UP_VNIC_ECC_ERROR;
		ecc_status = status;
	}
	atomic_notifier_call_chain(&rc_notify_list, 0, &rc_param);
	rc_evt_ecc(ecc_cause, ecc_status);
}

int rc_err_monitor(void *data)
{
	set_user_nice(current, -20);
	set_current_state(TASK_INTERRUPTIBLE);

	while (!kthread_should_stop()) {
		rc_pps_monitor();
		rc_mac_monitor();
		rc_ecc_monitor();
		schedule_timeout(5 * HZ);
		set_current_state(TASK_INTERRUPTIBLE);
	}
	__set_current_state(TASK_RUNNING);
	return 0;
}

void setup_int_handler(void)
{
	int ind_irq, r;

	// Temporarily disable the mac interrupts as no action will be taken.
	writereg(RC_MREGS_PPS0_INT_MAC_MSK, 0);
	writereg(RC_MREGS_PPS2_INT_MAC_MSK, 0);

	ind_irq =
		palo_int_to_irq(RC_INDIRECT_INT_LVL, RC_INDIRECT_INT_SRC);
	r = request_irq(ind_irq, rc_intr_handler, IRQF_DISABLED,
			"rc-ind", NULL);
	if (r) {
		printk(KERN_ERR "Unable to request RC ind irq: %d: %x\n",
				ind_irq, r);
		return;
	}
}

void release_rcindirect_table(int index)
{
	int vnic_cfg_ram[3] = { 0 };

	writeregs(RC_MREGS_DHS_UP_VNIC_CFG + (16 * index),
                &vnic_cfg_ram[0], VNIC_CFG_LENGTH);
	kfree(rc_vnic[index].ind_base);
}

int register_rc_event_notifier(struct notifier_block *nb)
{
	return atomic_notifier_chain_register(&rc_notify_list, nb);
}
