/*
 * Copyright 2012 Cisco Systems, Inc.  All rights reserved.
 */
#ident "$Id: pci-rc.c 43 2016-12-03 22:59:19Z buildsa $"
#include <linux/init.h>
#include <linux/kernel.h>
#include <linux/pci.h>
#include <linux/kthread.h>
#include <linux/delay.h>
#include <linux/module.h>
#include <asm/mach-palo/rc.h>

#include <palo/include/misc_mregs.h>
#include <palo/include/pcie_mregs.h>
#include <palo/include/cruz/rc_mregs.h>
#include <palo/include/cruz/rc_regs_defs.h>
#include <palo/include/cruz/proc_mregs.h>
#include <palo/fw/mcpu/include/palo_conf.h>
#include <palo/fw/mcpu/include/board_conf.h>

#include "rc.h"



#define VALID_CFG 0x1
#define PCIE_TIMEOUT 64
#define TLP_TYPE_1  1
#define CFG_HDR_1   5
#define TLP_TYPE_0  0
#define CFG_HDR_0   4

#define RESP_READY      1
#define RESP_CRS        (1 << 3)
#define RESP_UR         (1 << 2)
#define RESP_CA         (1 << 4)
#define RESP_FAILED    (7 << 2)
#define RESP_TIMEOUT    (1 << 1)
#define RESP_POISONED   (1 << 5)
#define CRS_DATA        0xffff0001


static spinlock_t rc_lock;
static struct resource rc_mem_resource = {
	.start	= PORT_0_MEM_START,
	.end	= PORT_0_MEM_END,
	.name	= "RC PCI MEM",
	.flags	= IORESOURCE_MEM
};

static struct resource rc_io_resource = {
	.start	= PORT_0_IO_START,
	.end	= PORT_0_IO_END,
	.name	= "RC IO MEM",
	.flags	= IORESOURCE_IO
};

static struct resource rc_mem_resource2 = {
	.start	= PORT_2_MEM_START,
	.end	= PORT_2_MEM_END,
	.name	= "RC PCI MEM",
	.flags	= IORESOURCE_MEM
};

static struct resource rc_io_resource2 = {
	.start	= PORT_2_IO_START,
	.end	= PORT_2_IO_END,
	.name	= "RC IO MEM",
	.flags	= IORESOURCE_IO,
};


static int rc_read_config2(struct pci_bus *bus, unsigned int devfn, int reg,
	int size, u32 *val);
static int rc_read_config(struct pci_bus *bus, unsigned int devfn, int reg,
	int size, u32 *val);

static int rc_write_config2(struct pci_bus *bus, unsigned int devfn, int reg,
	int size, u32 val);
static int rc_write_config(struct pci_bus *bus, unsigned int devfn, int reg,
	int size, u32 val);

static int rc_busno(struct pci_controller *hose);

struct pci_ops rc_pci_ops2 = {
	rc_read_config2,
	rc_write_config2,
};

struct pci_ops rc_pci_ops = {
	rc_read_config,
	rc_write_config,
};

static int root_up_bdf[2] = {
	RC_MREGS_CFG_UP_RC0_ROOT_BDF,
	RC_MREGS_CFG_UP_RC1_ROOT_BDF
};

static int root_dn_bdf[2] = {
	RC_MREGS_PPS0_CFG_DN_ROOT_BDF,
	RC_MREGS_PPS2_CFG_DN_ROOT_BDF
};

static int rc_pipe_reg[2] = {
	RC_MREGS_PCSD_CFG_0_PIPE,
	RC_MREGS_PCSD_CFG_2_PIPE
};

static int rc_serdes_reg[2] = {
	RC_MREGS_PCSD_CFG_0_SER_RESET,
	RC_MREGS_PCSD_CFG_2_SER_RESET
};

static int rc_root_cfg[2] = {
	RC_MREGS_PPS0_DHS_RC_CFG,
	RC_MREGS_PPS2_DHS_RC_CFG
};

static struct pci_controller rc_controller2 = {
	.pci_ops = &rc_pci_ops2,
	.mem_resource = &rc_mem_resource2,
	.mem_offset = 0x0UL,
	.io_resource = &rc_io_resource2,
	.get_busno = rc_busno,
	.port_sel = 1,
	.primary = 0,
	.io_offset = 0x0UL
};

static struct pci_controller rc_controller = {
	.pci_ops = &rc_pci_ops,
	.mem_resource = &rc_mem_resource,
	.mem_offset = 0x0UL,
	.io_resource = &rc_io_resource,
	.get_busno = rc_busno,
	.port_sel = 0,
	.primary = 0,
	.io_offset = 0x0UL
};

static int rc_cfg_read(int offset, int size, u32 *val, unsigned char port)
{
	int reg = port ? RC_MREGS_PPS2_DHS_RC_CFG : RC_MREGS_PPS0_DHS_RC_CFG;

	*val = readreg(reg + (offset) - (offset % 4));
	switch (size) {
		case 1:
			*val = (*val >> ((offset % 4) * 8))  & 0xff;
			break;
		case 2:
			*val = (*val >> ((offset % 4) * 8))  & 0xffff;
			break;
	}
	return PCIBIOS_SUCCESSFUL;
}

static int rc_cfg_write(int offset, int size, u32 val, unsigned char port)
{
	int data, mult = ~0;
	int reg = port ? RC_MREGS_PPS2_DHS_RC_CFG : RC_MREGS_PPS0_DHS_RC_CFG;

	data = readreg(reg + (offset) - (offset % 4));
	switch (size) {
		case 1:
			mult = ~(0xFF << ((offset % 4) * 8));
			break;
		case 2:
			mult = ~(0xFFFF << ((offset % 4) * 8));
			break;
	}
	data &= mult;
	val = (val << ((offset % 4) * 8));
	data |= val;
	writereg(reg + (offset) - (offset % 4), data);

	return PCIBIOS_SUCCESSFUL;
}

static int rc_read_config2(struct pci_bus *bus, unsigned int devfn, int reg,
		int size, u32 *val)
{
	struct pci_controller *hose = (struct pci_controller *)bus->sysdata;
	int dev, busno, func;
	int bdf;
	int actual_reg;
	int data = 0;
	int data_offset = 0;
	int local_offset = 0;
	int delay = 10;
	int type_req = TLP_TYPE_1;
	int hdr = CFG_HDR_1; // type 0
	int data_reg[] = {
		RC_MREGS_PPS2_STA_DN_RSP_DATA0,
		RC_MREGS_PPS2_STA_DN_RSP_DATA1,
		RC_MREGS_PPS2_STA_DN_RSP_DATA2,
		RC_MREGS_PPS2_STA_DN_RSP_DATA3,
	};

	busno = bus->number;
	dev = PCI_SLOT(devfn);
	func = PCI_FUNC(devfn);

	bdf = make_bdf(busno, dev, func);
	if (bdf == make_bdf(hose->primary, 0, 0))
		return rc_cfg_read(reg, size, val, 2);

	if (busno == hose->primary && dev)
		return PCIBIOS_FUNC_NOT_SUPPORTED;

	if (busno == (hose->primary + 1)) {
		type_req = TLP_TYPE_0;
		hdr = CFG_HDR_0;
	}

	actual_reg = ((type_req << 29) | (bdf << 13) | (reg << 1) | VALID_CFG);
	data_offset = (reg % 16) / 4;
	local_offset = reg % 4;

	writereg(RC_MREGS_PPS2_CFG_DN_REQ_SIZE, size);
	writereg(RC_MREGS_PPS2_CFG_DN_REQ_HDR, hdr << 23);
	writereg(RC_MREGS_PPS2_CFG_DN_CONFIG_REQ, actual_reg);

up:
	data = readreg(RC_MREGS_PPS2_STA_DN_RSP);
	if (data & RESP_READY) {
		if ((data & RESP_TIMEOUT) || (data & RESP_POISONED)) {
			return PCIBIOS_DEVICE_NOT_FOUND;
		} else if (data & RESP_CRS) {
			*val = CRS_DATA;
			return PCIBIOS_SUCCESSFUL;
		} else if ((data & RESP_UR) || (data & RESP_CA)) {
			return PCIBIOS_FUNC_NOT_SUPPORTED;
		} else {
			*val = readreg(data_reg[data_offset]);
			switch (size) {
				case 1:
					*val =
					((*val) >> (local_offset * 8)) & 0xff;
					break;
				case 2:
					*val =
					((*val) >> (local_offset * 8)) & 0xffff;
					break;
			}
		}
	} else {
		if (delay > PCIE_TIMEOUT * 1000)
			return PCIBIOS_DEVICE_NOT_FOUND;
        udelay(delay);
		delay *= 2;
		goto up;
	}
	return PCIBIOS_SUCCESSFUL;
}

static int rc_write_config2(struct pci_bus *bus, unsigned int devfn, int reg,
	int size, u32 val)
{
	struct pci_controller *hose = (struct pci_controller *)bus->sysdata;
	int dev, busno, func;
	int delay = 1;
	int bdf, actual_reg, type_req = TLP_TYPE_1;
	int data_offset, local_offset, data;
	int hdr = CFG_HDR_1;
	int data_reg[] = {
		RC_MREGS_PPS2_CFG_DN_REQ_DATA0,
		RC_MREGS_PPS2_CFG_DN_REQ_DATA1,
		RC_MREGS_PPS2_CFG_DN_REQ_DATA2,
		RC_MREGS_PPS2_CFG_DN_REQ_DATA3,
	};

	busno = bus->number;
	dev = PCI_SLOT(devfn);
	func = PCI_FUNC(devfn);

	bdf = make_bdf(busno, dev, func);
	if (bdf == make_bdf(hose->primary, 0, 0))
		return rc_cfg_write(reg, size, val, 2);

	if (busno == hose->primary && dev)
		return PCIBIOS_DEVICE_NOT_FOUND;

	if (busno == (hose->primary + 1)) {
		type_req = TLP_TYPE_0;
		hdr = CFG_HDR_0;
	}

	data_offset = (reg % 16) / 4;
	local_offset = reg % 4;
	actual_reg = ((type_req << 29) | (bdf << 13) | (reg << 1) | VALID_CFG);

	switch (size) {
		case 1:
			val = (val << (local_offset * 8));
			break;
		case 2:
			val = (val << (local_offset * 8));
		break;
	}

	writereg(data_reg[data_offset], val);
	writereg(RC_MREGS_PPS2_CFG_DN_REQ_SIZE, size);
	writereg(RC_MREGS_PPS2_CFG_DN_REQ_HDR, (hdr << 23) | 4);
	writereg(RC_MREGS_PPS2_CFG_DN_CONFIG_REQ, actual_reg);

up:
	data = readreg(RC_MREGS_PPS2_STA_DN_RSP);
	if (data & RESP_READY) {
		return PCIBIOS_SUCCESSFUL;
	} else {
		if (delay > PCIE_TIMEOUT * 1000)
			return PCIBIOS_DEVICE_NOT_FOUND;
		udelay(delay);
		delay *= 2;
		goto up;
	}
}


static int rc_read_config(struct pci_bus *bus, unsigned int devfn, int reg,
		int size, u32 *val)
{
	struct pci_controller *hose = (struct pci_controller *)bus->sysdata;
	int dev, busno, func;
	int bdf;
	int actual_reg;
	int data = 0;
	int data_offset = 0;
	int local_offset = 0;
	int delay = 1;
	int type_req = TLP_TYPE_1;
	int hdr = CFG_HDR_1;
	int data_reg[] = {
		RC_MREGS_PPS0_STA_DN_RSP_DATA0,
		RC_MREGS_PPS0_STA_DN_RSP_DATA1,
		RC_MREGS_PPS0_STA_DN_RSP_DATA2,
		RC_MREGS_PPS0_STA_DN_RSP_DATA3,
	};

	busno = bus->number;
	dev = PCI_SLOT(devfn);
	func = PCI_FUNC(devfn);

	bdf = make_bdf(busno, dev, func);
	if (bdf == make_bdf(hose->primary, 0, 0))
		return rc_cfg_read(reg, size, val, 0);

	if (busno == hose->primary && dev)
		return PCIBIOS_FUNC_NOT_SUPPORTED;

	if (busno == (hose->primary + 1)) {
		type_req = TLP_TYPE_0;
		hdr = CFG_HDR_0;
	}

	actual_reg = ((type_req << 29) | (bdf << 13) | (reg << 1) | VALID_CFG);
	data_offset = (reg % 16) / 4;
	local_offset = reg % 4;

	writereg(RC_MREGS_PPS0_CFG_DN_REQ_SIZE, size);
	writereg(RC_MREGS_PPS0_CFG_DN_REQ_HDR, hdr << 23);
	writereg(RC_MREGS_PPS0_CFG_DN_CONFIG_REQ, actual_reg);

up:
	data = readreg(RC_MREGS_PPS0_STA_DN_RSP);
	if (data & RESP_READY) {
		if ((data & RESP_TIMEOUT) || (data & RESP_POISONED)) {
			return PCIBIOS_DEVICE_NOT_FOUND;
		} else if (data & RESP_CRS) {
			*val = CRS_DATA;
			return PCIBIOS_SUCCESSFUL;
		} else if ((data & RESP_UR) || (data & RESP_CA)) {
			return PCIBIOS_FUNC_NOT_SUPPORTED;
		} else {
			*val = readreg(data_reg[data_offset]);
			switch (size) {
				case 1:
					*val =
					((*val) >> (local_offset * 8)) & 0xff;
					break;
				case 2:
					*val =
					((*val) >> (local_offset * 8)) & 0xffff;
					break;
			}
		}
	} else {
		if (delay > PCIE_TIMEOUT * 1000)
			return PCIBIOS_DEVICE_NOT_FOUND;
		udelay(delay);
		delay *= 2;
		goto up;
	}
	return PCIBIOS_SUCCESSFUL;
}

static int rc_write_config(struct pci_bus *bus, unsigned int devfn, int reg,
	int size, u32 val)
{
	struct pci_controller *hose = (struct pci_controller *)bus->sysdata;
	int dev, busno, func;
	int delay = 1;
	int bdf, actual_reg, type_req = TLP_TYPE_1;
	int data_offset, local_offset, data;
	int hdr = CFG_HDR_1; // type 1
	int data_reg[] = {
		RC_MREGS_PPS0_CFG_DN_REQ_DATA0,
		RC_MREGS_PPS0_CFG_DN_REQ_DATA1,
		RC_MREGS_PPS0_CFG_DN_REQ_DATA2,
		RC_MREGS_PPS0_CFG_DN_REQ_DATA3,
	};

	busno = bus->number;
	dev = PCI_SLOT(devfn);
	func = PCI_FUNC(devfn);

	bdf = make_bdf(busno, dev, func);
	if (bdf == make_bdf(hose->primary, 0, 0))
		return rc_cfg_write(reg, size, val, 0);

	if (busno == hose->primary && dev)
		return PCIBIOS_DEVICE_NOT_FOUND;

	if (busno == (hose->primary + 1)) {
		type_req = TLP_TYPE_0;
		hdr = CFG_HDR_0;
	}

	data_offset = (reg % 16) / 4;
	local_offset = reg % 4;
	actual_reg = ((type_req << 29) | (bdf << 13) | (reg << 1) | VALID_CFG);

	switch (size) {
		case 1:
			val = (val << (local_offset * 8));
			break;
		case 2:
			val = (val << (local_offset * 8));
			break;
	}

	writereg(data_reg[data_offset], val);
	writereg(RC_MREGS_PPS0_CFG_DN_REQ_SIZE, size);
	writereg(RC_MREGS_PPS0_CFG_DN_REQ_HDR, (hdr << 23) | 4);
	writereg(RC_MREGS_PPS0_CFG_DN_CONFIG_REQ, actual_reg);

up:
	data = readreg(RC_MREGS_PPS0_STA_DN_RSP);
	if (data & RESP_READY) {
		return PCIBIOS_SUCCESSFUL;
	} else {
		if (delay > PCIE_TIMEOUT * 1000)
			return PCIBIOS_DEVICE_NOT_FOUND;
		udelay(delay);
		delay *= 2;
		goto up;
	}
};

static int rc_busno(struct pci_controller *hose)
{
	if (hose->prev) {
		struct pci_bus *bus;

		bus = hose->prev->bus;
		printk(KERN_DEBUG "primary: %x, secondary: %x, Subordinate: %x\n",
                    bus->number, bus->secondary, bus->subordinate);

		hose->primary = bus->subordinate + 1;

		writereg(root_up_bdf[hose->port_sel],
				make_bdf(hose->primary, 0, 0));
		writereg(root_dn_bdf[hose->port_sel],
				make_bdf(hose->primary, 0, 0));
	} else {
		hose->primary = 0;
	}

	printk(KERN_DEBUG "returning busno: %d\n", hose->primary);
	return hose->primary;
}

static int rc_cfg_lnkwidth = 0; // Default in x4
static int rc_err_handler = 1;
static int rc_reset_delay = 1;
static int rc_pipe_unreset_mask = 0xFFFFFC1F;
static int rc_pipe_reset_mask = 0x3E0;
static int rc_serdes_unreset_mask = 0xFFFFFFFC;
static int rc_serdes_reset_mask = 0x3;
static int rc_eq_0x80c = 0x2086E;
static int rc_eq_0x8ac = 0x30;
static int rc_eq_0x8a8 = 0x20;
static int rc_eq_0x890 = 0x401;
int rc_pcie_hotplug_supported = 0;

static void rc_pipe_unreset(unsigned char port)
{
	int pipe_data = readreg(rc_pipe_reg[port]);

	pipe_data |= ((port * 2) << 6);
	pipe_data &= rc_pipe_unreset_mask;
	writereg(rc_pipe_reg[port], pipe_data);
}

static void rc_pipe_reset(unsigned char port)
{
	int pipe_data = readreg(rc_pipe_reg[port]);

	pipe_data |= ((port * 2) << 6);
	pipe_data |= rc_pipe_reset_mask;
	writereg(rc_pipe_reg[port], pipe_data);
}

static void rc_ser_unreset(unsigned char port)
{
	int serdes_data = readreg(rc_serdes_reg[port]);

	serdes_data |= ((port * 2) << 6);
	serdes_data &= rc_serdes_unreset_mask;
	writereg(rc_serdes_reg[port], serdes_data);
}

static void rc_ser_reset(unsigned char port)
{
	int serdes_data = readreg(rc_serdes_reg[port]);

	serdes_data |= ((port * 2) << 6);
	serdes_data |= rc_serdes_reset_mask;
	writereg(rc_serdes_reg[port], serdes_data);
}

static int __init rc_pll_cfg1(void)
{
	int polls;

	// RPLL lock sequence

	// val0:pll_en high,load high,lresetn=0
	writereg(MISC_MREGS_MIPL_CLK_RPLL, 0x03f92708);
	readreg (MISC_MREGS_MIPL_CLK_RPLL);
	// val1:restn=1, rest are the same
	writereg(MISC_MREGS_MIPL_CLK_RPLL, 0x07f92708);
	readreg (MISC_MREGS_MIPL_CLK_RPLL);
	// val2:lrestn=0
	writereg(MISC_MREGS_MIPL_CLK_RPLL, 0x03f92708);
	readreg (MISC_MREGS_MIPL_CLK_RPLL);
	// val3:lrestn=1
	writereg(MISC_MREGS_MIPL_CLK_RPLL, 0x07f92708);
	readreg (MISC_MREGS_MIPL_CLK_RPLL);
	// same as val3
	writereg(MISC_MREGS_MIPL_CLK_RPLL, 0x07f92708);
	readreg (MISC_MREGS_MIPL_CLK_RPLL);
	// same as above to give enough cycles for load to be stable.
	writereg(MISC_MREGS_MIPL_CLK_RPLL, 0x07f92708);
	readreg (MISC_MREGS_MIPL_CLK_RPLL);

	// now poll pll lock
	for (polls = 0; polls < 10; polls++) {
		if (readreg(MISC_MREGS_PIRC_PLL_500_LOCK)) {
			return polls;
		}
		udelay(1000);
	}
	return -1;
}

static int __init rc_pll_cfg(void)
{
	int i, polls;

	for (i = 0; i < 10; i++) {
		if ((polls = rc_pll_cfg1()) >= 0) {
			printk(KERN_INFO PFX "rc_pll_cfg locked %d:%d\n",
			       i, polls);
			return 0;
		}
	}
	printk(KERN_ERR PFX "rc_pll_cfg failed to lock\n");
	return -1;
}

static void __init rc_asic_setup(void)
{
	int ports = 2;
	unsigned char i;

	rc_pll_cfg();

	// Serdes PLL lock for port 0
	writereg(RC_MREGS_PCSD_CFG_0_SER_PLL_LO, 0x84485926);
	writereg(RC_MREGS_PCSD_CFG_0_SER_PLL_HI, 0x00300020);

	// Serdes PLL lock for port 2
	writereg(RC_MREGS_PCSD_CFG_2_SER_PLL_LO, 0x84485926);
	writereg(RC_MREGS_PCSD_CFG_2_SER_PLL_HI, 0x00300020);

	for (i = 0; i < ports; i++) {
		// serdes reset sequence for each port
		rc_ser_unreset(i);
		rc_ser_reset(i);
		rc_ser_unreset(i);
	}
}

typedef struct oventry {
	u_int32_t off;
	u_int32_t val;
} oventry_t;

#define MAXOV 32
typedef struct ovinfo {
	u_int32_t nov;
	oventry_t ov[MAXOV];
} ovinfo_t;

#define MAXPPS 2
typedef struct ovgroup {
	ovinfo_t gbl;
	ovinfo_t pps[MAXPPS];
} ovgroup_t;

static ovgroup_t rc_serovg;
static ovgroup_t rc_pcsovg;
static ovgroup_t rc_cfgovg;

static void
ovinfo_init(ovinfo_t *ovinfo, char *str)
{
	int n;
	char *off, *val;

	n = 0;
	off = str;
	while (off && n < MAXOV) {
		val = strchr(off, '=');
		if (val) {
			*val++ = '\0'; // squash '=' to terminate "off"
			ovinfo->ov[n].off = simple_strtoul(off, NULL, 0);
			ovinfo->ov[n].val = simple_strtoul(val, NULL, 0);
			n++;

			// find next off=val
			val = strchr(val, ',');
			if (val) val++;
		}
		off = val;
	}
	ovinfo->nov = n;
}

static void
ovgroup_init(ovgroup_t *ovg, char *str)
{
	extern char *prom_getenv(const char *s);
	char *val, buf[80];
	int pps;

	// Get global settings
	val = prom_getenv(str);
	if (val) {
		printk("%s override = %s\n", str, val);
		ovinfo_init(&ovg->gbl, val);
	}
	for (pps = 0; pps < MAXPPS; pps++) {
		snprintf(buf, sizeof (buf), "%s_%d", str, pps);
		val = prom_getenv(buf);
		if (val) {
			printk("%s override = %s\n", buf, val);
			ovinfo_init(&ovg->pps[pps], val);
		}
	}
}

static void
ovinfo_write(ovinfo_t *ovinfo, u_int32_t base)
{
	int i;

	for (i = 0; i < ovinfo->nov; i++) {
		oventry_t *ov = &ovinfo->ov[i];
		writereg(base + ov->off, ov->val);
	}
}

static void __devinit rc_check_env(void)
{
	extern char *prom_getenv(const char *s);
	char *val;

	val = prom_getenv("rc_eq_0x80c");
	if (val) {
		rc_eq_0x80c = simple_strtoul(val, NULL, 0);
		printk("rc_eq_0x80c override: 0x%x\n", rc_eq_0x80c);
	}

	val = prom_getenv("rc_eq_0x8ac");
	if (val) {
		rc_eq_0x8ac = simple_strtoul(val, NULL, 0);
		printk("rc_eq_0x8ac override: 0x%x\n", rc_eq_0x8ac);
	}

	val = prom_getenv("rc_eq_0x8a8");
	if (val) {
		rc_eq_0x8a8 = simple_strtoul(val, NULL, 0);
		printk("rc_eq_0x8a8 override: 0x%x\n", rc_eq_0x8a8);
	}

	val = prom_getenv("rc_eq_0x890");
	if (val) {
		rc_eq_0x890 = simple_strtoul(val, NULL, 0);
		printk("rc_eq_0x890 override: 0x%x\n", rc_eq_0x890);
	}

	val = prom_getenv("rc_cfg_lnkwidth");
	if (val) {
		rc_cfg_lnkwidth = simple_strtoul(val, NULL, 0);
		printk("rc_cfg_lnkwidth override: 0x%x\n", rc_cfg_lnkwidth);
	}

	val = prom_getenv("rc_err_handler");
	if (val) {
		rc_err_handler = simple_strtoul(val, NULL, 0);
		printk("rc_err_handler override: 0x%x\n", rc_err_handler);
	}

	val = prom_getenv("rc_reset_delay");
	if (val) {
		rc_reset_delay = simple_strtoul(val, NULL, 0);
		printk("rc_reset_delay override: 0x%x\n", rc_reset_delay);
	}

	val = prom_getenv("rc_pipe_unreset_mask");
	if (val) {
		rc_pipe_unreset_mask = simple_strtoul(val, NULL, 0);
		printk("rc_pipe_unreset_mask override: 0x%x\n",
				rc_pipe_unreset_mask);
	}

	val = prom_getenv("rc_pipe_reset_mask");
	if (val) {
		rc_pipe_reset_mask = simple_strtoul(val, NULL, 0);
		printk("rc_pipe_reset_mask override: 0x%x\n",
				rc_pipe_reset_mask);
	}

	val = prom_getenv("rc_serdes_unreset_mask");
	if (val) {
		rc_serdes_unreset_mask = simple_strtoul(val, NULL, 0);
		printk("rc_serdes_unreset_mask override: 0x%x\n",
				rc_serdes_unreset_mask);
	}

	val = prom_getenv("rc_serdes_reset_mask");
	if (val) {
		rc_serdes_reset_mask = simple_strtoul(val, NULL, 0);
		printk("rc_serdes_reset_mask override: 0x%x\n",
				rc_serdes_reset_mask);
	}

	val = prom_getenv("rc_pcie_hotplug_supported");
	if (val) {
		rc_pcie_hotplug_supported = simple_strtoul(val, NULL, 0);
		printk("rc_pcie_hotplug_supported override: 0x%x\n",
				rc_pcie_hotplug_supported);
	}

	ovgroup_init(&rc_pcsovg, "rc_pcs");
	ovgroup_init(&rc_serovg, "rc_ser");
	ovgroup_init(&rc_cfgovg, "rc_cfg");
}

static void rc_pcs_init(void)
{
	ovinfo_write(&rc_pcsovg.gbl, RC_MREGS_DHS_PIPE_0);
	ovinfo_write(&rc_pcsovg.pps[0], RC_MREGS_DHS_PIPE_0);
	ovinfo_write(&rc_pcsovg.gbl, RC_MREGS_DHS_PIPE_2);
	ovinfo_write(&rc_pcsovg.pps[1], RC_MREGS_DHS_PIPE_2);
}

static void rc_ser_init(void)
{
	ovinfo_write(&rc_serovg.gbl, RC_MREGS_DHS_SER_0);
	ovinfo_write(&rc_serovg.pps[0], RC_MREGS_DHS_SER_0);
	ovinfo_write(&rc_serovg.gbl, RC_MREGS_DHS_SER_2);
	ovinfo_write(&rc_serovg.pps[1], RC_MREGS_DHS_SER_2);
}

static void rc_cfg_init(void)
{
	ovinfo_write(&rc_cfgovg.gbl, RC_MREGS_PPS0_DHS_RC_CFG);
	ovinfo_write(&rc_cfgovg.pps[0], RC_MREGS_PPS0_DHS_RC_CFG);
	ovinfo_write(&rc_cfgovg.gbl, RC_MREGS_PPS2_DHS_RC_CFG);
	ovinfo_write(&rc_cfgovg.pps[1], RC_MREGS_PPS2_DHS_RC_CFG);
}

static void gen3_negotiation(unsigned char port)
{
    // Equalization register programming.
	writereg(rc_root_cfg[port] + 0x80C, rc_eq_0x80c);
	writereg(rc_root_cfg[port] + 0x8AC, rc_eq_0x8ac);
	writereg(rc_root_cfg[port] + 0x8A8, rc_eq_0x8a8);
	writereg(rc_root_cfg[port] + 0x890, rc_eq_0x890);
	// New preset values compatible for PLX switch to avoid bit errors
	writereg(rc_root_cfg[port] + 0x154, 0x75757575);
	writereg(rc_root_cfg[port] + 0x158, 0x75757575);
	writereg(rc_root_cfg[port] + 0x15c, 0x75757575);
	writereg(rc_root_cfg[port] + 0x160, 0x75757575);
}

static void clear_ecc_sram(void)
{
	char *acl_ram = (char *)CKSEG1ADDR(RC_MREGS_DHS_UP_ACL_RAM);
	char *vnic_ram = (char *)CKSEG1ADDR(RC_MREGS_DHS_UP_VNIC_CFG);

	memset(acl_ram, 0, RC_MREGS_DHS_UP_ACL_RAM_MAXCNT *
            RC_MREGS_DHS_UP_ACL_RAM_BLKSZ);
	memset(vnic_ram, 0, RC_MREGS_DHS_UP_VNIC_CFG_MAXCNT *
            RC_MREGS_DHS_UP_VNIC_CFG_BLKSZ);
}

static int __init rc_pci_setup(void)
{
	int i = 0, ports = 2;
	struct task_struct *tsk;
	int dn_mode = 0;
	extern char *prom_getenv(const char *s);
	char *val = NULL;
	int rc_noscan = 0;

	val = prom_getenv("rc_noscan");
	if (val) {
		rc_noscan = simple_strtoul(val, NULL, 0);
		printk("rc_noscan: %d\n", rc_noscan);
	}

	// Cruz RC only available on montclair, calimesab and cruzita
	if (!board_conf_has_rc() || rc_noscan)
		return 0;

	if (palo_conf_is_asic()) {
		rc_check_env();
		writereg(RC_MREGS_CFG_LINKWIDTH, rc_cfg_lnkwidth);
		rc_asic_setup();
        for (i = 0; i < 2; i++) {
		    rc_pipe_unreset(i);
		    rc_pipe_reset(i);
		    rc_pipe_unreset(i);
        }
		writereg(RC_MREGS_CFG_PORT_PERST_OUT, 3);
		writereg(RC_MREGS_SW_HARD_RST, 0);
		writereg(RC_MREGS_SW_SOFT_RST, 0);
		msleep(rc_reset_delay);
		for (i = 0; i < ports; i++) {
			gen3_negotiation(i);
			// Disable ASPM for now
			writereg(rc_root_cfg[i] + 0x7C, 0x413083);
		}
		rc_pcs_init();
		rc_ser_init();
		rc_cfg_init();
    } else {
		writereg(RC_MREGS_CFG_LINKWIDTH, ((1 << 2) | 1));
		writereg(RC_MREGS_CFG_PORT_PERST_OUT, 3);
		writereg(RC_MREGS_SW_HARD_RST, 0);
		writereg(RC_MREGS_SW_SOFT_RST, 0);
		msleep(5);
	}

	dn_mode = readreg(RC_MREGS_(CFG_DN_MAIN_MODE, 0));
	writereg(RC_MREGS_(CFG_DN_MAIN_MODE, 0), dn_mode | (1023 << 15));
	dn_mode = readreg(RC_MREGS_(CFG_DN_MAIN_MODE, 2));
	writereg(RC_MREGS_(CFG_DN_MAIN_MODE, 2), dn_mode | (1023 << 15));

	printk(KERN_DEBUG "EQ status register: %x\n",
			readreg(rc_root_cfg[0] + 0xA0));

	writereg(RC_MREGS_PPS0_WRO_BARRIER_OPEN, 1);
	writereg(RC_MREGS_PPS0_CFG_MAC0, 0x00020181);

	writereg(RC_MREGS_PPS2_WRO_BARRIER_OPEN, 1);
	writereg(RC_MREGS_PPS2_CFG_MAC0, 0x00020181);

	register_pci_controller(&rc_controller);
	// Register second controller only if linkwidth is x4
	if (readreg(RC_MREGS_CFG_LINKWIDTH) & 1)
		register_pci_controller(&rc_controller2);

	writereg(RC_MREGS_CFG_UP_VNIC_NUMBER_BASE, 0); // 1920
	writereg(RC_MREGS_PPS0_CFG_UP_MODE, 0); // set Posted_wr_ack_en to 0.
	writereg(RC_MREGS_CFG_UP_MAX_RD_PENDING, 64);

	if (rc_err_handler) {
		tsk = kthread_create(rc_err_monitor, NULL, "rc_err_monitor");
		if (!IS_ERR(tsk)) {
			printk(KERN_INFO "RC error monitoring thread created\n");
			kthread_bind(tsk, 0);
			wake_up_process(tsk);
		}
	}
	clear_ecc_sram();
	setup_int_handler();
	spin_lock_init(&rc_lock);
	return 0;
}
arch_initcall(rc_pci_setup);

static inline u32
rc_lbdf2vnic_tcam_x_addr(const u16 index)
{
	return (u32)RC_MREGS_DHS_UP_LBDF2VNIC_CAM_ADDR_COMPUTE(
            RC_MREGS_BASE_ADDR, index);
}

static inline u32
rc_lbdf2vnic_ram_addr(const u16 index)
{
	return (u32)RC_MREGS_DHS_UP_LBDF2VNIC_RAM_ADDR_COMPUTE(
            RC_MREGS_BASE_ADDR, index);
}

static void set_dns_tbl(int index)
{
#define VALID (1 << 18)
	struct pci_dev *pdev = rc_vnic[index].pdev;
	struct pci_controller *hose =
		(struct pci_controller *)pdev->bus->sysdata;
	int bdf = rc_vnic[index].bdf;
	int busno = bdf_to_bus(bdf);
	int cfg_hdr = (busno > (hose->bus->number + 1)) ? 1 : 0;
	int bdf_table_addr = RC_MREGS_DHS_DN_RC_BDF + (4 * index);
	int reg = (cfg_hdr << 17) |
		(hose->port_sel << 16) | VALID | (bdf & 0xffff);

	writereg(bdf_table_addr, reg);
}

int get_dns_tbl(int index)
{
	int bdf_table_addr = RC_MREGS_DHS_DN_RC_BDF + (4 * index);

	return readreg(bdf_table_addr);
}


static void release_dns_table(int index)
{
	int bdf_table_addr = RC_MREGS_DHS_DN_RC_BDF + (4 * index);

	writereg(bdf_table_addr, 0);
}

static void set_lbdf2vnic_cam(int index)
{
	struct pci_dev *pdev = rc_vnic[index].pdev;
	struct pci_controller *hose =
		(struct pci_controller *)pdev->bus->sysdata;
	int port_no = hose->port_sel << 16;
	int req_id = rc_vnic[index].bdf;
	int tcam_valid = (1 << 17);

	writereg(rc_lbdf2vnic_tcam_x_addr(index),
			port_no | req_id | tcam_valid);
}

int get_lbdf2vnic_cam(int index)
{
	return readreg(rc_lbdf2vnic_tcam_x_addr(index));
}

int get_lbdf2vnic_ram(int index)
{
	return readreg(rc_lbdf2vnic_ram_addr(index));
}

static void release_ups_table(int index)
{
	writereg(rc_lbdf2vnic_tcam_x_addr(index), 0);
	writereg(rc_lbdf2vnic_ram_addr(index), 0);
}

static void set_lbdf2vnic_ram(int index, char mem_space, char action,
		char addr_modifier, char hiwidth)
{
	int ram_valid = 1;
	int addr_space = (mem_space <<
            RC_MREGS_DHS_UP_LBDF2VNIC_RAM_ADDR_SPACE_SHIFT);
	int lbdf2vnic_action = (action <<
			RC_MREGS_DHS_UP_LBDF2VNIC_RAM_ACTION_SHIFT);
	int addr_mod = addr_modifier <<
		RC_MREGS_DHS_UP_LBDF2VNIC_RAM_ADDR_MOD_SHIFT;
	int host_width = hiwidth <<
		RC_MREGS_DHS_UP_LBDF2VNIC_RAM_HOST_INDEX_WIDTH_SHIFT;

	writereg(rc_lbdf2vnic_ram_addr(index),
			addr_space | ram_valid | lbdf2vnic_action
			| host_width | addr_mod);
}

int update_lbdf2vnic_ram(struct pci_dev *dev, char action,
		char addr_space, char addr_modifier, char hiwidth)
{
	int busno = dev->bus->number;
	int pdev = PCI_SLOT(dev->devfn);
	int func = PCI_FUNC(dev->devfn);
	int bdf = make_bdf(busno, pdev, func);
	int i, found = 0;

	for (i = 0; i < key_index; i++) {
		if (rc_vnic[i].bdf == bdf) {
			found = 1;
			break;
		}
	}
	if (found) {
		set_lbdf2vnic_cam(i);
		set_lbdf2vnic_ram(i, addr_space, action, addr_modifier,
				hiwidth);
		return 0;
	}
	return -1;
}
EXPORT_SYMBOL(update_lbdf2vnic_ram);

void cleanup_lbdf2vnic_ram(struct pci_dev *dev)
{
	int busno = dev->bus->number;
	int pdev = PCI_SLOT(dev->devfn);
	int func = PCI_FUNC(dev->devfn);
	int bdf = make_bdf(busno, pdev, func);
	int i, found = 0;

	for (i = 0; i < key_index; i++) {
		if (rc_vnic[i].bdf == bdf) {
			found = 1;
			break;
		}
	}
	if (found) {
		// Restore the entry to make the device accessible in
		// locally attached mode.
		update_lbdf2vnic_ram(dev, 0, 1, 0, 0);
	}
}
EXPORT_SYMBOL(cleanup_lbdf2vnic_ram);

void setup_rcvnic_reg(struct pci_dev *pdev, unsigned int intr_address,
        char mem_space, char action, unsigned int *ind_base)
{
	int busno = pdev->bus->number;
	int dev = PCI_SLOT(pdev->devfn);
	int func = PCI_FUNC(pdev->devfn);
	int bdf = make_bdf(busno, dev, func);
	struct pci_controller *hose = (struct
			pci_controller*)pdev->bus->sysdata;
	int i;

	for (i = 0; i < key_index; i++) {
		if (!rc_vnic[i].valid)
			break;
	}

	spin_lock(&rc_lock);
	rc_vnic[i].bdf = bdf;
	rc_vnic[i].index = key_index;
	rc_vnic[i].port = hose->port_sel;
	rc_vnic[i].pdev = pdev;

	set_dns_tbl(i);
	set_lbdf2vnic_cam(i);
	set_lbdf2vnic_ram(i, mem_space, action, 0, 0);
	setup_rcindirect_table(i, intr_address, ind_base);

	rc_vnic[i].valid = 1;
	if (i >= key_index)
		key_index++;
	spin_unlock(&rc_lock);
}

void release_rcvnic_reg(struct pci_dev *pdev)
{
	int busno = pdev->bus->number;
	int dev = PCI_SLOT(pdev->devfn);
	int func = PCI_FUNC(pdev->devfn);
	int bdf = make_bdf(busno, dev, func);
	int i;

	spin_lock(&rc_lock);
	for (i = 0; i < key_index; i++) {
		if (bdf == rc_vnic[i].bdf) {
			release_dns_table(i);
			release_ups_table(i);
			release_rcindirect_table(i);
			rc_vnic[i].valid = 0;
		}
	}
	spin_unlock(&rc_lock);
}

void retrain_port(unsigned char port)
{
	// Write link control 3 register with bit 0
	if (port > 1)
		return;
	rc_cfg_write(0x14C, 1, 1, port);
	// Write link control register with bit 5 set
	rc_cfg_write(0x80, 1, (1 << 5), port);
}

void rc_reset(void)
{
#define RC_PORT_0_ON 1
#define RC_PORT_2_ON (1 << 1)

	// Turn off the the port
	writereg(RC_MREGS_CFG_PORT_PERST_OUT, 0);
	msleep(10);
	// Turn on the port
	writereg(RC_MREGS_CFG_PORT_PERST_OUT, (RC_PORT_0_ON | RC_PORT_2_ON));
}
