/*
 * Copyright 2012 Cisco Systems, Inc.  All rights reserved.
 */
#ident "$Id: rc.h 43 2016-12-03 22:59:19Z buildsa $"
#ifndef __RC_H__
#define __RC_H__

#include <palo/include/pcie_mregs.h>
#include <palo/include/misc_regs.h>
#include <asm/mach-palo/rc.h>

#include "regio.h"


#define PORT_MEM_SIZE		(PALOMEM_RC_SIZE / 2)
#define PORT_IO_SIZE		0x8000

#define PORT_0_MEM_START	PALOMEM_RC_BASE
#define PORT_0_MEM_END		(PORT_0_MEM_START + PORT_MEM_SIZE - 1)
#define PORT_0_IO_START		0x0000
#define PORT_0_IO_END		(PORT_0_IO_START + PORT_IO_SIZE - 1)

#define PORT_2_MEM_START	(PORT_0_MEM_START + PORT_MEM_SIZE)
#define PORT_2_MEM_END		(PORT_2_MEM_START + PORT_MEM_SIZE - 1)
#define PORT_2_IO_START		(PORT_0_IO_START + PORT_IO_SIZE)
#define PORT_2_IO_END		(PORT_2_IO_START + PORT_IO_SIZE - 1)

#define MAC_RADM_RCVD_REQ_UR    (1 << 27)
#define MAC_RADM_MLF_TLP_ERR    (1 << 26)
#define MAC_RADM_ECRC_ERR       (1 << 25)
#define UP_RESPERR_CA_UR        (1 << 24)
#define UP_RESPERR_EP           (1 << 23)
#define MAC_PME_MSI             (1 << 22)
#define MAC_PME_INT             (1 << 21)
#define MAC_SYS_ERR_RC          (1 << 20)
#define MAC_AER_RC_ERR_MSI      (1 << 19)
#define MAC_AER_RC_ERR_INT      (1 << 18)
#define DN_CPL_EOT_ERR          (1 << 17)
#define DN_CPL_LOW_ADDR_ERR     (1 << 16)
#define DN_CPL_BYTE_CNT_ERR     (1 << 15)
#define DN_REQ_DRF_ERR_UNC      (1 << 14)
#define DN_REQ_CRF_ERR_UNC      (1 << 13)
#define DN_CPU_REQ_ERR          (1 << 12)
#define DN_RSP_DERR             (1 << 11)
#define DN_RSP_AERR             (1 << 10)
#define DN_CPL_TIMEOUT          (1 << 9)
#define DN_CPL_STAT_CRS_ERR     (1 << 8)
#define DN_CPL_STAT_UR_ERR      (1 << 7)
#define DN_CPL_STAT_CA_ERR      (1 << 6)
#define DN_CPL_ECRC_ERR         (1 << 5)
#define DN_CPL_TLP_POISONED     (1 << 4)
#define DN_CPL_TLP_ABORT        (1 << 3)
#define DN_CPL_DLLP_ABORT       (1 << 2)
#define DN_CPL_UNEXP            (1 << 1)
#define DN_REQ_UNS              (1)


#define DRV_NAME "rc"
#define DRV_DESCRIPTION "Driver for Cruz RC module"
#define DRV_VERSION "1.0"
#define DRV_COPYRIGHT "Copyright 2013 Cisco Systems, Inc"
#define PFX DRV_NAME ": "

#define RC_NUM_PORTS		2
#define RC_ERROR_STAT_COUNT	28
#define MAX_VNIC_COUNT		16
#define VNIC_CFG_LENGTH		3
#define BAR_COUNT			3

typedef struct {
	unsigned int dev_base;
	unsigned int remap_base;
	unsigned char window;
	unsigned char valid;
	unsigned int vdev_base;
} dev_mem_t;

typedef struct {
	unsigned char index;
	unsigned int bdf;
	unsigned int *ind_base;
	unsigned char valid;
	unsigned char cpl_id;
	unsigned char port;
	struct pci_dev *pdev;
	dev_mem_t mem_mapped[BAR_COUNT];
} rc_vnic_info_t;

extern int error_stats[RC_NUM_PORTS][RC_ERROR_STAT_COUNT];
extern rc_vnic_info_t rc_vnic[MAX_VNIC_COUNT];
extern int key_index;

static inline int make_bdf(const int bus, const int dev, const int fnc)
{
	return ((bus & 0xff) << 8) | ((dev & 0x1f) << 3) | (fnc & 0x7);
}

static inline int bdf_to_bus(const int bdf)
{
	return (bdf >> 8) & 0xff;
}
static inline int bdf_to_dev(const int bdf)
{
	return (bdf >> 3) & 0x1f;
}
static inline int bdf_to_fnc(const int bdf)
{
	return bdf & 0x7;
}
void setup_rcvnic_reg(struct pci_dev *pdev, unsigned int intr_address,
        char mem_space, char action, unsigned int *ind_base);
void setup_rcindirect_table(int index, unsigned int intr_address,
        unsigned int *ind_base);
void setup_int_handler(void);
void release_rcvnic_reg(struct pci_dev *pdev);
void release_rcindirect_table(int index);
void retrain_port(unsigned char port);
void rc_reset(void);

int get_root_aer_info(int port, char *buf);
int get_dev_aer_info(int index, char *buf);

int get_dns_tbl(int index);
int get_lbdf2vnic_cam(int index);
int get_lbdf2vnic_ram(int index);
void get_vnic_cfg_ram(int index, int *buf);
int rc_err_monitor(void *data);
void *map_cfg(struct pci_dev *pdev);

#endif
