/*
 * Copyright 2012 Cisco Systems, Inc.  All rights reserved.
 */
#ident "$Id: sysfs-rc.c 43 2016-12-03 22:59:19Z buildsa $"
#include <linux/device.h>
#include <linux/kernel.h>
#include <linux/module.h>
#include <linux/init.h>
#include <linux/fs.h>
#include <linux/platform_device.h>
#include <linux/pci.h>
#include <linux/poll.h>

#include "rc.h"
#include <asm/mach-palo/rc.h>
#include <palo/include/cruz/rc_mregs.h>

MODULE_DESCRIPTION(DRV_DESCRIPTION);
MODULE_AUTHOR("Sagar Borikar <sborikar@cisco.com>");
MODULE_LICENSE("GPL");
MODULE_VERSION(DRV_VERSION);

static int rc_major;

static struct {
	u16 flags;
	u16 bdf;
	u32 addr;
	u16 memspace;
	u16 bar;
	int ring_base;
} dvnic;

static char *err_string [] = {
	"DN_REQ_UNS",
	"DN_CPL_UNEXP",
	"DN_CPL_DLLP_ABORT",
	"DN_CPL_TLP_ABORT",
	"DN_CPL_TLP_POISONED",
	"DN_CPL_ECRC_ERR",
	"DN_CPL_STAT_CA_ERR",
	"DN_CPL_STAT_UR_ERR",
	"DN_CPL_STAT_CRS_ERR",
	"DN_CPL_TIMEOUT",
	"DN_RSP_AERR",
	"DN_RSP_DERR",
	"DN_CPU_REQ_ERR",
	"DN_REQ_CRF_ERR_UNC",
	"DN_REQ_DRF_ERR_UNC",
	"DN_CPL_BYTE_CNT_ERR",
	"DN_CPL_LOW_ADDR_ERR",
	"DN_CPL_EOT_ERR",
	"MAC_AER_RC_ERR_INT",
	"MAC_AER_RC_ERR_MSI",
	"MAC_SYS_ERR_RC",
	"MAC_PME_INT",
	"MAC_PME_MSI",
	"UP_RESPERR_EP",
	"UP_RESPERR_CA_UR",
	"MAC_RADM_ECRC_ERR",
	"MAC_RADM_MLF_TLP_ERR",
	"MAC_RADM_RCVD_REQ_UR",
};

static ssize_t rc_sysfs_port0_int_err_stats_show(
		struct device *dev,
		struct device_attribute *attr,
		char *buf)
{
	int i;
	char *buf_ret = buf;

	for (i = 0; i < RC_ERROR_STAT_COUNT; i++) {
		if (error_stats[0][i]) {
			buf_ret += sprintf(buf_ret, "%s : %d\n",
				err_string[i], error_stats[0][i]);
		}
	}
	return buf_ret - buf;
}

static ssize_t rc_sysfs_port2_int_err_stats_show(
		struct device *dev,
		struct device_attribute *attr,
		char *buf)
{
	int i;
	char *buf_ret = buf;

	for (i = 0; i < RC_ERROR_STAT_COUNT; i++) {
		if (error_stats[1][i]) {
			buf_ret += sprintf(buf_ret, "%s : %d\n",
				err_string[i], error_stats[1][i]);
		}
	}
	return buf_ret - buf;
}

static ssize_t rc_sysfs_lbdf2vnic_cam_show(
		struct device *dev,
		struct device_attribute *attr,
		char *buf)
{
	int i;
	char *buf_ret = buf;

	if (!buf_ret)
		return 0;

	for (i = 0; i < key_index; i++) {
		if (rc_vnic[i].valid) {
			buf_ret += sprintf(buf_ret, "%d: 0x%x\n", i,
					get_lbdf2vnic_cam(i));
		}
	}
	return buf_ret - buf;
}

static ssize_t rc_sysfs_lbdf2vnic_ram_show(
		struct device *dev,
		struct device_attribute *attr,
		char *buf)
{
	int i;
	char *buf_ret = buf;

	if (!buf_ret)
		return 0;

	for (i = 0; i < key_index; i++) {
		if (rc_vnic[i].valid) {
			buf_ret += sprintf(buf_ret, "%d: 0x%x\n", i,
					get_lbdf2vnic_ram(i));
		}
	}
	return buf_ret - buf;
}

static ssize_t rc_sysfs_dns_tbl_show(
		struct device *dev,
		struct device_attribute *attr,
		char *buf)
{
	int i;
	char *buf_ret = buf;

	if (!buf_ret)
		return 0;

	for (i = 0; i < key_index; i++) {
		if (rc_vnic[i].valid) {
			buf_ret += sprintf(buf_ret, "%d: 0x%x\n", i,
					get_dns_tbl(i));
		}
	}
	return buf_ret - buf;
}

static ssize_t rc_sysfs_vnic_cfg_ram_show(
		struct device *dev,
		struct device_attribute *attr,
		char *buf)
{
	int i, j, data[VNIC_CFG_LENGTH] = { 0 };
	char *buf_ret = buf;

	if (!buf_ret)
		return 0;

	for (i = 0; i < key_index; i++) {
		if (rc_vnic[i].valid) {
			get_vnic_cfg_ram(i, data);
			for (j = 0; j < VNIC_CFG_LENGTH; j++)
				buf_ret += sprintf(buf_ret,
						"%d - %d: 0x%x\n", i, j,
						data[j]);
		}
	}
	return buf_ret - buf;
}

static ssize_t rc_sysfs_ind_base_show(
		struct device *dev,
		struct device_attribute *attr,
		char *buf)
{
	int i;
	char *buf_ret = buf;
	int phys_addr;

	if (!buf_ret)
		return 0;

	for (i = 0; i < key_index; i++) {
		if (rc_vnic[i].valid) {
			phys_addr = (int)virt_to_phys(rc_vnic[i].ind_base);
			buf_ret += sprintf(buf_ret, "%d: 0x%x\n",
					i, (int)phys_addr);
		}
	}
	return buf_ret - buf;
}

static ssize_t rc_sysfs_rc_vnic_show(
		struct device *dev,
		struct device_attribute *attr,
		char *buf)
{
	int i;
	char *buf_ret = buf;
	int bus, device, func;

	if (!buf_ret)
		return 0;

	for (i = 0; i < key_index; i++) {
		if (rc_vnic[i].valid) {
			bus = bdf_to_bus(rc_vnic[i].bdf);
			device = bdf_to_dev(rc_vnic[i].bdf);
			func = bdf_to_fnc(rc_vnic[i].bdf);

			buf_ret += sprintf(buf_ret, "\nRC dev: %x\n", i);
			buf_ret += sprintf(buf_ret,
					"%d--> %02x:%02x.%02x --> port: %x\n",
					i, bus, device, func, rc_vnic[i].port);
		}
	}
	return buf_ret - buf;
}

static ssize_t rc_sysfs_dev_aer_show(
		struct device *dev,
		struct device_attribute *attr,
		char *buf)
{
	if (buf) {
		unsigned int bus, slot;
		struct pci_dev *pcidev;
		bus = bdf_to_bus(dvnic.bdf);
		slot = (bdf_to_dev(dvnic.bdf) << 3) | bdf_to_fnc(dvnic.bdf);
		pcidev = pci_get_bus_and_slot(bus, slot);

		if (pcidev != NULL)
			return dev_aer_info(pcidev, buf);
        else
            return 0;
	}
	else
		return 0;
}

static ssize_t rc_sysfs_acl_info_show(
		struct device *dev,
		struct device_attribute *attr,
		char *buf)
{
	if (buf)
		return acl_info_show(buf);
	else 
		return 0;
}

static ssize_t rc_sysfs_remap_info_show(
		struct device *dev,
		struct device_attribute *attr,
		char *buf)
{
	char *buf_ret = buf;
	int bus, device, func;
	unsigned char i, j;
	dev_mem_t *pdev;
	rc_vnic_info_t *rcvnic;

	if (buf) {
		for (i = 0; i < key_index; i++) {
			if (rc_vnic[i].valid) {
				bus = bdf_to_bus(rc_vnic[i].bdf);
				device = bdf_to_dev(rc_vnic[i].bdf);
				func = bdf_to_fnc(rc_vnic[i].bdf);

				buf_ret +=
					sprintf(buf_ret, "\nRC dev: %x\n", i);
				buf_ret +=
					sprintf(buf_ret, "id  b: d. f dev_base"
						"   remap_base window\n");
				rcvnic = &rc_vnic[i];
				for (j = 0; j < BAR_COUNT; j++) {
					if (rcvnic->mem_mapped[j].valid) {
						pdev = &rcvnic->mem_mapped[j];
						buf_ret +=
							sprintf(buf_ret,
							"%02x %02x:%02x.%02x"
							" 0x%08x 0x%08x %02x\n",
							i, bus, device, func,
							pdev->dev_base,
							pdev->remap_base,
							pdev->window);
					}
				}
			}
		}
		return buf_ret - buf;
	} else {
		return 0;
	}
}

static ssize_t rc_sysfs_retrain_store(
		struct device *dev,
		struct device_attribute *attr,
		const char *buf, size_t count)
{
	unsigned long val;
	char *endp;

	val = simple_strtoul(buf, &endp, 0);
	if (endp == buf || val > 1)
		return -EINVAL;
	val ? retrain_port(1) : retrain_port(0);
	return count;
}

static ssize_t rc_sysfs_rc_reset_store(
		struct device *dev,
		struct device_attribute *attr,
		const char *buf, size_t count)
{
	unsigned long val;
	char *endp;

	val = simple_strtoul(buf, &endp, 0);
	if (endp == buf)
		return -EINVAL;
	rc_reset();
	return count;
}

#define DVNICF_INSTALLED	0x0001

static ssize_t rc_sysfs_dvnic_bdf_show(struct device *dev,
					  struct device_attribute *attr,
					  char *buf)
{
	return sprintf(buf, "0x%04x\n", dvnic.bdf);
}

static ssize_t rc_sysfs_dvnic_bdf_store(struct device *dev,
					   struct device_attribute *attr,
					   const char *buf,
					   size_t len)
{
	u16 bdf = simple_strtoul(buf, NULL, 0);

	if (dvnic.bdf != bdf) {
		dvnic.bdf = bdf;
		dvnic.flags &= ~DVNICF_INSTALLED;
	}
	return len;
}

static ssize_t rc_sysfs_dvnic_memspace_show(struct device *dev,
					  struct device_attribute *attr,
					  char *buf)
{
	return sprintf(buf, "0x%04x\n", dvnic.memspace);
}

static ssize_t rc_sysfs_dvnic_memspace_store(struct device *dev,
					   struct device_attribute *attr,
					   const char *buf,
					   size_t len)
{
	dvnic.memspace = simple_strtoul(buf, NULL, 0);
	return len;
}

static ssize_t rc_sysfs_dvnic_ring_base_show(struct device *dev,
					     struct device_attribute *attr,
					     char *buf)
{
	return sprintf(buf, "0x%x\n", dvnic.ring_base);
}

static ssize_t rc_sysfs_dvnic_addr_show(struct device *dev,
					struct device_attribute *attr,
					char *buf)
{
	return sprintf(buf, "0x%08x\n", dvnic.addr);
}

static ssize_t rc_sysfs_dvnic_addr_store(struct device *dev,
					 struct device_attribute *attr,
					 const char *buf,
					 size_t len)
{
	dvnic.addr = simple_strtoul(buf, NULL, 0);
	return len;
}

static ssize_t rc_sysfs_dvnic_cmd_show(struct device *dev,
				       struct device_attribute *attr,
				       char *buf)
{
	return 0;
}

static ssize_t rc_sysfs_dvnic_cmd_store(struct device *dev,
					struct device_attribute *attr,
					const char *buf,
					size_t len)
{
	char cmd[20], *ep;
	size_t cmdlen;
	unsigned int bus, slot;
	struct pci_dev *pcidev;

	// Trim trailing white space from cmd.
	cmdlen = len < sizeof(cmd) - 1 ? len : sizeof(cmd) - 1;
	memcpy(cmd, buf, cmdlen);
	ep = &cmd[cmdlen - 1];
	while (cmdlen > 0 && (*ep == ' ' || *ep == '\n')) {
		ep--;
		cmdlen--;
	}
	cmd[cmdlen] = '\0';

	if (strcmp(cmd, "install") == 0 &&
	    (dvnic.flags & DVNICF_INSTALLED) == 0) {
		bus = bdf_to_bus(dvnic.bdf);
		slot = (bdf_to_dev(dvnic.bdf) << 3) | bdf_to_fnc(dvnic.bdf);
		pcidev = pci_get_bus_and_slot(bus, slot);
		if (pcidev != NULL) {
			const char action = 1; // Indirect addressing

			setup_rcvnic_reg(pcidev, dvnic.addr,
					 dvnic.memspace, action,
					 &dvnic.ring_base);
			dvnic.flags |= DVNICF_INSTALLED;
		} else {
			printk(KERN_INFO PFX "no device at %02x:%02x.%02x\n",
					bdf_to_bus(dvnic.bdf),
					bdf_to_dev(dvnic.bdf),
					bdf_to_fnc(dvnic.bdf));
		}
	} else if (strcmp(cmd, "uninstall") == 0) {
		bus = bdf_to_bus(dvnic.bdf);
		slot = (bdf_to_dev(dvnic.bdf) << 3) | bdf_to_fnc(dvnic.bdf);
		pcidev = pci_get_bus_and_slot(bus, slot);
		if (pcidev != NULL) {
			release_rcvnic_reg(pcidev);
			dvnic.flags &= ~DVNICF_INSTALLED;
		}
	}
	return len;
}

#define rc_sysfs_ro(name) \
static DEVICE_ATTR(name, 0444, rc_sysfs_##name##_show, NULL);

#define rc_sysfs_wo(name) \
static DEVICE_ATTR(name, 0666, NULL, rc_sysfs_##name##_store);

#define rc_sysfs_rw(name) \
static DEVICE_ATTR(name, 0666, \
                   rc_sysfs_##name##_show, \
                   rc_sysfs_##name##_store);

rc_sysfs_ro(port0_int_err_stats);
rc_sysfs_ro(port2_int_err_stats);
rc_sysfs_ro(lbdf2vnic_cam);
rc_sysfs_ro(lbdf2vnic_ram);
rc_sysfs_ro(dns_tbl);
rc_sysfs_ro(vnic_cfg_ram);
rc_sysfs_ro(ind_base);
rc_sysfs_ro(rc_vnic);
rc_sysfs_ro(dev_aer);
rc_sysfs_ro(remap_info);
rc_sysfs_ro(acl_info);
rc_sysfs_wo(retrain);
rc_sysfs_wo(rc_reset);
rc_sysfs_rw(dvnic_bdf);
rc_sysfs_rw(dvnic_addr);
rc_sysfs_ro(dvnic_ring_base);
rc_sysfs_rw(dvnic_memspace);
rc_sysfs_rw(dvnic_cmd);

struct rc_info rc_dev_info;

static int rc_open(struct inode *inode, struct file *filep)
{
	struct rc_info *rcdev = &rc_dev_info;

	atomic_inc(&rcdev->refcnt);
	return 0;
}

static int rc_release(struct inode *inode, struct file *filep)
{
	struct rc_info *rcdev = &rc_dev_info;

	atomic_dec(&rcdev->refcnt);
	return 0;
}

static unsigned int rc_poll(struct file *filep, poll_table *wait)
{
	struct rc_info *rcdev = &rc_dev_info;

	poll_wait(filep, &rcdev->waitq, wait);
	if (rc_have_events()) {
		return POLLIN | POLLRDNORM;
	}
	return 0;
}

static struct file_operations rc_fops = {
	.owner = THIS_MODULE,
	.open = rc_open,
	.release = rc_release,
	.unlocked_ioctl = rc_ioctl,
	.poll = rc_poll,
};

static int __devinit rc_probe(struct platform_device *pdev)
{
	int r = 0;

	if (device_create_file(&pdev->dev, &dev_attr_port0_int_err_stats)) {
		printk(KERN_WARNING PFX
				"port0_int_err_stats file create failed\n");
	}
	if (device_create_file(&pdev->dev, &dev_attr_port2_int_err_stats)) {
		printk(KERN_WARNING PFX
				"port2_int_err_stats file create failed\n");
	}
	if (device_create_file(&pdev->dev, &dev_attr_lbdf2vnic_cam)) {
		printk(KERN_WARNING PFX "lbdf2vnic_cam file create failed\n");
	}
	if (device_create_file(&pdev->dev, &dev_attr_lbdf2vnic_ram)) {
		printk(KERN_WARNING PFX "lbdf2vnic_ram file create failed\n");
	}
	if (device_create_file(&pdev->dev, &dev_attr_dns_tbl)) {
		printk(KERN_WARNING PFX "dns_tbl file create failed\n");
	}
	if (device_create_file(&pdev->dev, &dev_attr_vnic_cfg_ram)) {
		printk(KERN_WARNING PFX "vnic_cfg_ram file create failed\n");
	}
	if (device_create_file(&pdev->dev, &dev_attr_ind_base)) {
		printk(KERN_WARNING PFX "ind_base file create failed\n");
	}
	if (device_create_file(&pdev->dev, &dev_attr_rc_vnic)) {
		printk(KERN_WARNING PFX "rc_vnic file create failed\n");
	}
	if (device_create_file(&pdev->dev, &dev_attr_dev_aer)) {
		printk(KERN_WARNING PFX "dev_aer file create failed\n");
	}
	if (device_create_file(&pdev->dev, &dev_attr_remap_info)) {
		printk(KERN_WARNING PFX "remap_info file create failed\n");
	}
	if (device_create_file(&pdev->dev, &dev_attr_retrain)) {
		printk(KERN_WARNING PFX "retrain file create failed\n");
	}
	if (device_create_file(&pdev->dev, &dev_attr_dvnic_bdf)) {
		printk(KERN_WARNING PFX "dvnic_bdf create failed\n");
	}
	if (device_create_file(&pdev->dev, &dev_attr_dvnic_memspace)) {
		printk(KERN_WARNING PFX "dvnic_memspace create failed\n");
	}
	if (device_create_file(&pdev->dev, &dev_attr_dvnic_addr)) {
		printk(KERN_WARNING PFX "dvnic_addr create failed\n");
	}
	if (device_create_file(&pdev->dev, &dev_attr_dvnic_ring_base)) {
		printk(KERN_WARNING PFX "dvnic_ring_base create failed\n");
	}
	if (device_create_file(&pdev->dev, &dev_attr_dvnic_cmd)) {
		printk(KERN_WARNING PFX "dvnic_cmd create failed\n");
	}
	if (device_create_file(&pdev->dev, &dev_attr_rc_reset)) {
		printk(KERN_WARNING PFX "rc_reset file create failed\n");
	}
	if (device_create_file(&pdev->dev, &dev_attr_acl_info)) {
		printk(KERN_WARNING PFX "acl_info file create failed\n");
	}

	r = register_chrdev(rc_major, DRV_NAME, &rc_fops);
	if (r < 0)
		return r;

	rc_major = r;
	printk(KERN_INFO PFX "Registering rc driver with %d major\n", rc_major);
	rc_evt_init();
	return 0;
}

static int __devinit rc_remove(struct platform_device *dev)
{
	// Decrease the reference counter and free kobj
	return 0;
}

static struct platform_driver rc_driver = {
	.driver = {
		.name = DRV_NAME,
		.owner = THIS_MODULE,
	},
	.probe = rc_probe,
	.remove = rc_remove,
};

static int __init rc_init(void)
{
	printk(KERN_INFO PFX "%s, ver %s\n", DRV_DESCRIPTION, DRV_VERSION);
	printk(KERN_INFO PFX "%s\n", DRV_COPYRIGHT);

	return platform_driver_register(&rc_driver);
}

static void __exit rc_exit(void)
{
	if (rc_major)
		unregister_chrdev(rc_major, DRV_NAME);
	platform_driver_unregister(&rc_driver);
}

module_init(rc_init);
module_exit(rc_exit);
