
/*
 * Copyright (c) 2013 Cisco Systems, Inc.  All rights reserved.
 */
#ident "$Id: palo-flash.c 68698 2011-01-12 21:53:47Z buildsa $"

#include <linux/init.h>
#include <linux/module.h>
#include <linux/types.h>
#include <linux/kernel.h>
#include <linux/ioport.h>

#include <linux/mtd/mtd.h>
#include <linux/mtd/map.h>
#include <linux/mtd/cfi.h>
#include <linux/mtd/partitions.h>

#include <asm/io.h>

#include <palo/fw/mcpu/include/paloflash.h>

static struct mtd_partition palo_p30_partitions[] = {
	{
		.name	= PALOFLASH_OBFL_NAME,
		.offset = PALOFLASH_P30_OBFL_OFFSET,
		.size	= PALOFLASH_P30_OBFL_SIZE,
	},
	{
		.name	= PALOFLASH_CONFIG_NAME,
		.offset = PALOFLASH_P30_CONFIG_OFFSET,
		.size	= PALOFLASH_P30_CONFIG_SIZE,
	},
	{
		.name	= PALOFLASH_FIRMWARE_NAME,
		.offset = PALOFLASH_P30_FIRMWARE_OFFSET,
		.size	= PALOFLASH_P30_FIRMWARE_SIZE,
	},
	{
		.name	= PALOFLASH_BOOT_NAME,
		.offset = PALOFLASH_P30_BOOT_OFFSET,
		.size	= PALOFLASH_P30_BOOT_SIZE,
	},
	{
		.name	= PALOFLASH_FWMAP_NAME,
		.offset = PALOFLASH_P30_FWMAP_OFFSET,
		.size	= PALOFLASH_P30_FWMAP_SIZE,
	},
	{
		.name	= PALOFLASH_ENV_NAME,
		.offset = PALOFLASH_P30_ENV_OFFSET,
		.size	= PALOFLASH_P30_ENV_SIZE,
	},
	{
		.name	= PALOFLASH_BOARDID_NAME,
		.offset = PALOFLASH_P30_BOARDID_OFFSET,
		.size	= PALOFLASH_P30_BOARDID_SIZE,
	},
	{
		.name	= PALOFLASH_KDUMP_NAME,
		.offset = PALOFLASH_P30_KDUMP_OFFSET,
		.size	= PALOFLASH_P30_KDUMP_SIZE,
	},
	{
		.name	= PALOFLASH_DIAG_NAME,
		.offset = PALOFLASH_P30_DIAG_OFFSET,
		.size	= PALOFLASH_P30_DIAG_SIZE,
	},
};

static struct mtd_partition palo_m29_partitions[] = {
	{
		.name	= PALOFLASH_OBFL_NAME,
		.offset = PALOFLASH_M29_OBFL_OFFSET,
		.size	= PALOFLASH_M29_OBFL_SIZE,
	},
	{
		.name	= PALOFLASH_CONFIG_NAME,
		.offset = PALOFLASH_M29_CONFIG_OFFSET,
		.size	= PALOFLASH_M29_CONFIG_SIZE,
	},
	{
		.name	= PALOFLASH_FIRMWARE_NAME,
		.offset = PALOFLASH_M29_FIRMWARE_OFFSET,
		.size	= PALOFLASH_M29_FIRMWARE_SIZE,
	},
	{
		.name	= PALOFLASH_BOOT_NAME,
		.offset = PALOFLASH_M29_BOOT_OFFSET,
		.size	= PALOFLASH_M29_BOOT_SIZE,
	},
	{
		.name	= PALOFLASH_FWMAP_NAME,
		.offset = PALOFLASH_M29_FWMAP_OFFSET,
		.size	= PALOFLASH_M29_FWMAP_SIZE,
	},
	{
		.name	= PALOFLASH_ENV_NAME,
		.offset = PALOFLASH_M29_ENV_OFFSET,
		.size	= PALOFLASH_M29_ENV_SIZE,
	},
	{
		.name	= PALOFLASH_BOARDID_NAME,
		.offset = PALOFLASH_M29_BOARDID_OFFSET,
		.size	= PALOFLASH_M29_BOARDID_SIZE,
	},
	{
		.name	= PALOFLASH_KDUMP_NAME,
		.offset = PALOFLASH_M29_KDUMP_OFFSET,
		.size	= PALOFLASH_M29_KDUMP_SIZE,
	},
	{
		.name	= PALOFLASH_DIAG_NAME,
		.offset = PALOFLASH_M29_DIAG_OFFSET,
		.size	= PALOFLASH_M29_DIAG_SIZE,
	},
};

static struct map_info palo_map = {
	.name = "PALO Flash",
	.size = PALOFLASH_SIZE,
	.phys = PALOFLASH_PHYS,
	.bankwidth = 2,
};

struct mtd_info *palo_mtd;
EXPORT_SYMBOL(palo_mtd);
static u_int32_t palo_kdump_phys;
static int (*raw_write_buf)(u_int16_t *waddr, u_int16_t *buf, int nwords);

u_int32_t palo_kdump_size;
EXPORT_SYMBOL(palo_kdump_size);

#define FLASH_BUFNWORDS	    32			/* lowest common size */

#define FCMD_P30_CLEAR_SR	0x50
#define FCMD_P30_ERASE_SETUP	0x20
#define FCMD_P30_ERASE_CONF	0xd0
#define FCMD_P30_BUFPGM_SETUP	0xe8
#define FCMD_P30_BUFPGM_CONF	0xd0

static inline void w16(u_int16_t *addr, u_int16_t data)
{
	*(volatile u_int16_t *)addr = data;
}

static inline u_int16_t r16(u_int16_t *addr)
{
	return *(volatile u_int16_t *)addr;
}

static int fl_buf_write_p30(u_int16_t *waddr, u_int16_t *buf, int nwords)
{
	u_int32_t w;
	u_int16_t sr;

	if (nwords < 1 || nwords > FLASH_BUFNWORDS) {
		printk(KERN_WARNING "fl_buf_write: size "
				"too big (%d)\n", nwords);
		return -1;
	}

	/*
	 * Prep buffered write.
	 */
	do {
		w16(waddr, FCMD_P30_CLEAR_SR);
		w16(waddr, FCMD_P30_BUFPGM_SETUP);
		sr = r16(waddr);   // check for other operations in progress
	} while (!(sr & 0x80));
	w16(waddr, nwords - 1);

	/*
	 * Fill buffer.
	 */
	for (w = 0; w < nwords; w++) {
		w16(waddr + w, buf[w]);
	}

	/*
	 * Confirm.
	 */
	w16(waddr, FCMD_P30_BUFPGM_CONF);

	/*
	 * Wait for completion and return.
	 */
	while (!((sr = r16(waddr)) & 0x80))
		; /* spin */
	if (sr != 0x80) {
		printk(KERN_WARNING "Bad SR %04x\n", sr);
		return -1;
	}

	return 0;
}

#define DQ6	(1 << 6)
#define DQ5	(1 << 5)

#define FLASH_CMD(x)	(u_int16_t *)CKSEG1ADDR(palo_kdump_phys + ((x) << 1))

static int fl_buf_write_m29(u_int16_t *waddr, u_int16_t *buf, int nwords)
{
	u_int16_t sr, osr;
	u_int32_t w;

	if (nwords < 1 || nwords > FLASH_BUFNWORDS) {
		printk(KERN_WARNING "fl_buf_write: too many (%d)\n", nwords);
		return -1;
	}

	/*
	 * Prepare buffered write.
	 */
	w16(FLASH_CMD(0x555), 0xaa);
	w16(FLASH_CMD(0x2aa), 0x55);
	w16(waddr, 0x25);
	w16(waddr, nwords - 1);

	/*
	 * Fill buffer.
	 */
	for (w = 0; w < nwords; w++) {
		w16(waddr + w, buf[w]);
	}

	/*
	 * Confirm.
	 */
	w16(waddr, 0x29);
	wmb();

	/*
	 * Wait for completion and return.
	 */
	sr = r16(waddr);
	do {
		osr = sr;
		sr = r16(waddr);
	} while (sr != osr);

	return 0;
}

int palo_kdump_write(u_int32_t offset, void *buf, int nbytes)
{
	u_int16_t *wp = (u_int16_t *)CKSEG1ADDR(palo_kdump_phys + offset);
	int ret, woffs, wremain, wcnt;
	u_int16_t *rp = buf;

	if (raw_write_buf == NULL || (offset & 0x1) || (nbytes & 0x1)) {
		return -ENXIO;
	}

	woffs = offset / 2;
	wremain = nbytes / 2;
	while (wremain > 0) {
		wcnt = min(wremain, FLASH_BUFNWORDS - woffs % FLASH_BUFNWORDS);
		ret = (*raw_write_buf)(wp, rp, wcnt);
		if (ret < 0) {
			return -ENXIO;
		}
		wremain -= wcnt;
		woffs += wcnt;
		wp += wcnt;
		rp += wcnt;
	}
	return 0;
}

static int __init palo_mtd_init(void)
{
	palo_map.virt = ioremap(palo_map.phys, palo_map.size);

	if (!palo_map.virt) {
		printk("Palo MTD Map ioremap failed !\n");
		return -EIO;
	}
	simple_map_init(&palo_map);

	palo_mtd = do_map_probe("cfi_probe", &palo_map);
	if (!palo_mtd) {
		printk("Palo MTD Map probe failed !\n");
		iounmap(palo_map.virt);
                return -ENXIO;
	}
	palo_mtd->owner = THIS_MODULE;

	switch (((struct cfi_private *)palo_map.fldrv_priv)->cfiq->P_ID) {
	case P_ID_INTEL_STD:
        case P_ID_INTEL_EXT:
		mtd_device_register(palo_mtd, palo_p30_partitions,
				    ARRAY_SIZE(palo_p30_partitions));
		palo_kdump_phys = PALOFLASH_PHYS + PALOFLASH_P30_KDUMP_OFFSET;
		palo_kdump_size = PALOFLASH_P30_KDUMP_SIZE;
		raw_write_buf = fl_buf_write_p30;
                printk("Registered P30 Partitions\n");
		break;

	case P_ID_AMD_STD:
        case P_ID_AMD_EXT:
		mtd_device_register(palo_mtd, palo_m29_partitions,
				    ARRAY_SIZE(palo_m29_partitions));
		palo_kdump_phys = PALOFLASH_PHYS + PALOFLASH_M29_KDUMP_OFFSET;
		palo_kdump_size = PALOFLASH_M29_KDUMP_SIZE;
		raw_write_buf = fl_buf_write_m29;
                printk("Registered M29 Partitions\n");
		break;
        default:
		printk("No partitions registered\n");
                break;
	}
	return 0;
}

static void __exit palo_mtd_cleanup(void)
{
	if (palo_mtd) {
		mtd_device_unregister(palo_mtd);
		map_destroy(palo_mtd);
	}
	if (palo_map.virt) {
		iounmap((void *)palo_map.virt);
		palo_map.virt = 0;
	}
}

module_init(palo_mtd_init);
module_exit(palo_mtd_cleanup);

MODULE_AUTHOR("Cisco Systems, Inc");
MODULE_DESCRIPTION("PALO MTD driver");
MODULE_LICENSE("GPL");
