/*
 * Copyright 2012 Cisco Systems, Inc.  All rights reserved.
 */
#ident "$Id: mipsperf.c 21 2016-12-02 21:25:59Z buildsa $"

#include <linux/fs.h>
#include <linux/init.h>
#include <linux/kernel.h>
#include <linux/miscdevice.h>
#include <asm/uaccess.h>
#include <linux/module.h>
#include "mipsperf.h"

static long mipsperf_ioctl(struct file *file,
		unsigned int cmd, unsigned long arg)
{
	void __user *argp = (void __user *)arg;
	struct mipsperf_data data;
	static u_int32_t last_c0_count;
	u_int32_t t_now;
	int ret = -EOPNOTSUPP;

	switch (cmd) {
	case MIPSPERF_READCOUNTS:
		data.val[0] = read_c0_perfcntr0();
		data.val[1] = read_c0_perfcntr1();
		t_now = read_c0_count();
		data.dcount = t_now - last_c0_count;
		last_c0_count = t_now;
		if (copy_to_user(argp, &data, sizeof (data))) {
			return -EFAULT;
		}
		return 0;

	case MIPSPERF_READCONFIG:
		data.val[0] = read_c0_perfctrl0();
		data.val[1] = read_c0_perfctrl1();
		if (copy_to_user(argp, &data, sizeof (data))) {
			return -EFAULT;
		}
		return 0;

	case MIPSPERF_WRITECONFIG:
		if (copy_from_user(&data, argp, sizeof (data))) {
			return -EFAULT;
		}
		write_c0_perfctrl0(data.val[0]);
		write_c0_perfctrl1(data.val[1]);
		write_c0_perfcntr0(0);
		write_c0_perfcntr1(0);
		last_c0_count = read_c0_count();
		return 0;

	default:
		break;
	}
	return ret;
}

static struct file_operations mipsperf_fops = {
	.owner   = THIS_MODULE,
	.unlocked_ioctl   = mipsperf_ioctl,
	.open    = nonseekable_open,
};

static struct miscdevice mipsperf_dev = {
	.minor   = MISC_DYNAMIC_MINOR,
	.name    = "mipsperf",
	.fops    = &mipsperf_fops
};

static int __init mipsperf_init(void)
{
	int ret;

	ret = misc_register(&mipsperf_dev);
	if (ret) {
		printk(KERN_ERR "mipsref registration failed: %d\n", ret);
		return ret;
	} else {
		printk(KERN_INFO "mipsperf minor=%d\n", mipsperf_dev.minor);
	}
	return ret;
}

static void __exit mipsperf_exit(void)
{
	misc_deregister(&mipsperf_dev);
}

module_init(mipsperf_init);
module_exit(mipsperf_exit);


MODULE_LICENSE("GPL");

