
#include <asm/asm.h>
#include <asm/regdef.h>
#include <asm/mipsregs.h>
#include <palo/fw/mcpu/include/ecpuctx.h>
#include <palo/fw/mcpu/include/paloecpu.h>

	.set	mips32r2
	.set	noreorder

/*
 * NMI entry point via FLASH (u-boot)
 * On entry:
 *	PC in kseg1 (uncached)
 *	Status.ERL=1
 *	Status.BEV=1
 *	k0 -> &ecpuctx[CPUNum] (kseg1)
 *	k0, k1 and t0 saved in ectx
 *	ErrorEPC=return address (no branch delay slot info)
 *
 * NMI is used to break us into the debugger.  It's imperfect compared to an
 * EJTAG interrupt, but it's what we have.
 * The task here is to recover the registers and enter the debugger via SDBBP.
 */
LEAF(paloecpu_nmi_entry)
	lw	t0, ECTX_NMITMP(k0)	    // restore t0
	lw	k1, ECTX_NMIK1(k0)	    // restore k1
	lw	k0, ECTX_NMIK0(k0)	    // restore k0
	sdbbp				    // enter debugger (doesn't return)
	/* not reached */
	END(paloecpu_nmi_entry)

/*
 * EJTAG entry point via FLASH (u-boot).
 * On entry:
 *	PC in kseg1 (uncached)
 *	Debug.DM=1 (in debug mode)
 *	Debug.DSS, DBp, DDBL, DDBS, DIB, DINT, DDBLImpr and DDBSImpr: set
 *	k0 -> &ecpuctx[CPUNum] (kseg1)
 *	k0, k1 saved in ectx
 *	DEPC=return address
 *	Debug.DBD=branch delay slot info
 */
LEAF(paloecpu_ejtag_entry)
	/*
	 * The debug stack is at top of the debug page.  Locate the stack
	 * and save the registers.
	 */
	lw	k1, ECTX_DBPAGE(k0)		// k1 -> dbpage KSEG1 addr
	addiu	k1, PALOECPU_DBPAGESIZE - PALOECPU_XCPTREGS_SIZE // k1 -> TOS
	sw	zero, 0x00(k1)
	.set	noat
	sw	AT, 0x04(k1)
	.set	at
	sw	v0, 0x08(k1)
	sw	v1, 0x0c(k1)
	sw	a0, 0x10(k1)
	sw	a1, 0x14(k1)
	sw	a2, 0x18(k1)
	sw	a3, 0x1c(k1)
	sw	t0, 0x20(k1)
	sw	t1, 0x24(k1)
	sw	t2, 0x28(k1)
	sw	t3, 0x2c(k1)
	sw	t4, 0x30(k1)
	sw	t5, 0x34(k1)
	sw	t6, 0x38(k1)
	sw	t7, 0x3c(k1)
	sw	s0, 0x40(k1)
	sw	s1, 0x44(k1)
	sw	s2, 0x48(k1)
	sw	s3, 0x4c(k1)
	sw	s4, 0x50(k1)
	sw	s5, 0x54(k1)
	sw	s6, 0x58(k1)
	sw	s7, 0x5c(k1)
	sw	t8, 0x60(k1)
	sw	t9, 0x64(k1)
	lw	t0, ECTX_DBGK0(k0)
	sw	t0, 0x68(k1)
	lw	t0, ECTX_DBGK1(k0)
	sw	t0, 0x6c(k1)
	sw	gp, 0x70(k1)
	sw	sp, 0x74(k1)
	sw	s8, 0x78(k1)
	sw	ra, 0x7c(k1)
	mflo	t0
	sw	t0, 0x80(k1)
	mfhi	t0
	sw	t0, 0x84(k1)

	/*
	 * If the Status register has NMI set, then we came from the NMI
	 * vector.  Clear NMI, BEV and ERL, and use ErrorEPC as the DEPC.
	 */
	mfc0	t0, CP0_STATUS
	li	t1, ST0_NMI
	and	t1, t0			    // NMI set?
	beqz	t1, 1f			    // no
	nop

	/*
	 * From NMI
	 */
	li	t1, ~(ST0_BEV | ST0_NMI | ST0_ERL)
	and	t0, t1			    // clear BEV and NMI
	mtc0	t0, CP0_STATUS
	sw	t0, 0x88(k1)		    // save it
	mfc0	t0, CP0_ERROREPC	    // get ErrorPC
	sw	t0, 0x98(k1)		    // save it as PC
	b	2f
	nop

	/*
	 * Not from NMI (from Debug Exception)
	 */
1:
	sw	t0, 0x88(k1)		    // save Status unmodified
	mfc0	t0, CP0_DEPC		    // Get DEPC
	sw	t0, 0x98(k1)		    // save it as PC

	/*
	 * Continuing...
	 */
2:
	mfc0	t0, CP0_DEBUG
	sw	t0, 0x9c(k1)
	mfc0	t0, CP0_CAUSE
	sw	t0, 0x8c(k1)
	mfc0	t0, CP0_EPC
	sw	t0, 0x90(k1)
	mfc0	t0, CP0_BADVADDR
	sw	t0, 0x94(k1)

	/*
	 * Set up and call the ejtag handler, still in KSEG1.
	 * Use the ejtag stack.  Provide the ecpu ctx and stacked
	 * registers as the arg.
	 */
	move	a0, k0
	move	a1, k1
	move	s0, k1
	move	sp, k1
	jal	ejtag_handler		    // jal stays within kseg1
	nop

	/*
	 * On return, restore all registers and return.
	 */
	move	a0, s0
	b	paloecpu_ejtag_return
	nop
	END(paloecpu_ejtag_entry)

/*
 * Return from debug exception.
 * Entry:
 *	a0 -> xcptregs_t
 */
LEAF(paloecpu_ejtag_return)
	move	k1, a0
	.set	noat
	lw	AT, 0x04(k1)
	.set	at
	lw	v0, 0x08(k1)
	lw	v1, 0x0c(k1)
	lw	a0, 0x10(k1)
	lw	a1, 0x14(k1)
	lw	a2, 0x18(k1)
	lw	a3, 0x1c(k1)
	lw	t0, 0x20(k1)
	lw	t1, 0x24(k1)
	lw	t2, 0x28(k1)
	lw	t3, 0x2c(k1)
	lw	t4, 0x30(k1)
	lw	t5, 0x34(k1)
	lw	t6, 0x38(k1)
	lw	t7, 0x3c(k1)
	lw	s0, 0x40(k1)
	lw	s1, 0x44(k1)
	lw	s2, 0x48(k1)
	lw	s3, 0x4c(k1)
	lw	s4, 0x50(k1)
	lw	s5, 0x54(k1)
	lw	s6, 0x58(k1)
	lw	s7, 0x5c(k1)
	lw	t8, 0x60(k1)
	lw	t9, 0x64(k1)
	lw	gp, 0x70(k1)
	lw	sp, 0x74(k1)
	lw	s8, 0x78(k1)
	lw	ra, 0x7c(k1)
	lw	k0, 0x80(k1)
	mtlo	k0
	lw	k0, 0x84(k1)
	mthi	k0
	lw	k0, 0x88(k1)
	mtc0	k0, CP0_STATUS
	lw	k0, 0x8c(k1)
	mtc0	k0, CP0_CAUSE
	lw	k0, 0x90(k1)
	mtc0	k0, CP0_EPC
	lw	k0, 0x98(k1)
	mtc0	k0, CP0_DEPC
	lw	k0, 0x68(k1)
	lw	k1, 0x6c(k1)
	ehb
	sync
	deret
	END(paloecpu_ejtag_return)
