
/*
 * Copyright 2013 Cisco Systems, Inc.  All rights reserved.
 */
#ident "$Id: palo_ecpu.h 43 2016-12-03 22:59:19Z buildsa $"

#ifndef _MIPS24K_H_
#define _MIPS24K_H_

/*
 * Processor-specific fields in Debug Register	(CP0 Register 23, Select 0)
 */
#define DBG_DBD		0x80000000	/* branch delay slot */
#define DBG_DM		0x40000000	/* in debug mode */
#define DBG_NoDCR	0x20000000	/* dseg is not present */
#define DBG_LSNM	0x10000000	/* load/store dseg/remain */
#define DBG_DOZE	0x08000000	/* was in low power mode */
#define DBG_HALT	0x04000000	/* system bus clock was running */
#define DBG_COUNTDM	0x02000000	/* count reg counts in debug mode */
#define DBG_IBusEP	0x01000000	/* instruction bus error pending */
#define DBG_MCheckP	0x00800000	/* machine check pending */
#define DBG_CacheEP	0x00400000	/* cache error pending */
#define DBG_DBusEP	0x00200000	/* data bus error pending */
#define DBG_IEXI	0x00100000	/* imprecise exception inhibit */
#define DBG_DDBSimpr	0x00080000	/* debug data break imprecise */
#define DBG_DDBLimpr	0x00040000	/* debug data break imprecise */
#define DBG_VERMASK	0x00038000	/* EJTAG version */
#define DBG_VERSHIFT	15
#define DBG_DEXCMASK	0x00007c00	/* latest exception code */
#define DBG_DEXCSHIFT	10
#define DBG_NoSST	0x00000200	/* no single-step */
#define DBG_SST		0x00000100	/* single-step enable */
#define DBG_DINT	0x00000020	/* debug interrupt exception */
#define DBG_DIB		0x00000010	/* debug instruction break */
#define DBG_DDBS	0x00000008	/* debug data break on store */
#define DBG_DDBL	0x00000004	/* debug data break on load */
#define DBG_DBP		0x00000002	/* debug breakpoint */
#define DBG_DSS		0x00000001	/* debug single-step */

#define DBG_INTBITS \
	(DBG_DINT | DBG_DIB | DBG_DDBS | DBG_DDBL | DBG_DBP | DBG_DSS)

/*
 * Cache macro.
 */
#define cache_op(op,addr)						\
	__asm__ __volatile__(						\
	"	.set	push					\n"	\
	"	.set	noreorder				\n"	\
	"	.set	mips3\n\t				\n"	\
	"	cache	%0, %1					\n"	\
	"	.set	pop					\n"	\
	:								\
	: "i" (op), "R" (*(unsigned char *)(addr)))


/*
 * Should be in asm/mipsregs.h, but aren't.
 */
#define read_c0_datalo()	 __read_32bit_c0_register($28, 1)
#define write_c0_datalo(val)	 __write_32bit_c0_register($28, 1, val)


#define PALOECPU_DBG_STAT(x) \
	PALOECPU_DBG_STAT_MK(__LINE__, PALOECPU_DBG_STAT_##x)

#endif
