
/*
 * Copyright (c) 2008 Cisco Systems, Inc.  All rights reserved.
 */
#ident "$Id: palo_i2c.h 21 2016-12-02 21:25:59Z buildsa $"

#ifndef _PALO_I2C_DRV_H_
#define _PALO_I2C_DRV_H_

#define DRV_NAME "paloi2c"
#define DRV_DESCRIPTION "Driver for Palo I2C"
#define DRV_VERSION "1.0"
#define DRV_COPYRIGHT "Copyright(c) 2008 Cisco Systems"
#define PFX DRV_NAME ": "

#define PALO_I2C_TIMEOUT_MS     50

#define LOGLEN          1024            // 1024 x 64B entries = 64KB

extern int palo_i2c_logmajor;

struct palo_i2c {
    int                 target;
    int                 port;
    int                 error;
    unsigned long       xfer_timeout;
    u_int64_t           last_jiffies;
    u_int32_t           last_cycles;
    struct device       *dev;
    struct i2c_adapter  adapter;
    palo_i2c_logent_t   *log;
    struct cdev         log_cdev;
    u_int32_t           txfsz;
    u_int32_t           flags;
    int                 loghead;
    int                 logtail;
    u_int32_t           seq;
    struct mutex        mut;
    spinlock_t          spinlock;
    wait_queue_head_t   log_waitq;
    int                 break_armed;
    int                 break_cnt;
    int                 break_total;
};
#define PI2CF_LOGCDEV 0x1

static inline struct palo_i2c *
dev_to_palo_i2c(struct device *dev)
{
    struct platform_device *pdev = to_platform_device(dev);
    return (struct palo_i2c *)platform_get_drvdata(pdev);
}

static inline u32
readreg(const u32 pa)
{
    return readl((void *)CKSEG1ADDR(pa));
}

static void inline
writereg(const u32 pa, const u32 val)
{
    writel(val, (void *)CKSEG1ADDR(pa));
}

#define PALO_NUM_I2C    2

void palo_i2c_log_xfer(struct palo_i2c *i2c,
        struct i2c_msg *msg, int num, int ret);
int palo_i2c_log_init(struct palo_i2c *i2c);

#endif
