/*
 * Copyright 2008 Cisco Systems, Inc.  All rights reserved.
 */
#ident "$Id: palo_i2c_log.c 21 2016-12-02 21:25:59Z buildsa $"

#include <linux/init.h>
#include <linux/module.h>
#include <linux/sched.h>
#include <linux/i2c.h>
#include <linux/jiffies.h>
#include <linux/timex.h>
#include <linux/cdev.h>
#include <linux/kdev_t.h>
#include <linux/wait.h>
#include <linux/poll.h>
#include <linux/ioport.h>
#include <linux/interrupt.h>
#include <linux/spinlock.h>
#include <linux/timex.h>
#include <linux/slab.h>
//#include <asm/semaphore.h>
#include <asm/io.h>

#include <linux/platform_device.h>
#include "palo/include/synopsis/dw_i2c.h"
#include "palo/fw/mcpu/include/palo_i2c_defs.h"
#include "palo_i2c.h"

int palo_i2c_logmajor;

void
palo_i2c_log_xfer(struct palo_i2c *i2c, struct i2c_msg *msg, int num, int ret)
{
	palo_i2c_logent_t *lp = &i2c->log[i2c->loghead];

	switch (num) {
	case 1:
		if (msg[0].len > 32) {
			return;
		}
		lp->op = (msg[0].flags & I2C_M_RD) ? PALO_I2C_OP_READ :
			PALO_I2C_OP_WRITE;
		lp->adlen = 0;
		lp->dalen = msg[0].len;
		memcpy(lp->data, msg[0].buf, lp->dalen);
		break;

	case 2:
		if (msg[0].len > 4 || (msg[0].flags & I2C_M_RD) ||
		    msg[1].len > 32) {
			return;
		}
		lp->op = (msg[1].flags & I2C_M_RD) ? PALO_I2C_OP_READ :
			PALO_I2C_OP_WRITE;
		lp->adlen = msg[0].len;
		memcpy(lp->addr, msg[0].buf, lp->adlen);
		lp->dalen = msg[1].len;
		memcpy(lp->data, msg[1].buf, lp->dalen);
		break;

	default:
		return;
	}
	lp->jiffies = i2c->last_jiffies;
	lp->cycles = i2c->last_cycles;
	lp->seq = i2c->seq++;
	lp->target = msg[0].addr;
	lp->ret = ret;

	i2c->loghead = (i2c->loghead + 1) % LOGLEN;
	if (i2c->loghead == i2c->logtail)
		i2c->logtail = (i2c->logtail + 1) % LOGLEN;
	wake_up_interruptible(&i2c->log_waitq);
}

static int
palo_i2c_log_open(struct inode *inode, struct file *filp)
{
	struct palo_i2c *i2c;

	i2c = container_of(inode->i_cdev, struct palo_i2c, log_cdev);
	filp->private_data = i2c;

	return 0;
}

static ssize_t palo_i2c_log_read(struct file *filp,
	char __user *buf, size_t count, loff_t *pos)
{
	struct palo_i2c *i2c = filp->private_data;
	int avail_count, copy_count, copy_len, nitems, nbytes;

	if (mutex_lock_interruptible(&i2c->mut))
		return -ERESTARTSYS;

	while (i2c->loghead == i2c->logtail) {
		mutex_unlock(&i2c->mut);
		if (filp->f_flags & O_NONBLOCK)
			return -EAGAIN;
		if (wait_event_interruptible(i2c->log_waitq,
			i2c->loghead != i2c->logtail))
			return -ERESTARTSYS;
		if (mutex_lock_interruptible(&i2c->mut))
			return -ERESTARTSYS;
	}
	if (i2c->loghead > i2c->logtail)
		avail_count = i2c->loghead - i2c->logtail;
	else
		avail_count = LOGLEN - i2c->logtail + i2c->loghead;

	copy_count = min((int) count / (int) sizeof(palo_i2c_logent_t),
		avail_count);
	copy_len = copy_count * sizeof(palo_i2c_logent_t);
	if (copy_count > 0) {
		if (i2c->loghead > i2c->logtail) {
			nbytes = copy_len;
			if (copy_to_user(buf, &i2c->log[i2c->logtail], nbytes))
				goto fault;
		} else {
			nitems = min(copy_count, LOGLEN - i2c->logtail);
			nbytes = nitems * sizeof(palo_i2c_logent_t);
			if (copy_to_user(buf, &i2c->log[i2c->logtail], nbytes))
				goto fault;
			buf += nbytes;
			nitems = copy_count - nitems;
			if (nitems > 0) {
				nbytes = nitems * sizeof(palo_i2c_logent_t);
				if (copy_to_user(buf, &i2c->log[0], nbytes))
					goto fault;
			}
		}
	}
	i2c->logtail = (i2c->logtail + copy_count) % LOGLEN;
	mutex_unlock(&i2c->mut);
	return copy_len;
fault:
	mutex_unlock(&i2c->mut);
	return -EFAULT;
}

static ssize_t palo_i2c_log_write(struct file *filp,
	const char __user *buf, size_t count, loff_t *pos)
{
	struct palo_i2c *i2c = filp->private_data;
	struct i2c_msg msg;
	u_int8_t msgbuf[32];
	int nbytes;

	if (mutex_lock_interruptible(&i2c->mut))
		return -ERESTARTSYS;

	nbytes = min(sizeof(msgbuf), count);
	if (copy_from_user(msgbuf, buf, nbytes))
		goto fault;
	msg.addr = 0xff;
	msg.flags = 0;
	msg.len = nbytes;
	msg.buf = msgbuf;
	palo_i2c_log_xfer(i2c, &msg, 1, 0);

	mutex_unlock(&i2c->mut);
	return nbytes;

fault:
	mutex_unlock(&i2c->mut);
	return -EFAULT;
}

static unsigned int palo_i2c_log_poll(struct file *filp, poll_table *wait)
{
	struct palo_i2c *i2c = filp->private_data;
	unsigned int mask = 0;

	mutex_lock(&i2c->mut);
	poll_wait(filp, &i2c->log_waitq, wait);
	if (i2c->loghead != i2c->logtail)
		mask = POLLIN | POLLRDNORM;
	mutex_unlock(&i2c->mut);

	return mask;
}

static struct file_operations palo_i2c_log_fops = {
	.owner = THIS_MODULE,
	.open = palo_i2c_log_open,
	.read = palo_i2c_log_read,
	.write = palo_i2c_log_write,
	.poll = palo_i2c_log_poll,
};

int palo_i2c_log_init(struct palo_i2c *i2c)
{
	dev_t dev;
	int ret = -ENOMEM;

	init_waitqueue_head(&i2c->log_waitq);
	i2c->log = kmalloc(sizeof(*i2c->log) * LOGLEN, GFP_KERNEL);
	if (i2c->log == NULL) {
		return ret;
	}
	i2c->loghead = 0;
	i2c->logtail = 0;

	dev = MKDEV(palo_i2c_logmajor, i2c->port);
	cdev_init(&i2c->log_cdev, &palo_i2c_log_fops);
	i2c->log_cdev.owner = THIS_MODULE;
	ret = cdev_add(&i2c->log_cdev, dev, 1);
	if (ret) {
		printk(KERN_ERR DRV_NAME "cdev_add failed (log)\n");
		goto err;
	}
	i2c->flags |= PI2CF_LOGCDEV;

	return ret;
err:
	kfree(i2c->log);
	i2c->log = NULL;
	return ret;
}

void __exit palo_i2c_log_cleanup(struct palo_i2c *i2c)
{
	if (i2c->log != NULL) {
		kfree(i2c->log);
		i2c->log = NULL;
	}
	if (i2c->flags & PI2CF_LOGCDEV) {
		cdev_del(&i2c->log_cdev);
		i2c->flags &= ~PI2CF_LOGCDEV;
	}
}
