/*
 * Copyright 2014 Cisco Systems, Inc.  All rights reserved.
 */
#ident "$Id$"

#include <linux/device.h>
#include <linux/module.h>
#include <linux/moduleparam.h>
#include <linux/kernel.h>
#include <linux/errno.h>
#include <linux/types.h>
#include <linux/init.h>
#include <linux/fs.h>
#include <linux/platform_device.h>
#include <linux/interrupt.h>
#include <linux/printk.h>
#include <linux/slab.h>
#include <paloint.h>
#include <ecpu_res.h>
#include <asm/atomic.h>
#include <palomem.h>
#include "pciemgr.h"
#include "pciesvc.h"
#include "pmimpl.h"
#include "pmsys.h"
#include "ps_sys.h"


int pmimpl_sz;
pm_impl_t *pmimpl_vaddr;
struct pm_cpu_log*pm_cpu_log;
void **trnlog_array;

extern pcie_hid_stats_t *pcie_hid_stats;
extern trace_log_t trnlog;

void *
pmsys_init(int sz)
{
    extern void *high_memory;
    pmimpl_sz = sz;
    pmimpl_vaddr = (pm_impl_t *)CKSEG1ADDR(PALOMEM_PMPCIE_BASE);
    if (sz > PALOMEM_PMPCIE_SIZE) {
        printk(KERN_ERR "%s: !!! pmimpl_vaddr %p size 0x%x > reserved 0x%x\n",
                    __func__, pmimpl_vaddr, sz, PALOMEM_PMPCIE_SIZE);
        return NULL;
    }

    trnlog_array = (void **)pmimpl_vaddr->trnlog;
    trnlog_array[0] = (void *)&trnlog;

    pm_cpu_log = pmimpl_vaddr->pm_cpu_logs;
    pcie_hid_stats = pmimpl_vaddr->pcie_hid_stats[ps_cpuid()];
    memset(pcie_hid_stats, 0, sizeof (pmimpl_vaddr->pcie_hid_stats));

    printk("%s: pmimpl_vaddr %p, sz 0x%x high_memory %p\n",
             __func__, pmimpl_vaddr, sz, high_memory);
    return pmimpl_vaddr;
}

void
pmsys_shutdown(void)
{
}

u_int32_t
pmsys_readl(u_int32_t pa)
{
    return readl((u_int32_t *)CKSEG1ADDR(pa));
}

void
pmsys_writel(u_int32_t val, u_int32_t pa)
{
    writel(val, (void *)CKSEG1ADDR(pa));
}

void
pmsys_assert_failed(const char *s, const char *func, const char *file, int line)
{
    printk("AssertFailed(%s) on %s of file %s line %d\n", s, func, file, line);
}

void
pmsys_error(const char *fmt, ...)
{
    va_list ap;

    va_start(ap, fmt);
    vprintk(fmt, ap);
    va_end(ap);
    printk("\n");
}

void *
pmsys_malloc(int sz)
{
    return kmalloc(sz, GFP_KERNEL);
}

void
pmsys_free(void *ptr)
{
    kfree(ptr);
}

