/*
 * Copyright 2007 Nuova Systems, Inc.  All rights reserved.
 * Copyright 2009 Cisco Systems, Inc.  All rights reserved.
 */
#ident "$Id: ppcie_intr.c 144815 2013-09-22 21:59:07Z sborikar $"

#include <linux/device.h>
#include <linux/module.h>
#include <linux/moduleparam.h>
#include <linux/kernel.h>
#include <linux/errno.h>
#include <linux/types.h>
#include <linux/init.h>
#include <linux/fs.h>
#include <linux/platform_device.h>
#include <linux/interrupt.h>
#include <asm/atomic.h>

#include <palo_memsync.h>
#include <paloint.h>

#include <palo/fw/common/include/ecpu_res.h>
#include <palo/include/intrc_regs.h>
#include <palo/include/pcie_mregs.h>
#include <palo/include/pcie_regs_defs.h>


#include "pciemgr.h"
#include "pciesvc.h"
#include "ppcie_evt.h"
#include "ppcie.h"

int ps_palopcie_lvl = PCIE_IND_REQ_INT_LVL;
int ps_palopcie_src = PCIE_IND_REQ_INT_SRC;


#define PIN(N) \
static irqreturn_t ppcie_intr##N(int irq, void *arg)\
                   { ps_ppcie_intr(N); return (IRQ_HANDLED); }

PIN(0); // ppcie_intr0
#if defined CONFIG_BASE_CRUZ
         PIN(1);  PIN(2);  PIN(3);  PIN(4);  PIN(5);  PIN(6);  PIN(7);
PIN(8);  PIN(9);  PIN(10); PIN(11); PIN(12); PIN(13); PIN(14); PIN(15);
PIN(16); PIN(17); PIN(18); PIN(19); PIN(20); PIN(21); PIN(22); PIN(23);
PIN(24); PIN(25); PIN(26); PIN(27); PIN(28); PIN(29); PIN(30); PIN(31)

static struct ppcie_intrs_ent {
    irq_handler_t handler;
    const char *name;
} ppcie_intrs[32] = {
#define PIE(N) \
{ .handler = ppcie_intr##N, .name = DRV_NAME "-ind" #N }
    PIE(0),  PIE(1),  PIE(2),  PIE(3),  PIE(4),  PIE(5),  PIE(6),  PIE(7),
    PIE(8),  PIE(9),  PIE(10), PIE(11), PIE(12), PIE(13), PIE(14), PIE(15),
    PIE(16), PIE(17), PIE(18), PIE(19), PIE(20), PIE(21), PIE(22), PIE(23),
    PIE(24), PIE(25), PIE(26), PIE(27), PIE(28), PIE(29), PIE(30), PIE(31)
};
#endif /* CRUZ */


static int __devinit irq_init(struct platform_device *dev, int lvl, int src,
                              irq_handler_t handler, const char *intrname)
{
    int r, irq;

    irq = palo_int_to_irq(lvl, src);
    r = request_irq(irq, handler, IRQF_DISABLED, intrname, dev);
    if (r) {
        printk(KERN_ERR PFX
               "Unable to request indirect irq (lvl/src = %d/%d).\n",
               lvl, src);
        return (r);
    }
    return (0);
}

static int __devinit ppcie_irq_init(struct platform_device *dev)
{
#if defined CONFIG_BASE_PALO || defined CONFIG_BASE_SERENO
    return (irq_init(dev, ps_palopcie_lvl, ps_palopcie_src,
                    ppcie_intr0, DRV_NAME "-ind"));
#elif defined CONFIG_BASE_CRUZ
    struct ppcie_intrs_ent *pie;
    int lvl = ps_palopcie_lvl;
    int src = ps_palopcie_src;
    int idx;
    int idx_size;
    int i;
    /*
     * only setup the ones assigned to mcpu
     */
    idx = pm_tar_ind_get_req_idx_and_size(0, &idx_size);
    if (idx_size > 0) {
        pie = &ppcie_intrs[idx];
        for (i = 0; i < idx_size; i++, src++, pie++) {
             int r = irq_init(dev, lvl, src, pie->handler, pie->name);
             if (r) {
                  return (r);
             }
        }
    }
    return (0);
#endif /* CONFIG_BASE */
}

int __devinit ppcie_intr_init(struct platform_device *dev)
{
    int r;

    r = ppcie_irq_init(dev);
    return (r);
}
