/*
 * Copyright 2007 Nuova Systems, Inc.  All rights reserved.
 * Copyright 2009 Cisco Systems, Inc.  All rights reserved.
 */
#ident "$Id: ppcie_main.c 144815 2013-09-22 21:59:07Z sborikar $"

#include <linux/device.h>
#include <linux/module.h>
#include <linux/moduleparam.h>
#include <linux/kernel.h>
#include <linux/poll.h>
#include <linux/errno.h>
#include <linux/types.h>
#include <linux/init.h>
#include <linux/fs.h>
#include <linux/platform_device.h>
#include <linux/proc_fs.h>

#include <palo/fw/mcpu/include/palo_conf.h>
#include <palo/fw/mcpu/include/board_conf.h>
#include "palo_defs.h"

#include "pciemgr.h"
#include "pmimpl.h"
#include "pciesvc.h"
#include "ppcie_evt.h"
#include "ppcie.h"
#include "ppcie_pcie.h"
#include "ppcie_host.h"
#include "ps_sys.h"

MODULE_DESCRIPTION(DRV_DESCRIPTION);
MODULE_AUTHOR("Cisco Systems, Inc.");
MODULE_LICENSE("GPL");
MODULE_VERSION(DRV_VERSION);

struct ppcie_device_info ppcie_device_info;
static int pmdrv_major;

extern pcie_hid_stats_t *pcie_hid_stats;

static int ppcie_open(struct inode *inode, struct file *filep)
{
    struct ppcie_device_info *pdi = &ppcie_device_info;
    atomic_inc(&pdi->refcnt);
    return (0);
}

static int ppcie_release(struct inode *inode, struct file *filep)
{
    struct ppcie_device_info *pdi = &ppcie_device_info;
    atomic_dec(&pdi->refcnt);
    return (0);
}

static unsigned int ppcie_poll(struct file *filep, poll_table *wait)
{
    struct ppcie_device_info *pdi = &ppcie_device_info;

    poll_wait(filep, &pdi->waitq, wait);
    if (ppcie_have_events()) {
        return (POLLIN | POLLRDNORM);
    }
    return (0);
}

static struct file_operations ppcie_fops = {
    .owner   = THIS_MODULE,
    .open    = ppcie_open,
    .release = ppcie_release,
    .unlocked_ioctl   = ppcie_ioctl,
    .poll    = ppcie_poll,
};

extern void **trnlog_array;


typedef struct dev_extra_attr {
    struct device_attribute attr;
    unsigned long v;
} dev_extra_attr_t;


#define DEV_EXTRA_ATTR(name, mode, showf, storef, val) \
static dev_extra_attr_t dev_extra_attr_##name = \
      { __ATTR(name, mode, showf, storef), (val) }

static ssize_t
ppcie_sysfs_pcie_trnlog_show(struct device *dev,
                             struct device_attribute *attr, char *buf)

{
    dev_extra_attr_t *extra_attr = (dev_extra_attr_t *)attr;
    return (sprintf(buf, "0x%p\n", trnlog_array[extra_attr->v]));
}

DEV_EXTRA_ATTR(trnlog,  0444, ppcie_sysfs_pcie_trnlog_show, NULL, 0);
#ifdef CONFIG_BASE_CRUZ
DEV_EXTRA_ATTR(trnlog1, 0444, ppcie_sysfs_pcie_trnlog_show, NULL, 1);
DEV_EXTRA_ATTR(trnlog2, 0444, ppcie_sysfs_pcie_trnlog_show, NULL, 2);
DEV_EXTRA_ATTR(trnlog3, 0444, ppcie_sysfs_pcie_trnlog_show, NULL, 3);
DEV_EXTRA_ATTR(trnlog4, 0444, ppcie_sysfs_pcie_trnlog_show, NULL, 4);
#endif

static void __devinit ppcie_trn_sysfs_init(struct platform_device *dev)
{
    if (device_create_file(&dev->dev, &dev_extra_attr_trnlog.attr)) {
        printk(KERN_WARNING "trnlog sysfs create failed\n");
    }
#ifdef CONFIG_BASE_CRUZ
    if (device_create_file(&dev->dev, &dev_extra_attr_trnlog1.attr)) {
        printk(KERN_WARNING "trnlog1 sysfs create failed\n");
    }
    if (device_create_file(&dev->dev, &dev_extra_attr_trnlog2.attr)) {
        printk(KERN_WARNING "trnlog2 sysfs create failed\n");
    }
    if (device_create_file(&dev->dev, &dev_extra_attr_trnlog3.attr)) {
        printk(KERN_WARNING "trnlog3 sysfs create failed\n");
    }
    if (device_create_file(&dev->dev, &dev_extra_attr_trnlog4.attr)) {
        printk(KERN_WARNING "trnlog4 sysfs create failed\n");
    }
#endif
}

#ifdef CONFIG_BASE_CRUZ
struct proc_dir_entry *fcpu_proc_entry_1;
struct proc_dir_entry *fcpu_proc_entry_2;
struct proc_dir_entry *fcpu_proc_entry_3;
struct proc_dir_entry *fcpu_proc_entry_4;
#endif

extern struct pm_cpu_log *pm_cpu_log;

#ifdef CONFIG_BASE_CRUZ
static int
fcpu_proc_read_cmn(char *page, char **start, off_t off,
                   int count, int *eof, void *data, int fcp_no)
{
     unsigned int log_start;
     unsigned int log_end;
     unsigned int n, fn;

     struct pm_cpu_log *pm_fcpu_log = &pm_cpu_log[fcp_no -1];

     if (off >= PM_LOG_SIZE) {
         *eof = 1;
         return (0);
     }

     log_start = pm_fcpu_log->log_start + off;
     log_end = pm_fcpu_log->log_end;
     n = log_end - log_start;

     if (n == 0) { /* no data */
         *eof = 1;
         return (0);
     }

     if ( n > count ) {
         n = count;
     }

     fn =  PM_LOG_SIZE - (log_start & (PM_LOG_SIZE -1));
     if (fn > n) {
         fn = n;
      }

     if (fn) {
         memcpy(page, &pm_fcpu_log->log_buf[log_start & (PM_LOG_SIZE -1 )], fn);
     }

     if (n > fn) {
         memcpy(page + fn, &pm_fcpu_log->log_buf, n - fn);
     }
/*
     if (log_start == pm_fcpu_log->log_start) {
            pm_fcpu_log->log_start = log_start + n;
     }
*/

     *start = (char *)(log_start + n);
     return (n);
}

static int
fcpu_proc_write_cmn(struct file *file, const char *buffer, unsigned long count,
                    void *data, int fcp_no)
{
     struct pm_cpu_log *pm_fcpu_log = &pm_cpu_log[fcp_no -1];

     pm_fcpu_log->log_start = pm_fcpu_log->log_end;
     return (count);
}
#endif /* CONFIG_BASE_CRUZ */

/* fcpu 1 */
#define fcpu_proc_read_dcl(N) \
    static int  fcpu_proc_read_##N(char *page, char **start, off_t off,   \
                            int count, int *eof, void *data)              \
    { return (fcpu_proc_read_cmn(page, start, off, count, eof, data, N ));}

#define fcpu_proc_write_dcl(N) \
    static int fcpu_proc_write_##N(struct file *file, const char *buf,    \
                             unsigned long count, void *data)             \
    { return (fcpu_proc_write_cmn(file, buf, count, data, N )); }

#ifdef CONFIG_BASE_CRUZ
fcpu_proc_read_dcl(1);
fcpu_proc_write_dcl(1);
fcpu_proc_read_dcl(2);
fcpu_proc_write_dcl(2);
fcpu_proc_read_dcl(3);
fcpu_proc_write_dcl(3);
fcpu_proc_read_dcl(4);
fcpu_proc_write_dcl(4);
#endif

const char *fcpu_name[5] = {"mcpu", "fcpu1", "fcpu2", "fcpu3", "fcpu4" };

#define fpu_proc_dcl(N) {\
    fcpu_no = N; \
    fcpu_proc_entry_##N =                                               \
                create_proc_entry(fcpu_name[fcpu_no], 0644, proc_ppcie);\
    if (fcpu_proc_entry_##N) {                                          \
        fcpu_proc_entry_##N->read_proc = fcpu_proc_read_##N;            \
        fcpu_proc_entry_##N->write_proc = fcpu_proc_write_##N;          \
    } else {                                                            \
        printk("Can't create proc entry for %s\n", fcpu_name[fcpu_no]); \
        return;                                                         \
    }                                                                   \
}

struct proc_dir_entry *proc_ppcie;
struct proc_dir_entry *proc_entry_pmhost;
struct proc_dir_entry *proc_entry_stats;
struct proc_dir_entry *proc_entry_pmvnic;
static hostid_t proc_hostid;
static pmvnich_t proc_pmvnich = 0;

static int
proc_read_pmvnic(char *page, char **start, off_t off,
                      int count, int *eof, void *data)
{
    int total;
    int n;
    int iter;
    pmvnic_t *pmvnic = pmvnich_to_pmvnic(proc_pmvnich);

    if (pmvnic == NULL) {
        printk("invalid pmvnich %d\n", proc_pmvnich);
        *eof = 1;
        return (0);
    }

    total = pmvnic_sprintf(proc_pmvnich, page, count);

    iter = 0;
    while (1) {
        n = pmcfg_sprintf(pmvnic->pmcfgh, page + total, count - total, iter);
        if (n == 0) break;
        total += n;
        iter ++;
    }

    iter = 0;
    while (1) {
        n = pmbar_sprintf(proc_pmvnich, page + total, count - total, iter);
        if (n == 0) break;
        total += n;
        iter ++;
    }
    *eof = 1;

    return (total);
}

static int
proc_write_pmvnic(struct file *file, const char *buffer,
                       unsigned long count, void *data)
{
    pmvnich_t pmvnich = (pmvnich_t)simple_strtol(buffer, NULL, 0);
    if (pmvnich_to_pmvnic(pmvnich) == NULL) {
        printk("invalid pmvnich %d\n", pmvnich);
    } else {
        proc_pmvnich = pmvnich;
    }
    return (count);
}



static int
proc_read_pmhost(char *page, char **start, off_t off,
                      int count, int *eof, void *data)
{
    int total;
    u_int8_t bus;
    hid_t proc_hid;
    shid_t proc_shid;

    bus = (u_int8_t)off;

    proc_hid = ppcie_hostid_hid(proc_hostid);
    proc_shid = ppcie_hostid_shid(proc_hostid);
    total = ps_hid_sprintf(proc_hid,  proc_shid, page, count, bus);

    if (total == 0) {
        *eof = 0;
    } else {
        *(int *)start = 1;
    }

    return (total);
}

static int
proc_write_pmhost(struct file *file, const char *buffer,
                       unsigned long count, void *data)
{
    u_int16_t h = (u_int16_t)simple_strtol(buffer, NULL, 0);
    if (h >= PALO_MAX_HOSTS) {
        printk("invalid hostid %d\n", h);
    } else {
        proc_hostid = h;
    }
    return (count);
}

char *stats_names[] = {
        "cfg_rd",
        "cfg_wr",
        "bar_rd",
        "bar_wr",

        "cfg_unaligned_rd",
        "cfg_unaligned_wr",
        "cmt_miss",
        "bmt_miss",
        "bmt_miss_retries",
        "bmt_oor",
        "pmv",
        "unhandled_cause",
        "unexpected_reqidx",
        "req_excess_stall",
        "ltssm_st_changed"
};

#define CNT_STATS_NAME  (sizeof (stats_names) / sizeof (char *))
extern pm_impl_t *pmimpl_vaddr;

static int
proc_print_pcie_stats_err(char *page, int count, char *name, int nth)
{
    int name_printed = 0;
    int n;
    int i, j;
    u_int64_t ll, *lp;
    int cnt;
    int total = 0;

    cnt = 0;
    for (i = 0; i < PALO_MAX_HIDS; i++) {
        ll = 0;
        for (j = 0; j < PALO_MAX_ECPU; j++) {
            /* note: assume pcie_hid_stats is array of u_int64_t */
            lp = (u_int64_t *)&pmimpl_vaddr->pcie_hid_stats[j][i];
            ll += *(lp + nth);
        }
        if (ll) {
            if (name_printed == 0) {
                 n = snprintf(page+total, count-total, "%16s:", name);
                 total += n;
                name_printed = 1;
            }
            if (cnt && (cnt % 4) == 0) {
                 n = snprintf(page+total, count-total, "\n%16s ", " ");
                 total += n;
            }
            n = snprintf(page+total, count-total, " hid%d:(%lld)", i, ll);
            total += n;
            cnt++;
        }
    } 
    if (name_printed) 
        total += snprintf(page+total, count-total, "\n");
    return total;
}


static int
proc_read_pcie_stats(char *page, char **start, off_t off,
                      int count, int *eof, void *data)
{
    int total = 0;
    int n;
    int i, j;
    pcie_hid_stats_t *hstat;
    u_int64_t cfg_rd, cfg_wr, bar_rd, bar_wr;

    n = snprintf(page+total, count-total, "ecpu %16s %16s %16s %16s\n",
                  stats_names[0], stats_names[1],
                  stats_names[2], stats_names[3]);

    total += n;

    for (j = 0; j < PALO_MAX_ECPU; j++) {
        cfg_rd = 0;
        cfg_wr = 0;
        bar_rd = 0;
        bar_wr = 0;

        for (i = 0; i < PALO_MAX_HIDS; i++) {
            hstat = &pmimpl_vaddr->pcie_hid_stats[j][i];
            cfg_rd +=  hstat->cfg_rd;
            cfg_wr +=  hstat->cfg_wr;
            bar_rd +=  hstat->bar_rd;
            bar_wr +=  hstat->bar_wr;
        }
        n = snprintf(page+total, count-total, 
                     "%4d %16lld %16lld %16lld %16lld\n",
                     j, cfg_rd, cfg_wr, bar_rd, bar_wr);
        total += n;
    }


    for (i = 4; i < CNT_STATS_NAME; i++) {
        n = proc_print_pcie_stats_err(page + total, count-total, 
                                      stats_names[i], i);
        if (n > 0) total += n;
    }
    return (total);
}

static int
proc_write_pcie_stats(struct file *file, const char *buffer,
                       unsigned long count, void *data)
{
    memset(pcie_hid_stats, 0 , sizeof (*pcie_hid_stats));
    return (count);
}


static void __devinit ppcie_procs_init(void)
{
#ifdef CONFIG_BASE_CRUZ
    int fcpu_no;
#endif
    proc_ppcie = proc_mkdir("ppcie", NULL);

#ifdef CONFIG_BASE_CRUZ
    fpu_proc_dcl(1);
    fpu_proc_dcl(2);
    fpu_proc_dcl(3);
    fpu_proc_dcl(4);
#endif


    proc_entry_pmvnic = create_proc_entry("pmvnic", 0644, proc_ppcie);
    if (proc_entry_pmvnic) {
        proc_entry_pmvnic->read_proc = proc_read_pmvnic;
        proc_entry_pmvnic->write_proc = proc_write_pmvnic;
    } else {
        printk("Can't create proc entry pmvnic\n");
    }

    proc_entry_pmhost = create_proc_entry("host", 0644, proc_ppcie);
    if (proc_entry_pmhost) {
        proc_entry_pmhost->read_proc = proc_read_pmhost;
        proc_entry_pmhost->write_proc = proc_write_pmhost;
    } else {
        printk("Can't create proc entry pmhost\n");
    }

    proc_entry_stats = create_proc_entry("stats", 0644, proc_ppcie);
    if (proc_entry_stats) {
        proc_entry_stats->read_proc = proc_read_pcie_stats;
        proc_entry_stats->write_proc = proc_write_pcie_stats;
    } else {
        printk("Can't create proc entry stats\n");
    }
}


static int __devinit palopcie_probe(struct platform_device *dev)
{
    int r;
    struct ps_init_args ps_init_args;

    r  = register_chrdev(pmdrv_major, DRV_NAME, &ppcie_fops);
    if (r < 0) {
        return (r);
    }
    pmdrv_major = r;

    printk(KERN_INFO PFX "pmpcie major %d\n", pmdrv_major);

    memset(&ps_init_args, 0, sizeof (ps_init_args));

    ppcie_evt_init();
    ppcie_host_init();

    ps_init_args.do_init = 1; /* do pm area initialization */
    r = ps_init(&ps_init_args);
    if (r < 0) {
        printk(KERN_INFO PFX "ps_init failed with err 0x%x !!!\n", r);
        goto error_out;
    }

    r = ppcie_pcie_init(dev);
    if (r < 0)
        goto error_out;

    r = ppcie_intr_init(dev);
    if (r < 0)
        goto error_out;

    ppcie_trn_sysfs_init(dev);
    ppcie_procs_init();

    return (0);

error_out:
    printk("%s: error out !!!\n", __func__);
    unregister_chrdev(pmdrv_major, DRV_NAME);
    return (r);
}

static int __devinit palopcie_remove(struct platform_device *dev)
{
    /* not loadable */
    return (0);
}

static struct platform_driver palopcie_driver = {
    .driver = {
        .name = DRV_NAME,
        .owner = THIS_MODULE,
    },
    .probe = palopcie_probe,
    .remove = palopcie_remove,
};

static int __init palopcie_init(void)
{
    printk(KERN_INFO PFX "%s, ver %s\n", DRV_DESCRIPTION, DRV_VERSION);
    printk(KERN_INFO PFX "%s\n", DRV_COPYRIGHT);

    return (platform_driver_register(&palopcie_driver));
}

static void __exit palopcie_exit(void)
{
    platform_driver_unregister(&palopcie_driver);
}

module_init(palopcie_init);
module_exit(palopcie_exit);
