/*
 * Copyright 2008 Cisco Systems, Inc.  All rights reserved.
 * Copyright 2007 Nuova Systems, Inc.  All rights reserved.
 *
 * This program is free software; you may redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; version 2 of the License.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS
 * BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN
 * ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
 * CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 *
 */
#ident "$Id: vnic_dev.h 21 2016-12-02 21:25:59Z buildsa $"

#ifndef _VNIC_DEV_H_
#define _VNIC_DEV_H_

#include "vnic_resource.h"
#include "vnic_devcmd.h"

#ifndef VNIC_PADDR_TARGET
#define VNIC_PADDR_TARGET	0x0000000000000000ULL
#endif

#ifndef readq
static inline u64 readq(void __iomem *reg)
{
	return (((u64)readl(reg + 0x4UL) << 32) |
		(u64)readl(reg));
}

static inline void writeq(u64 val, void __iomem *reg)
{
	writel(val & 0xffffffff, reg);
	writel(val >> 32, reg + 0x4UL);
}
#endif

enum vnicky_dev_hw_version {
	VNIC_DEV_HW_VER_UNKNOWN,
	VNIC_DEV_HW_VER_A1,
	VNIC_DEV_HW_VER_A2,
};

enum vnicky_dev_intr_mode {
	VNIC_DEV_INTR_MODE_UNKNOWN,
	VNIC_DEV_INTR_MODE_INTX,
	VNIC_DEV_INTR_MODE_MSI,
	VNIC_DEV_INTR_MODE_MSIX,
};

struct vnicky_dev_bar {
	void __iomem *vaddr;
	dma_addr_t bus_addr;
	unsigned long len;
};

struct vnicky_dev_ring {
	void *descs;
	size_t size;
	dma_addr_t base_addr;
	size_t base_align;
	void *descs_unaligned;
	size_t size_unaligned;
	dma_addr_t base_addr_unaligned;
	unsigned int desc_size;
	unsigned int desc_count;
	unsigned int desc_avail;
};

struct vnicky_dev;
struct vnicky_stats;

void *vnicky_dev_priv(struct vnicky_dev *vdev);
unsigned int vnicky_dev_get_res_count(struct vnicky_dev *vdev,
	enum vnicky_res_type type);
void __iomem *vnicky_dev_get_res(struct vnicky_dev *vdev, enum vnicky_res_type type,
	unsigned int index);
dma_addr_t vnicky_dev_get_res_bus_addr(struct vnicky_dev *vdev,
	enum vnicky_res_type type, unsigned int index);
unsigned int vnicky_dev_desc_ring_size(struct vnicky_dev_ring *ring,
	unsigned int desc_count, unsigned int desc_size);
void vnicky_dev_clear_desc_ring(struct vnicky_dev_ring *ring);
int vnicky_dev_alloc_desc_ring(struct vnicky_dev *vdev, struct vnicky_dev_ring *ring,
	unsigned int desc_count, unsigned int desc_size);
void vnicky_dev_free_desc_ring(struct vnicky_dev *vdev,
	struct vnicky_dev_ring *ring);
int vnicky_dev_cmd(struct vnicky_dev *vdev, enum vnicky_devcmd_cmd cmd,
	u64 *a0, u64 *a1, int wait);
int vnicky_dev_fw_info(struct vnicky_dev *vdev,
	struct vnicky_devcmd_fw_info **fw_info);
int vnicky_dev_hw_version(struct vnicky_dev *vdev,
	enum vnicky_dev_hw_version *hw_ver);
int vnicky_dev_spec(struct vnicky_dev *vdev, unsigned int offset, unsigned int size,
	void *value);
int vnicky_dev_stats_clear(struct vnicky_dev *vdev);
int vnicky_dev_stats_dump(struct vnicky_dev *vdev, struct vnicky_stats **stats);
int vnicky_dev_hang_notify(struct vnicky_dev *vdev);
void vnicky_dev_packet_filter(struct vnicky_dev *vdev, int directed, int multicast,
	int broadcast, int promisc, int allmulti);
void vnicky_dev_packet_filter_all(struct vnicky_dev *vdev, int directed,
	int multicast, int broadcast, int promisc, int allmulti);
void vnicky_dev_add_addr(struct vnicky_dev *vdev, u8 *addr);
void vnicky_dev_del_addr(struct vnicky_dev *vdev, u8 *addr);
int vnicky_dev_mac_addr(struct vnicky_dev *vdev, u8 *mac_addr);
int vnicky_dev_raise_intr(struct vnicky_dev *vdev, u16 intr);
int vnicky_dev_notify_setcmd(struct vnicky_dev *vdev,
	void *notify_addr, dma_addr_t notify_pa, u16 intr);
int vnicky_dev_notify_set(struct vnicky_dev *vdev, u16 intr);
void vnicky_dev_notify_unsetcmd(struct vnicky_dev *vdev);
void vnicky_dev_notify_unset(struct vnicky_dev *vdev);
int vnicky_dev_link_status(struct vnicky_dev *vdev);
u32 vnicky_dev_port_speed(struct vnicky_dev *vdev);
u32 vnicky_dev_msg_lvl(struct vnicky_dev *vdev);
u32 vnicky_dev_mtu(struct vnicky_dev *vdev);
u32 vnicky_dev_link_down_cnt(struct vnicky_dev *vdev);
u32 vnicky_dev_notify_status(struct vnicky_dev *vdev);
int vnicky_dev_close(struct vnicky_dev *vdev);
int vnicky_dev_enable(struct vnicky_dev *vdev);
int vnicky_dev_disable(struct vnicky_dev *vdev);
int vnicky_dev_open(struct vnicky_dev *vdev, int arg);
int vnicky_dev_open_done(struct vnicky_dev *vdev, int *done);
int vnicky_dev_init(struct vnicky_dev *vdev, int arg);
int vnicky_dev_soft_reset(struct vnicky_dev *vdev, int arg);
int vnicky_dev_soft_reset_done(struct vnicky_dev *vdev, int *done);
void vnicky_dev_set_intr_mode(struct vnicky_dev *vdev,
	enum vnicky_dev_intr_mode intr_mode);
enum vnicky_dev_intr_mode vnicky_dev_get_intr_mode(struct vnicky_dev *vdev);
void vnicky_dev_unregister(struct vnicky_dev *vdev);
struct vnicky_dev *vnicky_dev_register(struct vnicky_dev *vdev,
	void *priv, struct pci_dev *pdev, struct vnicky_dev_bar *bar,
	unsigned int num_bars);
int vnicky_dev_init_done(struct vnicky_dev *vdev, int *done);
int vnicky_dev_get_size(void);
struct pci_dev *vnicky_dev_get_pdev(struct vnicky_dev *vdev);
int vnicky_dev_int13(struct vnicky_dev *vdev, u64 arg, u32 op);
int vnicky_dev_perbi(struct vnicky_dev *vdev, u64 arg, u32 op);
u32 vnicky_dev_perbi_rebuild_cnt(struct vnicky_dev *vdev);

#endif /* _VNIC_DEV_H_ */
