/*
 * SCSI library functions depending on DMA
 */

#include <linux/blkdev.h>
#include <linux/device.h>
#include <linux/export.h>
#include <linux/kernel.h>

#include <scsi/scsi.h>
#include <scsi/scsi_cmnd.h>
#include <scsi/scsi_device.h>
#include <scsi/scsi_host.h>
#ifdef CONFIG_BASE_CRUZ
int (*f_scsi_dma_map)(struct scsi_cmnd *cmd);
int (*f_scsi_dma_unmap)(struct scsi_cmnd *cmd);
int (*f_local_post_scsi_dma_map)(struct scsi_cmnd *cmd);
int (*f_local_pre_scsi_dma_unmap)(struct scsi_cmnd *cmd);

void scsi_dma_map_redirect(int (*nf_scsi_dma_map)(struct scsi_cmnd *cmd),
			   int (*nf_scsi_dma_unmap)(struct scsi_cmnd *cmd),
                           int (*nf_post_scsi_dma_map)(struct scsi_cmnd *cmd),
			   int (*nf_pre_scsi_dma_unmap)(struct scsi_cmnd *cmd)) 
{
	f_scsi_dma_map = nf_scsi_dma_map;
	f_scsi_dma_unmap = nf_scsi_dma_unmap;

	f_local_post_scsi_dma_map = nf_post_scsi_dma_map;
	f_local_pre_scsi_dma_unmap = nf_pre_scsi_dma_unmap;
}
EXPORT_SYMBOL(scsi_dma_map_redirect);
#endif
/**
 * scsi_dma_map - perform DMA mapping against command's sg lists
 * @cmd:	scsi command
 *
 * Returns the number of sg lists actually used, zero if the sg lists
 * is NULL, or -ENOMEM if the mapping failed.
 */
int scsi_dma_map(struct scsi_cmnd *cmd)
{
	int nseg = 0;
#ifdef CONFIG_BASE_CRUZ
	if (f_scsi_dma_map) {
		nseg = (*f_scsi_dma_map)(cmd);
		if (nseg != -1) {
			return nseg;
		}
	}

#endif
	if (scsi_sg_count(cmd)) {
		struct device *dev = cmd->device->host->dma_dev;

		nseg = dma_map_sg(dev, scsi_sglist(cmd), scsi_sg_count(cmd),
				  cmd->sc_data_direction);
		if (unlikely(!nseg))
			return -ENOMEM;
	}
#ifdef CONFIG_BASE_CRUZ
	if (f_local_post_scsi_dma_map) {
		 (*f_local_post_scsi_dma_map)(cmd);
        }
#endif
	return nseg;
}
EXPORT_SYMBOL(scsi_dma_map);

/**
 * scsi_dma_unmap - unmap command's sg lists mapped by scsi_dma_map
 * @cmd:	scsi command
 */
void scsi_dma_unmap(struct scsi_cmnd *cmd)
{
#ifdef CONFIG_BASE_CRUZ
	int nseg;

	if (f_scsi_dma_unmap) {
		nseg = (*f_scsi_dma_unmap)(cmd);
		if (nseg != -1) {
			return; 
		}
	}

	if (f_local_pre_scsi_dma_unmap) {
		(*f_local_pre_scsi_dma_unmap)(cmd);
	}
#endif
	if (scsi_sg_count(cmd)) {
		struct device *dev = cmd->device->host->dma_dev;

		dma_unmap_sg(dev, scsi_sglist(cmd), scsi_sg_count(cmd),
			     cmd->sc_data_direction);
	}
}
EXPORT_SYMBOL(scsi_dma_unmap);
