/*
 * Copyright 2007 Nuova Systems, Inc.  All rights reserved.
 */
#ident "$Id: hwecpu_addr.h 6 2016-12-02 03:26:55Z buildsa $"

#ifndef _HWECPU_ADDR_H_
#define _HWECPU_ADDR_H_

//
// A "local" address is a normal memory address from 0..memsize.
//
// A "hwecpu" address is an address with a high bit set indicating
// that the address is targeted to the ecpu memory space.  Hardware
// will strip off the high bit before sending it to memory.
//

#define _HWECPU_ADDR_MASK       (0x3ULL << 47)
#define _HWECPU_ADDR_ECPU       (0x2ULL << 47)

static inline u_int8_t
is_hwecpu(u_int64_t addr)
{
    return ((addr & _HWECPU_ADDR_MASK) == _HWECPU_ADDR_ECPU);
}

static inline u_int64_t
local_to_hwecpu(u_int64_t addr)
{
    return ((addr & ~_HWECPU_ADDR_MASK) | _HWECPU_ADDR_ECPU);
}

static inline u_int64_t
hwecpu_to_local(u_int64_t addr)
{
    // We want to mask off bit 47, with something like this:
    //    return (addr & ~_HWECPU_ADDR_MASK);
    // If the addr has come from reading a hardware register the
    // hardware might have replicated bit 48 through bit 63.  To convert
    // to a local memory address we simply want to clear the upper 32 bits.
    // Cast to 32 bits and the upper 32 bits are set to 0, preserving
    // the lower 32 bits.
    return ((u_int32_t)addr);
}

#endif // _HWECPU_ADDR_H_
