/*
 * Copyright 2009 Cisco Systems, Inc.  All rights reserved.
 */
#ident "$Id: board_conf.h 46 2016-12-03 23:17:24Z buildsa $"

#ifndef _BOARD_CONF_H_
#define _BOARD_CONF_H_

/*
 * The Board Id  is stored in flash in newer boards, so the order must
 * be maintained (only append to the end of the list).
 * Listed here via #define rather than enum as these values may be used
 * in assembly code.
 */
#define BOARD_CONF_BRD_UNKNOWN          0
#define BOARD_CONF_BRD_PALITO           1
#define BOARD_CONF_BRD_PALOTE           2
#define BOARD_CONF_BRD_SIM              3
#define BOARD_CONF_BRD_BAYLANDS         4
#define BOARD_CONF_BRD_PALOALTO         5
#define BOARD_CONF_BRD_MONTEREYPARK     6
#define BOARD_CONF_BRD_SERENOTE         7
#define BOARD_CONF_BRD_GREENWOOD        8
#define BOARD_CONF_BRD_VASONA           9
#define BOARD_CONF_BRD_COTATI           10
#define BOARD_CONF_BRD_LEXINGTON        11
#define BOARD_CONF_BRD_CRUZITA          12
#define BOARD_CONF_BRD_CALIMESA         13
#define BOARD_CONF_BRD_KIRKWOODLAKE     14
#define BOARD_CONF_BRD_ICEHOUSE         15
#define BOARD_CONF_BRD_ICEHOUSE2        16
#define BOARD_CONF_BRD_OAKLAND          17
#define BOARD_CONF_BRD_SUSANVILLE       18
#define BOARD_CONF_BRD_CALIMESAB        19
#define BOARD_CONF_BRD_MONTCLAIR        20
#define BOARD_CONF_BRD_CALISTOGA        21
#define BOARD_CONF_BRD_CLEARLAKE        22
#define BOARD_CONF_BRD_WALNUTCREEK      23
#define BOARD_CONF_BRD_MOUNTAINVIEW     24
#define BOARD_CONF_BRD_TORRANCE         25
#define BOARD_CONF_BRD_MOUNTTIAN        26
#define BOARD_CONF_BRD_MOORPARK1        27
#define BOARD_CONF_BRD_MOORPARK2        28
#define BOARD_CONF_BRD_BODEGUITA        29
#define BOARD_CONF_BRD_MOUNTTIAN2       30
#define BOARD_CONF_BRD_CLAREMONT        31
#define BOARD_CONF_BRD_MOORPARK_LITE    32
#define BOARD_CONF_BRD_BRADBURY         33
#define BOARD_CONF_BRD_BRENTWOOD        34
#define BOARD_CONF_BRD_BURLINGAME       35
#define BOARD_CONF_BRD_BAYSIDE          36
#define BOARD_CONF_BRD_BAKERSFIELD      37
#define BOARD_CONF_BRD_BOONVILLE        38

#ifndef __ASSEMBLY__

void __board_conf_init(void);
int __board_conf_ioctl(u_int32_t cmd, void *arg);

typedef struct {
    u_int16_t cartridge_id;
    u_int16_t instance_id;
    u_int8_t  host_link_id;
} hostid_map_t;

typedef struct {
    u_int32_t  nhostid_maps;
    hostid_map_t *maps;
} hostid_map_hdr_t;

struct __board_conf_info {
    u_int8_t    init;                   // init done flag
    u_int8_t    brd_type;               // board type
    char        brd_type_name[16];      // pretty name (e.g. "Palo Alto"
    char        brd_type_path[16];      // pathname (e.g. "paloalto"
    char        adapter_name[16];       // adapter name (e.g. "UCS P81E"
    u_int8_t    adaptercompat;          // nictool adapter compat class
    u_int8_t    adaptertype;            // nictool adapter type
    u_int32_t   memtop;                 // top of memory (incl. aliases)
    int (*ioctl)(struct __board_conf_info *bi, u_int32_t cmd, void *arg);
    u_int16_t   gpio_init_ddr;          // gpio init direction (1=out)
    u_int16_t   gpio_init_val;          // gpio init output data
    u_int16_t   gpio_boot_ddr;          // boot gpio init direction (1=out)
    u_int16_t   gpio_boot_val;          // boot gpio init output data
    u_int8_t    has_i2c;                // has i2c devices
    u_int8_t    has_i2c0reset;          // has i2c bus 0 reset ability
    u_int8_t    i2c0reset_pin;          // gpio for i2c bus 0 reset
    u_int8_t    do_9541_workaround;     // perform the NPX9541 workaround
    u_int8_t    use_alt_post_target;    // use alternate POST EEPROM address
    u_int8_t    card_address;           // i2c card address
    u_int8_t    card_address_valid;     // card address is valid
    u_int8_t    card_address_pin;       // card address GPIO
    u_int8_t    is_40g_valid;           // is_40g GPIO value valid
    u_int8_t    is_40g_mode;            // is cotati in 40g mode
    u_int8_t    has_statusled;          // has status LED
    u_int8_t    statusled_pin;          // LED GPIO
    u_int8_t    has_faultled;           // has fault LED
    u_int8_t    faultled_pin;           // LED GPIO
    u_int8_t    has_phyreset;           // cruz GPIO has PHY RESET
    u_int8_t    phyreset_pin;           // PHY RESET GPIO PIN
    u_int8_t    has_mfgdiag;            // has manufacturing diag GPIO
    u_int8_t    mfgdiag_pin;            // manufacturing diag GPIO
    u_int8_t    has_armflashreset;      // has flash reset arm capability
    u_int8_t    is_lowpower;            // non-zero if in low power mode
    u_int8_t    has_bmcphy;             // board includes a BMC phy
    u_int8_t    bmc_phyaddr;            // BMC PHY address
    u_int8_t    has_mdiomux;            // non-zero if mdio mux fitted
    u_int8_t    has_therm;              // non-zero if thermal monitor fitted
    u_int8_t    run_thermmon;           // non-zero if run thermal monitor
    u_int8_t    has_pex;                // non-zero if PEX8632 switch fitted
    u_int8_t    therm_i2c_port;         // id of i2c port (0 or 1)
    u_int8_t    is_therm_protected;     // non-zero if is behind arbiter
    u_int8_t    therm_type;             // Thermal monitor type
    u_int32_t   resetarm_mask;          // mask for SPARE PIN reset
    u_int32_t   resetarm_val;           // arm value for SPARE PIN reset
    u_int32_t   fwcompat;               // firmware compatibility flags
    u_int16_t   subsys_id;              // pci subsystem id
    u_int8_t    fru_cardtype;           // fru card type
    u_int16_t   fru_datasize;           // fru data size
    /*
     * Should not combine has_retimer and retimer_cnt because has_retimer
     * is decided based on the part number for Vasona and Cotati. For these two
     * card types, retimer_cnt would represent the number of retimers in
     * the version in which retimers are present
     */
    u_int8_t    has_retimer;            // FOR Sereno based boards with retimer
    u_int8_t    retimer_cnt;            // Retimer count
    u_int8_t    cisco_card_mode;        // Indicates not to use pcie_reset_ok
    u_int16_t   bmc_lifid;              // LIFID to assign to the BMC
    u_int8_t    is_pcie_card;           // Card is for rack server
    u_int8_t    has_blinker;            // Indicates if the card has blinker
    u_int8_t    reset_usercfg;          // create user.cfg if missing
    u_int8_t    poll_xcvr;              // During poll cycle, check xcvr status
    u_int8_t    has_rc;                 // Root Complex is present on the board
    /*
     * Supports multiple speeds and hence supports configuration of speed
     */
    u_int8_t    supported_speed_cfg;    // Supported speed configuration bitmap
    char        pcie_linkwidths[128];   // host pcie linkwidths config
    u_int8_t    pcie_poll_compliance;   // poll/reset pcie compliance state
    /*
     * mapping hostid used by palopcie driver to palo_host/pci_link as
     * specified through config/AMP.
     * Assumptions made about the entries:
     *  1. Cartridges have homogenous instances
     *  2. hostid is the index in the array pointed by maps.
     *  3. No duplicate array entries.
     *  4. The hierarchy of fields : cartridge_id -> instance_id ->
     *     host_link_id. It is assumed the list is ordered in increasing order
     *     with this hierarchy.
     */
    hostid_map_hdr_t hostid_map;
    u_int8_t    is_netflow_supp;        // is netflow supported on the board
    u_int8_t    has_krtools;            // support kreye/txfir cmds.
    u_int8_t    is_multihost;
    /*
     * Reset with host can be handled either in sw or in hw
     * On sereno, reset_with_host is handled in hw. On Cruz
     * reset_with_host is handled by sw upon detecting macdn
     * event.
     */
    u_int8_t    reset_with_host;   // reset with host pcie events

    /*
     * Sereno and Cruz asic have TSO sequence number corruption issue
     * when server is power cycled while running TSO traffic.
     * netblock reset is handled in sw to workaround the TSO issue that
     * cause corrupt sequence numbers.
     */
    u_int8_t net_rst_with_host;
};

enum {
    BOARD_CONF_THERM_MAX6698,
    BOARD_CONF_THERM_MAX6654,
    BOARD_CONF_THERM_AD7416,
};

#define BOARD_CONF_IS(brd)   (board_conf_brd_type() == BOARD_CONF_BRD_ ## brd)

#if defined(CONFIG_BASE_PALO)
#define PLO_BOARD_CONF_IS(board)   BOARD_CONF_IS(board)
#define SRO_BOARD_CONF_IS(board)   (0)
#define CRZ_BOARD_CONF_IS(board)   (0)

#elif defined(CONFIG_BASE_SERENO)
#define PLO_BOARD_CONF_IS(board)   (0)
#define SRO_BOARD_CONF_IS(board)   BOARD_CONF_IS(board)
#define CRZ_BOARD_CONF_IS(board)   (0)

#elif defined(CONFIG_BASE_CRUZ)
#define PLO_BOARD_CONF_IS(board)   (0)
#define SRO_BOARD_CONF_IS(board)   (0)
#define CRZ_BOARD_CONF_IS(board)   BOARD_CONF_IS(board)

#endif

#define board_conf_is_palito()        PLO_BOARD_CONF_IS(PALITO)
#define board_conf_is_palote()        PLO_BOARD_CONF_IS(PALOTE)
#define board_conf_is_baylands()      PLO_BOARD_CONF_IS(BAYLANDS)
#define board_conf_is_paloalto()      PLO_BOARD_CONF_IS(PALOALTO)
#define board_conf_is_montereypark()  PLO_BOARD_CONF_IS(MONTEREYPARK)

#define board_conf_is_serenote()      SRO_BOARD_CONF_IS(SERENOTE)
#define board_conf_is_greenwood()     SRO_BOARD_CONF_IS(GREENWOOD)
#define board_conf_is_vasona()        SRO_BOARD_CONF_IS(VASONA)
#define board_conf_is_cotati()        SRO_BOARD_CONF_IS(COTATI)
#define board_conf_is_lexington()     SRO_BOARD_CONF_IS(LEXINGTON)
#define board_conf_is_kirkwoodlake()  SRO_BOARD_CONF_IS(KIRKWOODLAKE)
#define board_conf_is_icehouse()      SRO_BOARD_CONF_IS(ICEHOUSE)
#define board_conf_is_icehouse2()     SRO_BOARD_CONF_IS(ICEHOUSE2)
#define board_conf_is_oakland()       SRO_BOARD_CONF_IS(OAKLAND)
#define board_conf_is_walnutcreek()   SRO_BOARD_CONF_IS(WALNUTCREEK)
#define board_conf_is_susanville()    SRO_BOARD_CONF_IS(SUSANVILLE)
#define board_conf_is_torrance()      SRO_BOARD_CONF_IS(TORRANCE)

#define board_conf_is_cruzita()       CRZ_BOARD_CONF_IS(CRUZITA)
#define board_conf_is_calimesa()      CRZ_BOARD_CONF_IS(CALIMESA)
#define board_conf_is_calimesab()     CRZ_BOARD_CONF_IS(CALIMESAB)
#define board_conf_is_montclair()     CRZ_BOARD_CONF_IS(MONTCLAIR)
#define board_conf_is_calistoga()     CRZ_BOARD_CONF_IS(CALISTOGA)
#define board_conf_is_clearlake()     CRZ_BOARD_CONF_IS(CLEARLAKE)
#define board_conf_is_mountainview()  CRZ_BOARD_CONF_IS(MOUNTAINVIEW)
#define board_conf_is_mounttian()     CRZ_BOARD_CONF_IS(MOUNTTIAN)
#define board_conf_is_mounttian2()    CRZ_BOARD_CONF_IS(MOUNTTIAN2)
#define board_conf_is_claremont()     CRZ_BOARD_CONF_IS(CLAREMONT)

extern struct __board_conf_info __board_conf_info;

#define board_conf_is_speed_supported(val) \
    (__board_conf_info.supported_speed_cfg & (1 << val))

#define board_conf_rate_limit_min() 0

#define board_conf_rate_limit_max() \
    ((board_conf_is_speed_supported(PALO_UPLINK_SPEED_4x10G) || \
      board_conf_is_speed_supported(PALO_UPLINK_SPEED_40G))? \
      (40000 * 1000) : (10000 * 1000))

/*
 * Static functions.
 */
#define board_conf_decl_type(field, T)   \
static inline T                          \
board_conf_##field(void)                 \
{                                        \
    __board_conf_init();                 \
    return (__board_conf_info.field);    \
}

#define board_conf_decl_ptr_type(field, T)   \
static inline T *                            \
board_conf_##field(void)                     \
{                                            \
    __board_conf_init();                     \
    return (&__board_conf_info.field);       \
}
#define board_conf_decl(field)   board_conf_decl_type(field, u_int32_t)

board_conf_decl(brd_type)
board_conf_decl_type(brd_type_name, char *)
board_conf_decl_type(brd_type_path, char *)
board_conf_decl_type(adapter_name, char *)
board_conf_decl(has_i2c)
board_conf_decl(has_faultled)
board_conf_decl(has_statusled)
board_conf_decl(has_i2c0reset)
board_conf_decl(has_mfgdiag)
board_conf_decl(has_armflashreset)
board_conf_decl(subsys_id)
board_conf_decl(fwcompat)
board_conf_decl(has_bmcphy)
board_conf_decl(bmc_phyaddr)
board_conf_decl(has_mdiomux)
board_conf_decl(has_therm)
board_conf_decl(run_thermmon)
board_conf_decl(therm_type)
board_conf_decl(do_9541_workaround)
board_conf_decl(use_alt_post_target)
board_conf_decl(therm_i2c_port)
board_conf_decl(is_therm_protected)
board_conf_decl(adaptercompat)
board_conf_decl(adaptertype)
board_conf_decl(memtop)
board_conf_decl(has_pex)
board_conf_decl(is_pcie_card)
board_conf_decl(has_rc)
board_conf_decl_type(has_blinker, u_int8_t)
board_conf_decl_type(reset_usercfg, u_int8_t)
board_conf_decl_type(retimer_cnt, u_int8_t)
board_conf_decl_type(poll_xcvr, u_int8_t)
board_conf_decl_type(bmc_lifid, u_int16_t)
board_conf_decl_type(pcie_linkwidths, char *)
board_conf_decl(pcie_poll_compliance)
board_conf_decl_ptr_type(hostid_map, hostid_map_hdr_t)
board_conf_decl(has_phyreset)
board_conf_decl(phyreset_pin)
board_conf_decl(is_netflow_supp)
board_conf_decl(has_krtools)
board_conf_decl(is_multihost)
board_conf_decl(reset_with_host)
board_conf_decl(net_rst_with_host)

/*
 * Dynamic Functions.
 */
enum {
    BOARDCMD_INIT,                      // init gpios (only for u-boot/kernel)
    BOARDCMD_GET_CARD_ADDRESS,          // get i2c card address
    BOARDCMD_SET_STATUSLED,             // set status led
    BOARDCMD_SET_FAULTLED,              // set fault led
    BOARDCMD_SET_PHYRESET,              // set phy reset pin
    BOARDCMD_BEGIN_I2C0RESET,           // begin i2c reset sequence
    BOARDCMD_SET_I2C0RESET,             // set i2c reset pin
    BOARDCMD_END_I2C0RESET,             // end i2c reset sequence
    BOARDCMD_GET_MFGDIAG,               // get mfg diag pin
    BOARDCMD_ARM_FLASHRESET,            // arm spare-pin reset
    BOARDCMD_SET_MDIOMUXBMC,            // point mdio mux towards bmc
    BOARDCMD_FRU_DATASIZE,              // get fru data size
    BOARDCMD_FRU_INIT,                  // initialize fru
    BOARDCMD_FRU_SHOW,                  // print fru contents
    BOARDCMD_FRU_VERIFY,                // verify fru checksums
    BOARDCMD_IS_40G_MODE,               // with passthrough cotati gives 40g
    BOARDCMD_CHECK_RETIMER_PRESENCE,    // check for the prsence of a retimer
    BOARDCMD_PWR_INTR_INIT,             // initial the power intr GPIO
    BOARDCMD_PWR_INTR_CHG_POL,          // change pol of level triggered intr
    BOARDCMD_IS_LOWPOWER,               // Provides dynamic power state
    BOARDCMD_CISCO_CARD_MODE,           // Dynamic BMC_RST_BYPASS status
};

/*
 * Functions that do stuff with no argument
 */
#define board_conf_cmd_decl(name, sym)          \
static inline void                              \
board_conf_##name(void)                         \
{                                               \
    __board_conf_ioctl(BOARDCMD_##sym, NULL);   \
}
#ifdef __KERNEL__
board_conf_cmd_decl(init, INIT)
#endif
board_conf_cmd_decl(begin_i2c0reset, BEGIN_I2C0RESET)
board_conf_cmd_decl(end_i2c0reset, END_I2C0RESET)
board_conf_cmd_decl(arm_flashreset, ARM_FLASHRESET)

/*
 * Functions taking an argument.
 */
#define board_conf_cmd_decl_type(name, sym, T)         \
static inline int                                      \
board_conf_##name(T val)                               \
{                                                      \
    return __board_conf_ioctl(BOARDCMD_##sym, &val);   \
}
board_conf_cmd_decl_type(set_statusled, SET_STATUSLED, int)
board_conf_cmd_decl_type(set_faultled, SET_FAULTLED, int)
board_conf_cmd_decl_type(set_phyreset, SET_PHYRESET, int)
board_conf_cmd_decl_type(set_i2c0reset, SET_I2C0RESET, int)
board_conf_cmd_decl_type(set_mdiomux_bmc, SET_MDIOMUXBMC, int)
board_conf_cmd_decl_type(fru_init, FRU_INIT, void *)
board_conf_cmd_decl_type(fru_show, FRU_SHOW, void *)
board_conf_cmd_decl_type(fru_verify, FRU_VERIFY, void *)

/*
 * Functions returning dynamic data.
 */
#define board_conf_get_decl_type(name, sym, T)  \
static inline T                                 \
board_conf_##name(void)                         \
{                                               \
    T val = 0;                                  \
    __board_conf_ioctl(BOARDCMD_##sym, &val);   \
    return val;                                 \
}
board_conf_get_decl_type(mfgdiag, GET_MFGDIAG, int)
board_conf_get_decl_type(card_address, GET_CARD_ADDRESS, int)
board_conf_get_decl_type(fru_datasize, FRU_DATASIZE, int)
board_conf_get_decl_type(is_40g_mode, IS_40G_MODE, int)
board_conf_get_decl_type(has_retimer, CHECK_RETIMER_PRESENCE, int)
board_conf_get_decl_type(pwr_intr_init, PWR_INTR_INIT, int)
board_conf_get_decl_type(pwr_intr_chg_pol, PWR_INTR_CHG_POL, int)
board_conf_get_decl_type(is_lowpower, IS_LOWPOWER, int)
board_conf_get_decl_type(cisco_card_mode, CISCO_CARD_MODE, int)

#endif /* __ASSEMBLY__ */

#endif /* __BOARD_CONF_H__ */
