/*
 * Copyright (c) 2007 Nuova Systems, Inc.  All rights reserved.
 */
#ident "$Id: palo_conf.h 46 2016-12-03 23:17:24Z buildsa $"

#ifndef __PALO_CONF_H__
#define __PALO_CONF_H__

#ifdef __KERNEL__
#include <palo/include/misc_regs.h>
#else
#include "misc_regs.h"
#endif

void __palo_conf_init(void);

struct __palo_conf_info {
    u_int32_t init;
    u_int32_t em_type;
    u_int32_t version;
    char *    basename;
    u_int32_t cpu_clk;
#ifdef __KERNEL__
    u_int32_t core_clk;
    u_int32_t uart_clk;
    u_int32_t uart_irq;
#endif // __KERNEL__
    u_int32_t netcore_clk;
    u_int32_t vnic_count;
    u_int32_t lif_count;
    u_int32_t wq_count;
    u_int32_t rq_count;
    u_int32_t cq_count;
    u_int32_t intr_count;
    u_int32_t intr_grp_count;
    u_int32_t tcam_count;
#ifdef CONFIG_BASE_CRUZ
    u_int32_t vif_tbl_count;
#endif
    u_int32_t uif_count;
    u_int32_t uifport_count;
    u_int32_t ecpu_count;
    u_int32_t tx_rx_stats;
    u_int32_t has_iommu;
    u_int32_t eg_pfifo_size;
    u_int32_t ig_pfifo_size;
    u_int32_t loop_pfifo_size; // Total loopback pfifo buffers in both uif
    u_int32_t pfifo_cos_unit_size;
    u_int32_t is_asic;
    u_int32_t is_cir_supported;
#ifdef CONFIG_BASE_PALO
    u_int32_t is_palo;
    u_int32_t is_palito;
    u_int32_t is_palote;
#elif defined CONFIG_BASE_SERENO
    u_int32_t is_sereno;
    u_int32_t is_serenote;
    u_int32_t memsize;
#endif /* CONFIG_BASE */
};

extern struct __palo_conf_info __palo_conf_info;

#define palo_conf_decl_type(field, T)   \
static inline T                         \
palo_conf_##field(void)                 \
{                                       \
    if (!__palo_conf_info.init) {       \
        __palo_conf_init();             \
    }                                   \
    return (__palo_conf_info.field);    \
}

#define palo_conf_decl(field)   palo_conf_decl_type(field, u_int32_t)

palo_conf_decl(em_type);
palo_conf_decl(version);
palo_conf_decl_type(basename, char *);
palo_conf_decl(cpu_clk);
#ifdef __KERNEL__
palo_conf_decl(core_clk);
palo_conf_decl(uart_clk);
palo_conf_decl(uart_irq);
#endif // __KERNEL__
palo_conf_decl(netcore_clk);
palo_conf_decl(vnic_count);
palo_conf_decl(lif_count);
palo_conf_decl(wq_count);
palo_conf_decl(rq_count);
palo_conf_decl(cq_count);
palo_conf_decl(intr_count);
palo_conf_decl(intr_grp_count);
palo_conf_decl(tcam_count);
#ifdef CONFIG_BASE_CRUZ
palo_conf_decl(vif_tbl_count);
#endif
palo_conf_decl(uif_count);
palo_conf_decl(uifport_count);
palo_conf_decl(ecpu_count);
palo_conf_decl(tx_rx_stats);
palo_conf_decl(has_iommu);
palo_conf_decl(ig_pfifo_size);
palo_conf_decl(eg_pfifo_size);
palo_conf_decl(loop_pfifo_size);
palo_conf_decl(pfifo_cos_unit_size);
palo_conf_decl(is_asic);
palo_conf_decl(is_cir_supported);
#ifdef CONFIG_BASE_PALO
palo_conf_decl(is_palo);
palo_conf_decl(is_palito);
palo_conf_decl(is_palote);
#elif defined CONFIG_BASE_SERENO
palo_conf_decl(is_sereno);
palo_conf_decl(is_serenote);
palo_conf_decl(memsize);
#endif /* CONFIG_BASE */

#ifdef MISC_EMULATION_SIM
// There's a simulation type for some base (compiled) types
#define palo_conf_is_sim()    (palo_conf_em_type() == MISC_EMULATION_SIM)
#else /* MISC_EMULATION_SIM */
#define palo_conf_is_sim()    (0)
#endif /* MISC_EMULATION_SIM */

#endif // __PALO_CONF_H__
