
/*
 * Copyright (c) 2007 Nuova Systems, Inc.  All rights reserved.
 */
#ident "$Id: palofwimg.h 43 2016-12-03 22:59:19Z buildsa $"

#ifndef _PALOFWIMG_H_
#define _PALOFWIMG_H_

/*
 * PALO Firmware Image header.
 * The header is prepended to all image files.
 * When installed, the header is kept for firmware and diag images, but
 * is stripped from the front of the boot block.  A second copy of the
 * header is embedded within the boot block at offset PALOFW_BOOTHDROFFSET.
 */

#define PALOFW_IMGMAGIC         0x253d0452      // application firmware
#define PALOFW_DIAGMAGIC        0x63452421      // manufacturing diags firmware
#define PALOFW_BOOTMAGIC        0x5746424d      // boot block

#define PALOFW_BUILDLEN         32
#define PALOFW_VERSLEN          32

#ifndef __ASSEMBLY__

#ifndef PACKED
#define PACKED __attribute__((packed))
#endif

typedef struct {
    u_int8_t    magic;                  // magic (0 - for compatibility)
    u_int8_t    vers;                   // format version
    u_int32_t   compat;                 // target compatibility flags
    u_int8_t    pad[26];                // pad to 32-bytes (compatibility)
} PACKED fwimginfo_t;

#define PALOFW_INFO_MAGIC       0
#define PALOFW_INFO_VERS        1

#define PALOFW_COMPAT_BAYLANDS      (1 << 0)
#define PALOFW_COMPAT_PALOALTO      (1 << 1)
#define PALOFW_COMPAT_MONTEREYPARK  (1 << 2)
#define PALOFW_COMPAT_GREENWOOD     (1 << 3)
#define PALOFW_COMPAT_VASONA        (1 << 4)
#define PALOFW_COMPAT_COTATI        (1 << 5)
#define PALOFW_COMPAT_LEXINGTON     (1 << 6)
#define PALOFW_COMPAT_CALIMESA      (1 << 7)
#define PALOFW_COMPAT_SEASIDE       (1 << 8)
#define PALOFW_COMPAT_KIRKWOODLAKE  (1 << 9)
#define PALOFW_COMPAT_ICEHOUSE      (1 << 10)
#define PALOFW_COMPAT_OAKLAND       (1 << 11)
#define PALOFW_COMPAT_MONTCLAIR     (1 << 12)
#define PALOFW_COMPAT_CALISTOGA     (1 << 13)
#define PALOFW_COMPAT_CLEARLAKE     (1 << 14)
#define PALOFW_COMPAT_ICEHOUSE2     (1 << 15)
#define PALOFW_COMPAT_SUSANVILLE    (1 << 16)
#define PALOFW_COMPAT_WALNUTCREEK   (1 << 17)
#define PALOFW_COMPAT_MOUNTAINVIEW  (1 << 18)
#define PALOFW_COMPAT_MOUNTTIAN     (1 << 19)
#define PALOFW_COMPAT_TORRANCE      (1 << 20)
#define PALOFW_COMPAT_MOUNTTIAN2    (1 << 21)
#define PALOFW_COMPAT_CLAREMONT     (1 << 22)

typedef struct {
    u_int32_t   magic;                          // magic (BOOT or IMG)
    u_int32_t   hcrc;                           // header crc
    u_int32_t   size;                           // payload size
    u_int32_t   load;                           // load address
    u_int32_t   ep;                             // entry point
    u_int32_t   dcrc;                           // payload data ctc
    u_int32_t   flags;                          // flags
    u_int8_t    build[PALOFW_BUILDLEN];         // build string
    fwimginfo_t info;                           // image info
    u_int8_t    vers[PALOFW_VERSLEN];           // version string
} PACKED fwimghdr_t;

#endif

#define PALOFWIMGF_GZIPPED      0x1             // data is gzipped

/*
 * Boot block image offset.
 */
#define PALOFW_BOOTHDROFFSET    0x800
#define PALOFW_BOOTHDRSPARE     0x100

#endif
