/*
 * Copyright 2007 Nuova Systems, Inc.  All rights reserved.
 * Copyright 2011 Cisco Systems, Inc.  All rights reserved.
 */
#ident "$Id: palo_conf.c 46 2016-12-03 23:17:24Z buildsa $"

#ifdef __KERNEL__

#include <linux/kernel.h>
#include <linux/module.h>
#include <linux/delay.h>
#include <asm/io.h>
#include <palo.h>
#include <paloint.h>
#include <palo/include/palo_impl.h>
#include <palo/include/fpga_mregs.h>
#include "regio.h"
#include <palo/fw/mcpu/include/palo_conf.h>
#include <palo/fw/mcpu/include/board_conf.h>

#else // __KERNEL__

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "misc_regs.h"
#include "fpga_mregs.h"

#include "regio.h"
#include "palo_conf.h"
#include "board_conf.h"
#include "palo_impl.h"

#endif // __KERNEL__

struct __palo_conf_info __palo_conf_info;
#ifdef __KERNEL__
EXPORT_SYMBOL_GPL(__palo_conf_info);
#endif


#if defined CONFIG_BASE_PALO || defined CONFIG_BASE_SERENO
static void
get_net_cfg_fpga(u_int16_t *eg_buf_depth, u_int16_t *ig_buf_depth,
  u_int8_t *tcam_size, u_int8_t *tx_rx_stats, u_int8_t *acl_size,
  u_int8_t *uif_count)
{
    u_int32_t net_cfg_fpga[2];

    readregs(NET_MREGS_CFG_FPGA, net_cfg_fpga, 2);

    *uif_count = net_cfg_fpga[0] & 0xf;
    *acl_size = (net_cfg_fpga[0] >> 4) & 0xf;
    *tx_rx_stats = (net_cfg_fpga[0] >> 8) & 0x1;
    *tcam_size = (net_cfg_fpga[0] >> 12) & 0xff;
    *ig_buf_depth = ((net_cfg_fpga[0] >> 20) & 0xfff) |
      ((net_cfg_fpga[1] & 0xf) << 12);
    *eg_buf_depth = (net_cfg_fpga[1] >> 4) & 0xffff;
}
#endif /* CONFIG_BASE */

#ifdef __KERNEL__
#if defined CONFIG_BASE_SERENO

/*
 * Sereno B can use the ASIC core clock for the UART.
 * See also similar version of this in mcpu/u-boot/board/palo/palo.c
 */
static void
palo_use_core_clk_for_uart(void)
{
    u_int32_t val;

    val = readreg(MISC_MREGS_UART_CLK_SEL);
    if (val) {
        // already set up in u-boot, nothing to do
        return;
    }
    writereg(MISC_MREGS_UART_CLK_SEL, 1);

    val = readreg(MISC_MREGS_SW_HARD_RST);
    val |= MISC_MREGS_SW_HARD_RST_APB | MISC_MREGS_SW_HARD_RST_UART;
    writereg(MISC_MREGS_SW_HARD_RST, val);
    udelay(1);  // wait for new clock signal to propagate
    val &= ~(MISC_MREGS_SW_HARD_RST_APB | MISC_MREGS_SW_HARD_RST_UART);
    writereg(MISC_MREGS_SW_HARD_RST, val);
    udelay(1);  // wait for reset signal to exit expander
}

#endif /* CONFIG_BASE_SERENO */
#endif /* __KERNEL__ */

void
__palo_conf_init()
{
    struct __palo_conf_info *ci = &__palo_conf_info;
#if defined CONFIG_BASE_PALO || defined CONFIG_BASE_SERENO
    u_int16_t eg_buf_depth, ig_buf_depth;
    u_int8_t tcam_size, tx_rx_stats, acl_size, uif_count;
#endif /* CONFIG_BASE */

    if (ci->init) {
        return;
    }
    ci->init = 1;

#if !defined(__KERNEL__) && !defined(PALO_FW)
    regio_init();
#endif // __KERNEL__
    ci->em_type = readreg(MISC_MREGS_EMULATION);
    ci->version = readreg(MISC_MREGS_VERSION);
    switch (ci->em_type) {

#if defined CONFIG_BASE_PALO

    case MISC_EMULATION_PALITO:
        get_net_cfg_fpga(&eg_buf_depth, &ig_buf_depth,
          &tcam_size, &tx_rx_stats, &acl_size, &uif_count);
        ci->basename = "Palo";
        ci->cpu_clk = PALO_FPGA_CPU_CLK;
#ifdef __KERNEL__
        ci->uart_clk = PALO_FPGA_UART_CLK;
        ci->uart_irq = PALO_SERIAL1_IRQ;
#endif // __KERNEL__
        ci->vnic_count = 128;
        ci->lif_count = 128;
        ci->wq_count = 64;
        ci->rq_count = 64;
        ci->cq_count = 64;
        ci->intr_count = 128;
        ci->tcam_count = tcam_size;
        ci->uif_count = uif_count;
        ci->uifport_count = 1;
        ci->ecpu_count = 1;
        ci->tx_rx_stats = tx_rx_stats;
        ci->has_iommu = 0;
        ci->ig_pfifo_size = 64 * 1024;
        ci->eg_pfifo_size = 64 * 1024;
        ci->pfifo_cos_unit_size = 32;
        ci->is_palito = 1;
        ci->is_cir_supported = 0;
        break;

    case MISC_EMULATION_PALOTE:
        get_net_cfg_fpga(&eg_buf_depth, &ig_buf_depth,
          &tcam_size, &tx_rx_stats, &acl_size, &uif_count);
        ci->basename = "Palo";
        ci->cpu_clk = PALO_FPGA_CPU_CLK;
#ifdef __KERNEL__
        ci->uart_clk = PALO_FPGA_UART_CLK;
        ci->uart_irq = PALO_SERIAL1_IRQ;
#endif // __KERNEL__
        ci->vnic_count = 128;
        ci->lif_count = 128;
        ci->wq_count = 1024;
        ci->rq_count = 1024;
        ci->cq_count = 1024;
        ci->intr_count = 128;
        ci->tcam_count = tcam_size;
        ci->uif_count = uif_count;
        ci->uifport_count = 1;
        ci->ecpu_count = 2;
        ci->tx_rx_stats = tx_rx_stats;
        ci->has_iommu = 1;
        ci->ig_pfifo_size = 128 * 1024;
        ci->eg_pfifo_size = 64 * 1024;
        ci->pfifo_cos_unit_size = 32;
        ci->is_palote = 1;
        ci->is_cir_supported = 0;
        break;

    case MISC_EMULATION_SIM:
        ci->basename = "Palo";
        ci->cpu_clk = PALO_SIM_CPU_CLK;
#ifdef __KERNEL__
        ci->uart_clk = PALO_SIM_UART_CLK;
        ci->uart_irq = MIPS_CPU_IRQ_BASE + 2;
#endif // __KERNEL__
        ci->vnic_count = 128;
        ci->lif_count = 128;
        ci->wq_count = 1024;
        ci->rq_count = 1024;
        ci->cq_count = 1024;
        ci->intr_count = 1536;
        ci->tcam_count = 1024;
        ci->uif_count = 2;
        ci->uifport_count = 1;
        ci->ecpu_count = 3;
        ci->tx_rx_stats = 1;
        ci->has_iommu = 0;
        ci->ig_pfifo_size = 1024 * 1024;
        ci->eg_pfifo_size = 256 * 1024;
        ci->pfifo_cos_unit_size = 32;
        ci->is_cir_supported = 0;
        break;

    default: // PALO ASIC
        ci->basename = "Palo";
        ci->cpu_clk = PALO_ASIC_CPU_CLK;
#ifdef __KERNEL__
        ci->uart_clk = PALO_ASIC_UART_CLK;
        ci->uart_irq = PALO_SERIAL1_IRQ;
#endif // __KERNEL__
        ci->vnic_count = 128;
        ci->lif_count = 128;
        ci->wq_count = 1024;
        ci->rq_count = 1024;
        ci->cq_count = 1024;
        ci->intr_count = 1536;
        ci->tcam_count = 1024;
        ci->uif_count = 2;
        ci->uifport_count = 1;
        ci->ecpu_count = 3;
        ci->tx_rx_stats = 1;
        ci->has_iommu = 1;
        ci->ig_pfifo_size = 1024 * 1024;
        ci->eg_pfifo_size = 256 * 1024;
        // Currently not configuring PALO loopback registers
        ci->loop_pfifo_size = 0;
        ci->pfifo_cos_unit_size = 32;
        ci->is_asic = 1;
        ci->is_palo = 1;
        ci->is_cir_supported = 0;
        break;

#elif defined CONFIG_BASE_SERENO

    case MISC_EMULATION_SERENOTE:
        get_net_cfg_fpga(&eg_buf_depth, &ig_buf_depth,
                        &tcam_size, &tx_rx_stats, &acl_size, &uif_count);
        ci->basename = "Sereno";
        ci->cpu_clk = PALO_FPGA_CPU_CLK;
#ifdef __KERNEL__
        ci->uart_clk = PALO_FPGA_UART_CLK;
        ci->uart_irq = PALO_SERIAL1_IRQ;
#endif // __KERNEL__
        ci->netcore_clk = PALO_FPGA_NETCORE_CLK;
        ci->vnic_count = 32;
        ci->lif_count = 128;  // Actually 256; just being cautious
        ci->wq_count = 64;
        ci->rq_count = 64;
        ci->cq_count = 64;
        ci->intr_count = 128;
        ci->intr_grp_count = 32;
        ci->tcam_count = 32;
        ci->uif_count = uif_count;
        ci->uifport_count = 4;
        ci->ecpu_count = 2;
        ci->tx_rx_stats = 1;
        ci->has_iommu = 0;
        ci->ig_pfifo_size = 128 * 1024;
        ci->eg_pfifo_size = 64 * 1024;
        ci->pfifo_cos_unit_size = 64;
        ci->is_serenote = 1;
        ci->is_cir_supported = 1;
        break;

    default: // Sereno asic
        ci->basename = "Sereno";
        ci->cpu_clk = PALO_ASIC_CPU_CLK;
#ifdef __KERNEL__
        ci->core_clk = PALO_ASIC_CORE_CLK;
        ci->uart_irq = PALO_SERIAL1_IRQ;
        if (ci->version > 0xa0) {
            palo_use_core_clk_for_uart();
            ci->uart_clk = ci->core_clk;
        } else {
            ci->uart_clk = PALO_ASIC_UART_CLK;
        }
#endif // __KERNEL__
        ci->netcore_clk = PALO_ASIC_NETCORE_CLK;
        ci->vnic_count = 256;
        ci->lif_count = 256;
        ci->wq_count = 1024;
        ci->rq_count = 1024;
        ci->cq_count = 1024;
        ci->intr_count = 1536;
        ci->intr_grp_count = 32;
        ci->tcam_count = 1024;
        ci->uif_count = 2;
        ci->uifport_count = 4;
        ci->ecpu_count = 3;
        ci->tx_rx_stats = 1;
        ci->has_iommu = 0;
        ci->ig_pfifo_size = 1024 * 1024;
        ci->eg_pfifo_size = 256 * 1024;
        ci->loop_pfifo_size = 128 * 1024;
        ci->pfifo_cos_unit_size = 64;
        ci->is_asic = 1;
        ci->is_sereno = 1;
        ci->is_cir_supported = 1;
        break;

#elif defined CONFIG_BASE_CRUZ

    case MISC_EMULATION_CRUZITA:
        ci->basename = "Cruz";
        ci->cpu_clk = PALO_FPGA_CPU_CLK;
#ifdef __KERNEL__
        ci->uart_clk = PALO_FPGA_UART_CLK;
        ci->uart_irq = PALO_SERIAL1_IRQ;
#endif // __KERNEL__
        ci->netcore_clk = PALO_FPGA_NETCORE_CLK;
        ci->vnic_count = 128;
        ci->lif_count = 128;
        ci->wq_count = 128;
        ci->rq_count = 128;
        ci->cq_count = 128;
        ci->intr_count = 192;
        ci->tcam_count = 128;
        ci->uif_count = 2;
        ci->uifport_count = 4;
        ci->vif_tbl_count = 1024; // per uif
        ci->ecpu_count = 5;
        ci->tx_rx_stats = 1;
        ci->ig_pfifo_size = 256 * 1024;
        ci->eg_pfifo_size = 128 * 1024;
        ci->pfifo_cos_unit_size = 64;
        break;

    default: // Cruz asic
        ci->basename = "Cruz";
        ci->cpu_clk = PALO_ASIC_CPU_CLK;
#ifdef __KERNEL__
        ci->core_clk = PALO_ASIC_CORE_CLK;
        ci->uart_irq = PALO_SERIAL1_IRQ;
        ci->uart_clk = PALO_ASIC_UART_CLK;
#endif // __KERNEL__
        ci->netcore_clk = PALO_ASIC_NETCORE_CLK;
        ci->vnic_count = 2048;
        ci->lif_count = 2048;
        ci->wq_count = 8192;
        ci->rq_count = 8192;
        ci->cq_count = 8192;
        ci->intr_count = 12288;
        ci->intr_grp_count = 256;
        ci->tcam_count = 8192;
        ci->vif_tbl_count = 16384; // per uif
        ci->uif_count = 2;
        ci->uifport_count = 4;
        ci->ecpu_count = 5;
        ci->tx_rx_stats = 1;
        ci->ig_pfifo_size = 1024 * 1024;
        ci->eg_pfifo_size = 512 * 1024;
        ci->loop_pfifo_size = 256 * 1024;
        ci->pfifo_cos_unit_size = 64;
        ci->is_asic = 1;
        break;
#endif /* CONFIG_BASE */
    }
}

#ifdef __KERNEL__
EXPORT_SYMBOL_GPL(__palo_conf_init);
#endif
