/*
 * Copyright 2011 Cisco Systems, Inc. All Rights Reserved
 */
#ident "$Id: pp_regs_defs.h 43 2016-12-03 22:59:19Z buildsa $"

#ifndef _CRUZ_PP_REGS_DEFS_H_
#define _CRUZ_PP_REGS_DEFS_H_
#ifndef __ASSEMBLY__

typedef enum pps_linkwidth_e {
    PPS_LINKWIDTH_x8 = 0x0,
    PPS_LINKWIDTH_x4 = 0x1,
    PPS_LINKWIDTH_x2 = 0x2,
} pps_linkwidth_t;

typedef enum umt_entry_e {
    UMTENTRY_CMT = 0x0,
    UMTENTRY_BMT = 0x1
} umt_entry_t;

// UMT search key_type field
typedef enum umt_key_e {
    UMTKEY_RSVD  = 0x0,
    UMTKEY_MEM   = 0x1,
    UMTKEY_IO    = 0x2,
    UMTKEY_CMT   = 0x3
} umt_key_t;

typedef enum urt_type_e {
    URTTYPE_CRT = 0,
    URTTYPE_BRT = 1
} urt_type_t;

//                                             1         2         3
//                                   01234567890123456789012345678901
//            1         2         3          4         5         6
//  01234567890123456789012345678901 23456789012345678901234567890123
// +--------------------------------+--------------------------------+
// |iiiiirwRWbbbbbbbbbbbbbbbbbbbbbbb|bbbbbbbbMMMMMMMMMMmmmmmmmmmm..tv|
// +--------------------------------+--------------------------------+
static inline void
urt_crt_entry_make(u_int32_t *reg,
                   const u_int8_t vld,
                   const u_int16_t reg_min,
                   const u_int16_t reg_max,
                   const u_int32_t res_base,
                   const u_int8_t int_wr,
                   const u_int8_t int_rd,
                   const u_int8_t int_idx,
                   const u_int8_t perm_wr,
                   const u_int8_t perm_rd)
{
    const u_int32_t res_base_dw = res_base >> 2; 
    const u_int16_t reg_min_dw = reg_min >> 2;
    const u_int16_t reg_max_dw = reg_max >> 2;
    const u_int8_t urt_type = URTTYPE_CRT;

    reg[0] = ((int_idx & 0x1f) |
              ((perm_rd & 0x1) << 5) |
              ((perm_wr & 0x1) << 6) |
              ((int_rd & 0x1) << 7) |
              ((int_wr & 0x1) << 8) |
              ((res_base_dw & 0x7fffff) << 9));
    reg[1] = (((res_base_dw >> 23) & 0xff) |
              ((reg_max_dw & 0x3ff) << 8) |
              ((reg_min_dw & 0x3ff) << 18) |
              ((urt_type & 0x1) << 30) |
              ((vld & 0x1) << 31));
    reg[2] = 0;
}

static inline void
urt_crt_entry_get(u_int32_t *reg,
                  u_int8_t *vld,
                  u_int16_t *reg_min,
                  u_int16_t *reg_max,
                  u_int32_t *res_base,
                  u_int8_t *int_wr,
                  u_int8_t *int_rd,
                  u_int8_t *int_idx,
                  u_int8_t *perm_wr,
                  u_int8_t *perm_rd)
{
    const u_int8_t urt_type = (reg[1] >> 30) & 0x1;
    u_int32_t res_base_dw;
    u_int16_t reg_min_dw;
    u_int16_t reg_max_dw;

    if (urt_type == URTTYPE_CRT) {
        *int_idx = (reg[0] >> 0) & 0x1f;
        *perm_rd = (reg[0] >> 5) & 0x1;
        *perm_wr = (reg[0] >> 6) & 0x1;
        *int_rd = (reg[0] >> 7) & 0x1;
        *int_wr = (reg[0] >> 8) & 0x1;
        res_base_dw = (((reg[1] & 0xff) << 23) |
                       ((reg[0] >> 9) & 0x7fffff));
        *res_base = res_base_dw << 2;
        reg_max_dw = (reg[1] >> 8) & 0x3ff;
        *reg_max = reg_max_dw << 2;
        reg_min_dw = (reg[1] >> 18) & 0x3ff;
        *reg_min = reg_min_dw << 2;
    } else {
        *int_idx = 0;
        *perm_rd = 0;
        *perm_wr = 0;
        *int_rd = 0;
        *int_wr = 0;
        *res_base = 0;
        *reg_max = 0;
        *reg_min = 0;
    }
    // always return vld from entry, even if mis-marked
    *vld = (reg[1] >> 31) & 0x1;
}

//
// Get the valid bit and entry type.
// Once we know the type we can fetch the details.
//
static inline void
urt_entry_get(u_int32_t *reg,
              u_int8_t *vld,
              u_int8_t *urt_type)
{
    *vld = (reg[1] >> 31) & 0x1;
    *urt_type = (reg[1] >> 30) & 0x1;
}

//                                             1         2         3
//                                   01234567890123456789012345678901
//            1         2         3          4         5         6
//  01234567890123456789012345678901 23456789012345678901234567890123
// +--------------------------------+--------------------------------+
// |iiiiirwRWvvvvvSsssssssssssbbbbbb|bbbbbbbbbbbbbbbbbbbbbbbbbP....tv|
// +--------------------------------+--------------------------------+
static inline void
urt_brt_entry_make(u_int32_t *reg,
                   const u_int8_t vld,
                   const u_int64_t res_base,
                   const u_int8_t res_size_cfg,
                   const u_int32_t res_size,
                   const u_int8_t vasidx,
                   const u_int8_t pmv_dis,
                   const u_int8_t int_wr,
                   const u_int8_t int_rd,
                   const u_int8_t int_idx,
                   const u_int8_t perm_wr,
                   const u_int8_t perm_rd)
{
    const u_int32_t res_base_dw = res_base >> 2;
    const u_int8_t urt_type = URTTYPE_BRT;

    reg[0] = ((int_idx & 0x1f) |
              ((perm_rd & 0x1) << 5) |
              ((perm_wr & 0x1) << 6) |
              ((int_rd & 0x1) << 7) |
              ((int_wr & 0x1) << 8) |
              ((vasidx & 0x1f) << 9) |
              ((res_size_cfg & 0x1) << 14) |
              ((res_size & 0x7ff) << 15) |
              ((res_base_dw & 0x3f) << 26));
    reg[1] = (((res_base_dw >> 6) & 0x1ffffff) |
              ((pmv_dis & 0x1) << 25) |
              ((urt_type & 0x1) << 30) |
              ((vld & 0x1) << 31));
    reg[2] = 0;
}

static inline void
urt_brt_entry_get(u_int32_t *reg,
                  u_int8_t *vld,
                  u_int32_t *res_base,
                  u_int8_t *res_size_cfg,
                  u_int32_t *res_size,
                  u_int8_t *vasidx,
                  u_int8_t *pmv_dis,
                  u_int8_t *int_wr,
                  u_int8_t *int_rd,
                  u_int8_t *int_idx,
                  u_int8_t *perm_wr,
                  u_int8_t *perm_rd)
{
    const u_int8_t urt_type = (reg[1] >> 30) & 0x1;
    u_int32_t res_base_dw;

    if (urt_type == URTTYPE_BRT) {
        *int_idx = (reg[0] >> 0) & 0x1f;
        *perm_rd = (reg[0] >> 5) & 0x1;
        *perm_wr = (reg[0] >> 6) & 0x1;
        *int_rd = (reg[0] >> 7) & 0x1;
        *int_wr = (reg[0] >> 8) & 0x1;
        *vasidx = (reg[0] >> 9) & 0x1f;
        *res_size_cfg = (reg[0] >> 14) & 0x1;
        *res_size = (reg[0] >> 15) & 0x7ff;
        res_base_dw = (((reg[1] & 0x1ffffff) << 6) |
                       ((reg[0] >> 26) & 0x3f));
        *res_base = res_base_dw << 2;
        *pmv_dis = (reg[1] >> 25) & 0x1;
    } else {
        *int_idx = 0;
        *perm_rd = 0;
        *perm_wr = 0;
        *int_rd = 0;
        *int_wr = 0;
        *vasidx = 0;
        *res_size_cfg = 0;
        *res_size = 0;
        res_base_dw = 0;
        *res_base = 0;
        *pmv_dis = 0;
    }
    // always return vld from entry, even if mis-marked
    *vld = (reg[1] >> 31) & 0x1;
}

//
// CMT Search Key:
//                                             1         2         3
//                                   01234567890123456789012345678901
//            1         2         3          4         5         6
//  01234567890123456789012345678901 23456789012345678901234567890123
// +--------------------------------+--------------------------------+
// |hhhhssfffdddddbbbbbbbbWaaaaaaaaa|a.....................kktttv....|
// +--------------------------------+--------------------------------+
//
static inline void
umt_tcam_cmt_entry_make(u_int64_t *datap,
                        u_int64_t *maskp,
                        const u_int8_t vld,
                        const u_int8_t hid,
                        const u_int8_t shid,
                        const u_int8_t tid,
                        const u_int8_t bus,
                        const u_int8_t dev,
                        const u_int8_t fnc)
{
    u_int32_t data[2], mask[2];
    const u_int8_t mask_hid = 0xf;
    const u_int8_t mask_shid = 0x3;
    const u_int8_t mask_bus = 0xff;
    const u_int8_t mask_dev = 0x1f;
    const u_int8_t mask_fnc = 0x7;
    const u_int8_t data_cfgwr = 0;
    const u_int8_t mask_cfgwr = 0x0;
    const u_int16_t data_cfg_addr = 0;
    const u_int16_t mask_cfg_addr = 0x0;
    const u_int8_t data_key_type = UMTKEY_CMT;
    const u_int8_t mask_key_type = 0x3;
    const u_int8_t data_tid = tid;
    const u_int8_t mask_tid = 0x7;
    const u_int8_t mask_vld = 0x1;

    // Assemble the tcam data
    data[0] = (((hid & 0xf) << 0) |
               ((shid & 0x3) << 4) |
               ((fnc & 0x7) << 6) |
               ((dev & 0x1f) << 9) |
               ((bus & 0xff) << 14) |
               ((data_cfgwr & 0x1) << 22) |
               ((data_cfg_addr & 0x1ff) << 23));
    data[1] = (((data_cfg_addr >> 9) & 0x1) |
               ((data_key_type & 0x3) << 22) |
               ((data_tid & 0x7) << 24) |
               ((vld & 0x1) << 27));
    *datap = ((u_int64_t)data[1] << 32) | data[0];

    // Assemble the tcam mask
    mask[0] = ((mask_hid << 0) |
               (mask_shid << 4) |
               (mask_fnc << 6) |
               (mask_dev << 9) |
               (mask_bus << 14) |
               (mask_cfgwr << 22) |
               ((mask_cfg_addr & 0x1ff) << 23));
    mask[1] = (((mask_cfg_addr >> 9) & 0x1) |
               (mask_key_type << 22) |
               (mask_tid << 24) |
               (mask_vld << 27));
    *maskp = ((u_int64_t)mask[1] << 32) | mask[0];
}

static inline void
umt_tcam_cmt_entry_get(u_int64_t datall,
                       u_int64_t maskll,
                       u_int8_t *vld,
                       u_int8_t *hid,
                       u_int8_t *shid,
                       u_int8_t *tid,
                       u_int8_t *bus,
                       u_int8_t *dev,
                       u_int8_t *fnc)
{
    u_int32_t data[2];

    data[0] = datall & 0xffffffff;
    data[1] = (datall >> 32) & 0xffffffff;

    *hid = (data[0] >> 0) & 0xf;
    *shid = (data[0] >> 4) & 0x3;
    *fnc = (data[0] >> 6) & 0x7;
    *dev = (data[0] >> 9) & 0x1f;
    *bus = (data[0] >> 14) & 0xff;
    *tid = (data[1] >> 24) & 0x7;
    *vld = (data[1] >> 27) & 0x1;
}

//
// Get the valid bit and entry type.
// Once we know the type we can fetch the details.
//
static inline void
umt_tcam_entry_get(u_int64_t datall,
                   u_int64_t maskll,
                   u_int8_t *vld,
                   u_int8_t *umt_type)
{
    u_int32_t data[2];

    data[0] = datall & 0xffffffff;
    data[1] = (datall >> 32) & 0xffffffff;

    *vld = (data[1] >> 27) & 0x1;
    *umt_type = (data[1] >> 22) & 0x3;
}

//
// CMT RAM Entry:
//                                             1         2         3
//                                   01234567890123456789012345678901
//            1         2         3          4         5         6
//  01234567890123456789012345678901 23456789012345678901234567890123
// +--------------------------------+--------------------------------+
// |tcccccCCCCCCCCCCCCCCCCCfffdddddb|bbbbbbbhhhhHHHHwwwwbbbbbbbbfffdd|
// +--------------------------------+--------------------------------+
//            1
//  01234567890123
//        7
//  45678901234567
// +--------------+
// |dddAtveeeeeeee|
// +--------------+
//
static inline void
umt_ram_cmt_entry_make(u_int32_t *ramdata,
                       const u_int8_t vld,
                       const u_int8_t hid,
                       const u_int8_t bus,
                       const u_int8_t dev,
                       const u_int8_t fnc,
                       const u_int32_t urt_base,
                       const u_int16_t urt_cnt,
                       const u_int8_t active)
{
    const u_int8_t tlp = 0;
    const u_int8_t wild = 0;
    const u_int8_t hid_max = 0, bus_max = 0, dev_max = 0, fnc_max = 0;
    const u_int8_t entry_type = UMTENTRY_CMT;
    const u_int8_t tar_active_standby = active;

    // Assemble the associated ram entry
    ramdata[0] = (((tlp & 0x1) << 0) |
                  ((urt_cnt & 0x1f) << 1) |
                  ((urt_base & 0x1ffff) << 6) |
                  ((fnc & 0x7) << 23) |
                  ((dev & 0x1f) << 26) |
                  ((bus & 0x1) << 31));
    ramdata[1] = (((bus >> 1) & 0x7f) |
                  ((hid & 0xf) << 7) |
                  ((hid_max & 0xf) << 11) |
                  ((wild & 0xf) << 15) |
                  ((bus_max & 0xff) << 19) |
                  ((fnc_max & 0x7) << 27) |
                  ((dev_max & 0x3) << 30));
    ramdata[2] = (((dev_max >> 2) & 0x7) |
                  ((tar_active_standby & 0x1) << 3) |
                  ((entry_type & 0x1) << 4) |
                  ((vld & 0x1) << 5));
}

static inline void
umt_ram_cmt_entry_get(u_int32_t *ramdata,
                      u_int8_t *vld,
                      u_int8_t *hid,
                      u_int8_t *bus,
                      u_int8_t *dev,
                      u_int8_t *fnc,
                      u_int32_t *urt_base,
                      u_int16_t *urt_cnt)
{
    const u_int8_t entry_type = (ramdata[2] >> 4) & 0x1;

    if (entry_type == UMTENTRY_CMT) {
        *urt_cnt = (ramdata[0] >> 1) & 0x1f;
        *urt_base = (ramdata[0] >> 6) & 0x1ffff;
        *fnc = (ramdata[0] >> 23) & 0x7;
        *dev = (ramdata[0] >> 26) & 0x1f;
        *bus = (((ramdata[0] >> 31) & 0x1) |
                ((ramdata[1] & 0x7f) << 1));
        *hid = (ramdata[1] >> 7) & 0xf;
    } else {
        // Not marked as a CMT entry.
        *urt_cnt = 0;
        *urt_base = 0;
        *fnc = 0;
        *dev = 0;
        *bus = 0;
        *hid = 0;
    }
    // always return vld from entry, even if mis-marked
    *vld = (ramdata[2] >> 5) & 0x1;
}

//
// BMT Search Key:
//            1         2         3          4         5         6
//  01234567890123456789012345678901 23456789012345678901234567890123
//            1         2         3            1         2         3
//  01234567890123456789012345678901 01234567890123456789012345678901
// +--------------------------------+--------------------------------+
// |hhhhssaaaaaaaaaaaaaaaaaaaaaaaaaa|aaaaaaaaaaaaaaaaaaaa..kktttv....|
// +--------------------------------+--------------------------------+
//
//     6         5         4          3         2         1
//  32109876543210987654321098765432 10987654321098765432109876543210
//   3         2         1            3         2         1
//  10987654321098765432109876543210 10987654321098765432109876543210
// +--------------------------------+--------------------------------+
// |....vtttkk..aaaaaaaaaaaaaaaaaaaa|aaaaaaaaaaaaaaaaaaaaaaaaaasshhhh|
// +--------------------------------+--------------------------------+
//
static inline void
umt_tcam_bmt_entry_make(u_int64_t *datap,
                        u_int64_t *maskp,
                        const u_int8_t vld,
                        const u_int8_t hid,
                        const u_int8_t shid,
                        const u_int8_t tid,
                        const u_int8_t key_type,
                        const u_int64_t bar_addr_dw,
                        const u_int64_t mask_bar_addr_dw)
{
    u_int32_t data[2], mask[2];
    const u_int8_t mask_hid = 0xf;
    const u_int8_t mask_shid = 0x3;
    const u_int8_t mask_key_type = 0x3;
    const u_int8_t data_tid = tid;
    const u_int8_t mask_tid = 0x7;
    const u_int8_t mask_vld = 0x1;

    // Assemble the tcam data
    data[0] = (((hid & 0xf) << 0) |
               ((shid & 0x3) << 4) |
               ((bar_addr_dw & 0x3ffffff) << 6));
    data[1] = (((bar_addr_dw >> 26) & 0xfffff) |
               ((key_type & 0x3) << 22) |
               ((data_tid & 0x7) << 24) |
               ((vld & 0x1) << 27));
    *datap = ((u_int64_t)data[1] << 32) | data[0];

    // Assemble the tcam mask
    mask[0] = (((mask_hid << 0)) |
               ((mask_shid << 4)) |
               ((mask_bar_addr_dw & 0x3ffffff) << 6));
    mask[1] = (((mask_bar_addr_dw >> 26) & 0xfffff) |
               ((mask_key_type & 0x3) << 22) |
               ((mask_tid & 0x7) << 24) |
               ((mask_vld & 0x1) << 27));
    *maskp = ((u_int64_t)mask[1] << 32) | mask[0];
}

static inline void
umt_tcam_bmt_entry_get(u_int64_t datall,
                       u_int64_t maskll,
                       u_int8_t *vld,
                       u_int8_t *hid,
                       u_int8_t *shid,
                       u_int8_t *tid,
                       u_int8_t *bar_type,
                       u_int64_t *bar_addr_dw,
                       u_int64_t *bar_addr_dw_mask)
{
    u_int32_t data[2], mask[2];

    data[0] = datall & 0xffffffff;
    data[1] = (datall >> 32) & 0xffffffff;

    mask[0] = maskll & 0xffffffff;
    mask[1] = (maskll >> 32) & 0xffffffff;

    *hid = data[0] & 0xf;
    *shid = (data[0] >> 4) & 0x3;
    *bar_addr_dw = ((((u_int64_t)data[1] & 0xfffff) << 26) |
                    ((data[0] >> 6) & 0x3ffffff));
    *bar_addr_dw_mask = ((((u_int64_t)mask[1] & 0xfffff) << 26) |
                         ((mask[0] >> 6) & 0x3ffffff));
    *tid = (data[1] >> 24) & 0x7;
    *bar_type = (data[1] >> 22) & 0x3;
    *vld = (data[1] >> 27) & 0x1;
}

//
// BMT RAM Entry:
//                                             1         2         3
//                                   01234567890123456789012345678901
//            1         2         3          4         5         6
//  01234567890123456789012345678901 23456789012345678901234567890123
// +--------------------------------+--------------------------------+
// |tcccccccccccccsssssbbbbbbbbbbbbb|bbbbfffdddddbbbbbbbbSSSSSacccccc|
// +--------------------------------+--------------------------------+
//            1
//  01234567890123
//        7
//  45678901234567
// +--------------+
// |cc..tveeeeeeee|
// +--------------+
//
static inline void
umt_ram_bmt_entry_make(u_int32_t *ramdata,
                       const u_int8_t vld,
                       const u_int8_t bus,
                       const u_int8_t dev,
                       const u_int8_t fnc,
                       const u_int32_t log2_bar_size,
                       const u_int32_t urt_base,
                       const u_int16_t urt_cnt,
                       const u_int32_t log2_urt_size,
                       const u_int8_t crc,
                       const u_int8_t active)
{
    const u_int8_t tlp = 0;
    const u_int8_t entry_type = UMTENTRY_BMT;
    const u_int8_t tar_active_standby = active;

    // Assemble the associated ram entry
    ramdata[0] = (((tlp & 0x1) << 0) |
                  ((urt_cnt & 0xfff) << 1) |
                  ((log2_urt_size & 0x1f) << 14) |
                  ((urt_base & 0x1fff) << 19));
    ramdata[1] = (((urt_base >> 13) & 0xf) |
                  ((fnc & 0x7) << 4) |
                  ((dev & 0x1f) << 7) |
                  ((bus & 0xff) << 12) |
                  ((log2_bar_size & 0x1f) << 20) |
                  ((tar_active_standby & 0x1) << 25) |
                  ((crc & 0x3f) << 26));
    ramdata[2] = (((crc >> 6) & 0x3) |
                  ((entry_type & 0x1) << 4) |
                  ((vld & 0x1) << 5));
}

static inline void
umt_ram_bmt_entry_get(u_int32_t *ramdata,
                      u_int8_t *vld,
                      u_int8_t *bus,
                      u_int8_t *dev,
                      u_int8_t *fnc,
                      u_int32_t *log2_bar_size,
                      u_int32_t *urt_base,
                      u_int16_t *urt_cnt,
                      u_int32_t *log2_urt_size,
                      u_int8_t *crc)
{
    const u_int8_t entry_type = (ramdata[2] >> 4) & 0x1;

    if (entry_type == UMTENTRY_BMT) {
        *urt_cnt = (ramdata[0] >> 1) & 0xfff;
        *log2_urt_size = (ramdata[0] >> 14) & 0x1f;
        *urt_base = (((ramdata[0] >> 19) & 0x1fff) |
                     ((ramdata[1] & 0xf) << 13));
        *fnc = (ramdata[1] >> 4) & 0x7;
        *dev = (ramdata[1] >> 7) & 0x1f;
        *bus = (ramdata[1] >> 12) & 0xff;
        *log2_bar_size = (ramdata[1] >> 20) & 0x1f;
        *crc = (((ramdata[1] >> 26) & 0x3f) |
                ((ramdata[2] & 0x3) << 6));
    } else {
        // entry not marked as BMT
        *urt_cnt = 0;
        *log2_urt_size = 0;
        *urt_base = 0;
        *fnc = 0;
        *dev = 0;
        *bus = 0;
        *log2_bar_size = 0;
        *crc = 0;
    }
    // always return vld from entry, even if mis-marked
    *vld = (ramdata[2] >> 5) & 0x1;
}

#define PP_MREGS_BASE(pp) \
    (PP_MREGS0_BASE_ADDR + \
     ((PP_MREGS1_BASE_ADDR - PP_MREGS0_BASE_ADDR) * (pp)))
#define PP_MREGS_(REG, pp) \
    (PP_MREGS_BASE(pp) + PP_MREGS_ ## REG ## _OFFSET)
#define PP_MREGS_PPS_OFFSET_(REG, pps) \
    (PP_MREGS_PPS0_ ## REG ## _OFFSET + \
     ((PP_MREGS_PPS1_ ## REG - PP_MREGS_PPS0_ ## REG) * (pps)))
#define PP_MREGS_PPS_(REG, pp, pps) \
    (PP_MREGS_BASE(pp) + PP_MREGS_PPS_OFFSET_(REG, pps))

#define PPSC_MREGS_BASE(pps) \
    (PPSC_MREGS0_BASE_ADDR + \
     ((PPSC_MREGS1_BASE_ADDR - PPSC_MREGS0_BASE_ADDR) * (pps)))
#define PPSC_MREGS_OFFSET_(REG, pps) \
    (PPSC_MREGS_PPS0_ ## REG ## _OFFSET + \
     ((PPSC_MREGS_PPS1_ ## REG - PPSC_MREGS_PPS0_ ## REG) * (pps)))
#define PPSC_MREGS_(REG, pp, pps) \
    (PPSC_MREGS_BASE(pp) + PPSC_MREGS_OFFSET_(REG, pps))

#define PPSP_MREGS_BASE(pps) \
    (PPSP_MREGS0_BASE_ADDR + \
     ((PPSP_MREGS1_BASE_ADDR - PPSP_MREGS0_BASE_ADDR) * (pps)))
#define PPSP_MREGS_OFFSET_(REG, pps) \
    (PPSP_MREGS_PPS0_ ## REG ## _OFFSET + \
     ((PPSP_MREGS_PPS1_ ## REG - PPSP_MREGS_PPS0_ ## REG) * (pps)))
#define PPSP_MREGS_(REG, pp, pps) \
    (PPSP_MREGS_BASE(pp) + PPSP_MREGS_OFFSET_(REG, pps))

#endif // __ASSEMBLY__
#endif // _CRUZ_PP_REGS_DEFS_H_
