/*
 * Copyright 2006 Nuova Systems, Inc. All Rights Reserved
 */
#ident "$Id: intr_regs_defs.h 6 2016-12-02 03:26:55Z buildsa $"

#ifndef _INTR_REGS_DEFS_H_
#define _INTR_REGS_DEFS_H_

#define INTRCTRL_HREGS_BASE_ADDR        INTR_HREGS_COALESCING_TIMER
#define INTRCTRL_HREGS_MAXCNT           INTR_HREGS_COALESCING_TIMER_MAXCNT
#define INTRCTRL_HREGS_UNITSZ           INTR_HREGS_COALESCING_TIMER_BLKSZ

#define INTRCTRL_MREGS_BASE_ADDR        INTR_MREGS_LEGACY_PIN
#define INTRCTRL_MREGS_MAXCNT           INTR_MREGS_LEGACY_PIN_MAXCNT
#define INTRCTRL_MREGS_UNITSZ           INTR_MREGS_LEGACY_PIN_BLKSZ

#define INTRMSI_HREGS_BASE_ADDR         INTR_HREGS_MESSAGE_ADDR
#define INTRMSI_HREGS_MAXCNT            INTR_HREGS_MESSAGE_ADDR_MAXCNT
#define INTRMSI_HREGS_UNITSZ            INTR_HREGS_MESSAGE_ADDR_BLKSZ

#define INTRASSERT_MREGS_BASE_ADDR      INTR_MREGS_INTERRUPT_ASSERT
#define INTRASSERT_MREGS_MAXCNT         INTR_MREGS_INTERRUPT_ASSERT_MAXCNT
#define INTRASSERT_MREGS_UNITSZ         INTR_MREGS_INTERRUPT_ASSERT_BLKSZ

#define INTRPBA_HREGS_BASE_ADDR         INTR_HREGS_PBA
#define INTRPBA_HREGS_MAXCNT            1
#define INTRPBA_HREGS_UNITSZ ((INTR_HREGS_PBA_RC - INTR_HREGS_PBA) * 2)

// types for the r2c arg
enum {
    NOR2C = 0,
    R2C = 1,
};

/**
 * intrpba_enc - Encode intr parameters to physical address of
 * interrupt PBA alias.
 *
 * @param addr Pointer to address used as destination of encoding.
 * @param intr Starting intr of the pba region, must be naturally aligned
 * on nintrs boundary.
 * @param nintrs Number of intr bits per word in pba alias.  Must be
 * an even power of 2.
 * @param r2c Select read-to-clear semantics.
 */
static inline void
intrpba_enc(u_int32_t *addr,
            const u_int16_t intr, const u_int16_t nintrs, const u_int8_t r2c)
{
    // expect number of intr bits per word to be power of 2
    //assert((nintrs & (nintrs - 1)) == 0);

    // Starting intr should be naturally aligned on nintrs boundary.
    //assert((intr & (nintrs - 1)) == 0);

#define INTRPBA_TO_ADDR(intr, r2c, PBA, shf) \
    (((r2c) ? INTR_HREGS_## PBA ##_RC : INTR_HREGS_## PBA) + \
     (((intr) >> (shf)) * INTR_HREGS_## PBA ##_BLKSZ))

    switch (nintrs) {
    case 1: *addr = INTRPBA_TO_ADDR(intr, r2c, PBA, 0); break;
    case 2: *addr = INTRPBA_TO_ADDR(intr, r2c, PBA_2BIT, 1); break;
    case 4: *addr = INTRPBA_TO_ADDR(intr, r2c, PBA_4BIT, 2); break;
    case 8: *addr = INTRPBA_TO_ADDR(intr, r2c, PBA_8BIT, 3); break;
    case 16: *addr = INTRPBA_TO_ADDR(intr, r2c, PBA_16BIT, 4); break;
    case 32: *addr = INTRPBA_TO_ADDR(intr, r2c, PBA_32BIT, 5); break;
    case 64: *addr = INTRPBA_TO_ADDR(intr, r2c, PBA_64BIT, 6); break;
    default:
        *addr = 0;
        //assert(0);
        break;
    }
#undef INTRPBA_TO_ADDR
}

/**
 * intrpba_dec - Decode intr parameters to physical address of
 * interrupt PBA alias.
 *
 * @param addr Address to decode.
 * @param intr Starting intr of the pba region.
 * @param nintrs Number of intr bits per word in pba alias.
 * @param r2c Select read-to-clear semantics.
 */
static inline void
intrpba_dec(u_int32_t addr,
            u_int16_t *intr, u_int16_t *nintrs, u_int8_t *r2c)
{
#define ADDR_IN_RANGE(addr, PBA) \
    ((addr) >= INTR_HREGS_## PBA && \
     (addr) < (INTR_HREGS_## PBA + \
               (INTR_HREGS_## PBA ##_BLKSZ * INTR_HREGS_## PBA ##_MAXCNT)))

#define ADDR_TO_INTR(addr, PBA, shf) \
    ((((addr) - INTR_HREGS_##PBA) / INTR_HREGS_## PBA ##_BLKSZ) << (shf))

    if (addr < INTR_HREGS_PBA_RC) { // non-RC range

        if (ADDR_IN_RANGE(addr, PBA)) {
            *intr = ADDR_TO_INTR(addr, PBA, 0);
            *nintrs = 1;
            *r2c = 0;
        } else if (ADDR_IN_RANGE(addr, PBA_2BIT)) {
            *intr = ADDR_TO_INTR(addr, PBA_2BIT, 1);
            *nintrs = 2;
            *r2c = 0;
        } else if (ADDR_IN_RANGE(addr, PBA_4BIT)) {
            *intr = ADDR_TO_INTR(addr, PBA_4BIT, 2);
            *nintrs = 4;
            *r2c = 0;
        } else if (ADDR_IN_RANGE(addr, PBA_8BIT)) {
            *intr = ADDR_TO_INTR(addr, PBA_8BIT, 3);
            *nintrs = 8;
            *r2c = 0;
        } else if (ADDR_IN_RANGE(addr, PBA_16BIT)) {
            *intr = ADDR_TO_INTR(addr, PBA_16BIT, 4);
            *nintrs = 16;
            *r2c = 0;
        } else if (ADDR_IN_RANGE(addr, PBA_32BIT)) {
            *intr = ADDR_TO_INTR(addr, PBA_32BIT, 5);
            *nintrs = 32;
            *r2c = 0;
        } else if (ADDR_IN_RANGE(addr, PBA_64BIT)) {
            *intr = ADDR_TO_INTR(addr, PBA_64BIT, 6);
            *nintrs = 64;
            *r2c = 0;
        } else {
            // addr out of PBA range?
            *r2c = 0;
            *nintrs = 0;
            *intr = 0;
            //assert(0);
        }

    } else { // RC range

        if (ADDR_IN_RANGE(addr, PBA_RC)) {
            *intr = ADDR_TO_INTR(addr, PBA_RC, 0);
            *nintrs = 1;
            *r2c = 1;
        } else if (ADDR_IN_RANGE(addr, PBA_2BIT_RC)) {
            *intr = ADDR_TO_INTR(addr, PBA_2BIT_RC, 1);
            *nintrs = 2;
            *r2c = 1;
        } else if (ADDR_IN_RANGE(addr, PBA_4BIT_RC)) {
            *intr = ADDR_TO_INTR(addr, PBA_4BIT_RC, 2);
            *nintrs = 4;
            *r2c = 1;
        } else if (ADDR_IN_RANGE(addr, PBA_8BIT_RC)) {
            *intr = ADDR_TO_INTR(addr, PBA_8BIT_RC, 3);
            *nintrs = 8;
            *r2c = 1;
        } else if (ADDR_IN_RANGE(addr, PBA_16BIT_RC)) {
            *intr = ADDR_TO_INTR(addr, PBA_16BIT_RC, 4);
            *nintrs = 16;
            *r2c = 1;
        } else if (ADDR_IN_RANGE(addr, PBA_32BIT_RC)) {
            *intr = ADDR_TO_INTR(addr, PBA_32BIT_RC, 5);
            *nintrs = 32;
            *r2c = 1;
        } else if (ADDR_IN_RANGE(addr, PBA_64BIT_RC)) {
            *intr = ADDR_TO_INTR(addr, PBA_64BIT_RC, 6);
            *nintrs = 64;
            *r2c = 1;
        } else {
            // addr out of PBA range?
            *r2c = 0;
            *nintrs = 0;
            *intr = 0;
            //assert(0);
        }
    }
#undef ADDR_IN_RANGE
#undef ADDR_TO_INTR
}

#endif /* _INTR_REGS_DEFS_H_ */
