/*
 * Copyright 2006 Nuova Systems, Inc. All Rights Reserved
 * Copyright 2011 Cisco Systems, Inc. All Rights Reserved
 */
#ident "$Id: intrc_regs_defs.h 43 2016-12-03 22:59:19Z buildsa $"

#ifndef _INTRC_REGS_DEFS_H_
#define _INTRC_REGS_DEFS_H_

//
// When accessing the set_int_level array region of the interrupt
// controller, the address selects which intr is sent like this:
//
//     Address[31:13] base address of the interrupt controller
//     Address[12:10] indicates the eCPU number
//     Address[9:7]   indicates the eCPU interrupt level
//     Address[6:2]   indicates which of 32 ints at that level
//

#define INTRC_MAX_ECPUS                 8
#define INTRC_MAX_INT_LVL               8
#define INTRC_MAX_INTS_PER_LVL          32

#define INTRC_BASE_MASK                 0xffffe000
#define INTRC_ECPU_SHIFT                10
#define INTRC_ECPU_MASK                 (INTRC_MAX_ECPUS-1)
#define INTRC_INT_LVL_SHIFT             7
#define INTRC_INT_LVL_MASK              (INTRC_MAX_INT_LVL-1)
#define INTRC_INTS_PER_LVL_SHIFT        2
#define INTRC_INTS_PER_LVL_MASK         (INTRC_MAX_INTS_PER_LVL-1)

// 001 01 1 001 01  00
static inline void
intrc_set_int_level_addr_enc(u_int32_t *addr,
                             u_int32_t base,
                             u_int8_t ecpu, u_int8_t lvl, u_int8_t intlvl)
{
    *addr = ((base & INTRC_BASE_MASK) |
             ((ecpu & INTRC_ECPU_MASK) << INTRC_ECPU_SHIFT) |
             ((lvl & INTRC_INT_LVL_MASK) << INTRC_INT_LVL_SHIFT) |
             ((intlvl & INTRC_INTS_PER_LVL_MASK) << INTRC_INTS_PER_LVL_SHIFT));
}

static inline void
intrc_set_int_level_addr_dec(u_int32_t addr,
                             u_int32_t *base,
                             u_int8_t *ecpu, u_int8_t *lvl, u_int8_t *intlvl)
{
    *base = addr & INTRC_BASE_MASK;
    *ecpu = (addr >> INTRC_ECPU_SHIFT) & INTRC_ECPU_MASK;
    *lvl = (addr >> INTRC_INT_LVL_SHIFT) & INTRC_INT_LVL_MASK;
    *intlvl = (addr >> INTRC_INTS_PER_LVL_SHIFT) & INTRC_INTS_PER_LVL_MASK;
}
#define INTRC_MREGS_(reg, ecpu, lvl) \
    (INTRC_MREGS_##reg##0_CPU0 + \
     ((INTRC_MREGS_##reg##0_CPU1 - INTRC_MREGS_##reg##0_CPU0) * (ecpu)) + \
     ((INTRC_MREGS_##reg##1_CPU0 - INTRC_MREGS_##reg##0_CPU0) * (lvl)))

#define INTRC_MREGS_SET_INT_LEVEL(ecpu, lvl) \
    INTRC_MREGS_(SET_INT_LEVEL, ecpu, lvl)
#define INTRC_MREGS_STATUS(ecpu, lvl)     INTRC_MREGS_(STATUS, ecpu, lvl)
#define INTRC_MREGS_MASK_SET(ecpu, lvl)   INTRC_MREGS_(MASK_SET, ecpu, lvl)
#define INTRC_MREGS_MASK_CLEAR(ecpu, lvl) INTRC_MREGS_(MASK_CLEAR, ecpu, lvl)

#endif /* _INTRC_REGS_DEFS_H_ */
