/*
 * Copyright 2006 Nuova Systems, Inc. All Rights Reserved
 */
#ident "$Id: net_regs_defs.h 6 2016-12-02 03:26:55Z buildsa $"

#ifndef _NET_REGDEFS_H_
#define _NET_REGDEFS_H_

/*
 * Definitions for the type values that are written to certain register
 */

/*
 * HREG: NIF cfg RSS HASH types
 */
enum rss_hash_types {
    RSS_HASH_TYPE_IPV4 = (1 << 1),
    RSS_HASH_TYPE_TCP_IPV4 = (1 << 2),
    RSS_HASH_TYPE_IPV6 = (1 << 3),
    RSS_HASH_TYPE_TCP_IPV6 = (1 << 4),
    RSS_HASH_TYPE_IPV6_EX = (1 << 5),
    RSS_HASH_TYPE_TCP_IPV6_EX = (1 << 6),
};

/*
 * MREG: LIF cfg vnic type
 */
enum net_mregs_lif_cfg_vnic_types {
    NET_MREGS_LIF_CFG_VNIC_TYPE_ETH = 0,  /* Ethernet, incl host FC vnics */
    NET_MREGS_LIF_CFG_VNIC_TYPE_FCOE = 1, /* FCoE */
    NET_MREGS_LIF_CFG_VNIC_TYPE_RAW = 15, /* RAW interface (no pkt rewrite) */
};

/*
 * MREG: Desc Size
 */
enum net_mregs_fwd_tcam_ops {
#if defined CONFIG_BASE_PALO
    NET_MREGS_FWD_TCAM_OP_WRITE = 0x0,
    NET_MREGS_FWD_TCAM_OP_FREE = 0x2,
    NET_MREGS_FWD_TCAM_OP_RESET = 0x4,
    NET_MREGS_FWD_TCAM_OP_NAA = 0x8,
    NET_MREGS_FWD_TCAM_OP_SEQ_SEARCH = 0x9,
    NET_MREGS_FWD_TCAM_OP_SEARCH_CLEAR = 0xc,
    NET_MREGS_FWD_TCAM_OP_READ = 0x10,
    NET_MREGS_FWD_TCAM_OP_SEARCH = 0x18,
#elif defined CONFIG_BASE_SERENO || defined CONFIG_BASE_CRUZ
    NET_MREGS_FWD_TCAM_OP_WRITE = 0x0,
    NET_MREGS_FWD_TCAM_OP_READ = 0x1,
    NET_MREGS_FWD_TCAM_OP_SEARCH = 0x2,
    NET_MREGS_FWD_TCAM_OP_CLEAR = 0x4,
#endif /* CONFIG_BASE */
};

// net_mregs_cfg_uif?_port?_make (and _get) are generated for Sereno

#if defined CONFIG_BASE_PALO
//
// Regtool doesn't generate these for us because the
// very large field makes it look like memory.
//
static inline void
net_mregs_cfg_uif_cfg_make(
  volatile u_int32_t *_regbase,
  u_int16_t ig_default_vlan,
  u_int8_t ig_default_dp,
  u_int8_t ig_default_cos,
  u_int16_t eg_default_vlan,
  u_int8_t eg_default_dp,
  u_int8_t eg_default_cos,
  u_int8_t is_trunk,
  u_int8_t ig_override_l2_tag,
  u_int8_t eg_override_l2_tag,
  u_int8_t eg_vlan_rewrite_mode,
  u_int8_t vntag_mode,
  u_int8_t local_fwd_enable,
  u_int8_t eg_i2n_cos_map_disable,
  u_int16_t vif_base,
  u_int32_t *ig_fwd_mask)
{
    u_int8_t *rb8 = (u_int8_t *)_regbase;
    u_int8_t *igfm8 = (u_int8_t *)ig_fwd_mask;
    u_int8_t i;

    _regbase[0] = (ig_default_vlan &
      NET_MREGS_CFG_UIF0_CFG_00_IG_DEFAULT_VLAN_MASK_FIELD) |
      ((ig_default_dp & NET_MREGS_CFG_UIF0_CFG_00_IG_DEFAULT_DP_MASK_FIELD) <<
      NET_MREGS_CFG_UIF0_CFG_00_IG_DEFAULT_DP_SHIFT) |
      ((ig_default_cos & NET_MREGS_CFG_UIF0_CFG_00_IG_DEFAULT_COS_MASK_FIELD) <<
      NET_MREGS_CFG_UIF0_CFG_00_IG_DEFAULT_COS_SHIFT) |
      ((eg_default_vlan & NET_MREGS_CFG_UIF0_CFG_00_EG_DEFAULT_VLAN_MASK_FIELD)
      << NET_MREGS_CFG_UIF0_CFG_00_EG_DEFAULT_VLAN_SHIFT) |
      ((eg_default_dp & NET_MREGS_CFG_UIF0_CFG_00_EG_DEFAULT_DP_MASK_FIELD) <<
      NET_MREGS_CFG_UIF0_CFG_00_EG_DEFAULT_DP_SHIFT) |
      ((eg_default_cos & NET_MREGS_CFG_UIF0_CFG_00_EG_DEFAULT_COS_MASK_FIELD) <<
      NET_MREGS_CFG_UIF0_CFG_00_EG_DEFAULT_COS_SHIFT);

    _regbase[1] = (is_trunk &
      NET_MREGS_CFG_UIF0_CFG_01_IS_TRUNK_MASK_FIELD) |
      ((ig_override_l2_tag &
      NET_MREGS_CFG_UIF0_CFG_01_IG_OVERRIDE_L2_TAG_MASK_FIELD) <<
      NET_MREGS_CFG_UIF0_CFG_01_IG_OVERRIDE_L2_TAG_SHIFT) |
      ((eg_override_l2_tag &
      NET_MREGS_CFG_UIF0_CFG_01_EG_OVERRIDE_L2_TAG_MASK_FIELD) <<
      NET_MREGS_CFG_UIF0_CFG_01_EG_OVERRIDE_L2_TAG_SHIFT) |
      ((eg_vlan_rewrite_mode &
      NET_MREGS_CFG_UIF0_CFG_01_EG_VLAN_REWRITE_MODE_MASK_FIELD) <<
      NET_MREGS_CFG_UIF0_CFG_01_EG_VLAN_REWRITE_MODE_SHIFT) |
      ((vntag_mode &
      NET_MREGS_CFG_UIF0_CFG_01_VNTAG_MODE_MASK_FIELD) <<
      NET_MREGS_CFG_UIF0_CFG_01_VNTAG_MODE_SHIFT) |
      ((local_fwd_enable &
      NET_MREGS_CFG_UIF0_CFG_01_LOCAL_FWD_ENABLE_MASK_FIELD) <<
      NET_MREGS_CFG_UIF0_CFG_01_LOCAL_FWD_ENABLE_SHIFT) |
      ((eg_i2n_cos_map_disable &
      NET_MREGS_CFG_UIF0_CFG_01_EG_I2N_COS_MAP_DISABLE_MASK_FIELD) <<
      NET_MREGS_CFG_UIF0_CFG_01_EG_I2N_COS_MAP_DISABLE_SHIFT) |
      ((vif_base &
      NET_MREGS_CFG_UIF0_CFG_01_VIF_BASE_MASK_FIELD) <<
      NET_MREGS_CFG_UIF0_CFG_01_VIF_BASE_SHIFT);

    // ig_fwd_mask isn't naturally aligned....blah
    for (i = 0; i < 16; i++) {
       rb8[6+i] = igfm8[i];
    }
    rb8[22] = 0;
    rb8[23] = 0;
}

static inline void
net_mregs_cfg_uif_cfg_get(
  volatile u_int32_t *_regbase,
  u_int16_t *ig_default_vlan,
  u_int8_t *ig_default_dp,
  u_int8_t *ig_default_cos,
  u_int16_t *eg_default_vlan,
  u_int8_t *eg_default_dp,
  u_int8_t *eg_default_cos,
  u_int8_t *is_trunk,
  u_int8_t *ig_override_l2_tag,
  u_int8_t *eg_override_l2_tag,
  u_int8_t *eg_vlan_rewrite_mode,
  u_int8_t *vntag_mode,
  u_int8_t *local_fwd_enable,
  u_int8_t *eg_i2n_cos_map_disable,
  u_int16_t *vif_base,
  u_int32_t *ig_fwd_mask)
{
    u_int8_t *rb8 = (u_int8_t *)_regbase;
    u_int8_t *igfm8 = (u_int8_t *)ig_fwd_mask;
    u_int8_t i;

    *ig_default_vlan =
      _regbase[0] & NET_MREGS_CFG_UIF0_CFG_00_IG_DEFAULT_VLAN_MASK_FIELD;
    *ig_default_dp =
      (_regbase[0] >> NET_MREGS_CFG_UIF0_CFG_00_IG_DEFAULT_DP_SHIFT) &
      NET_MREGS_CFG_UIF0_CFG_00_IG_DEFAULT_DP_MASK_FIELD;
    *ig_default_cos =
      (_regbase[0] >> NET_MREGS_CFG_UIF0_CFG_00_IG_DEFAULT_COS_SHIFT) &
      NET_MREGS_CFG_UIF0_CFG_00_IG_DEFAULT_COS_MASK_FIELD;
    *eg_default_vlan =
      (_regbase[0] >> NET_MREGS_CFG_UIF0_CFG_00_EG_DEFAULT_VLAN_SHIFT) &
      NET_MREGS_CFG_UIF0_CFG_00_EG_DEFAULT_VLAN_MASK_FIELD;
    *eg_default_dp =
      (_regbase[0] >> NET_MREGS_CFG_UIF0_CFG_00_EG_DEFAULT_DP_SHIFT) &
      NET_MREGS_CFG_UIF0_CFG_00_EG_DEFAULT_DP_MASK_FIELD;
    *eg_default_cos =
      (_regbase[0] >> NET_MREGS_CFG_UIF0_CFG_00_EG_DEFAULT_COS_SHIFT) &
      NET_MREGS_CFG_UIF0_CFG_00_EG_DEFAULT_COS_MASK_FIELD;

    *is_trunk =
      _regbase[1] & NET_MREGS_CFG_UIF0_CFG_01_IS_TRUNK_MASK_FIELD;
    *ig_override_l2_tag =
      (_regbase[1] >> NET_MREGS_CFG_UIF0_CFG_01_IG_OVERRIDE_L2_TAG_SHIFT) &
      NET_MREGS_CFG_UIF0_CFG_01_IG_OVERRIDE_L2_TAG_MASK_FIELD;
    *eg_override_l2_tag =
      (_regbase[1] >> NET_MREGS_CFG_UIF0_CFG_01_EG_OVERRIDE_L2_TAG_SHIFT) &
      NET_MREGS_CFG_UIF0_CFG_01_EG_OVERRIDE_L2_TAG_MASK_FIELD;
    *eg_vlan_rewrite_mode =
      (_regbase[1] >> NET_MREGS_CFG_UIF0_CFG_01_EG_VLAN_REWRITE_MODE_SHIFT) &
      NET_MREGS_CFG_UIF0_CFG_01_EG_VLAN_REWRITE_MODE_MASK_FIELD;
    *vntag_mode =
      (_regbase[1] >> NET_MREGS_CFG_UIF0_CFG_01_VNTAG_MODE_SHIFT) &
      NET_MREGS_CFG_UIF0_CFG_01_VNTAG_MODE_MASK_FIELD;
    *local_fwd_enable =
      (_regbase[1] >> NET_MREGS_CFG_UIF0_CFG_01_LOCAL_FWD_ENABLE_SHIFT) &
      NET_MREGS_CFG_UIF0_CFG_01_LOCAL_FWD_ENABLE_MASK_FIELD;
    *eg_i2n_cos_map_disable =
      (_regbase[1] >> NET_MREGS_CFG_UIF0_CFG_01_EG_I2N_COS_MAP_DISABLE_SHIFT) &
      NET_MREGS_CFG_UIF0_CFG_01_EG_I2N_COS_MAP_DISABLE_MASK_FIELD;
    *vif_base =
      (_regbase[1] >> NET_MREGS_CFG_UIF0_CFG_01_VIF_BASE_SHIFT) &
      NET_MREGS_CFG_UIF0_CFG_01_VIF_BASE_MASK_FIELD;

    // ig_fwd_mask isn't naturally aligned....blah
    for (i = 0; i < 16; i++) {
       igfm8[i] = rb8[6+i];
    }
}
#endif /* CONFIG_BASE_PALO */

#if defined CONFIG_BASE_PALO

static inline void
net_mregs_fwd_map_make(
  volatile u_int32_t *_regbase,
  const u_int32_t *lif_bitmap,
  u_int8_t uif0,
  u_int8_t uif1,
  u_int8_t ig_bcn_drop)
{
    _regbase[0] = lif_bitmap[0];
    _regbase[1] = lif_bitmap[1];
    _regbase[2] = lif_bitmap[2];
    _regbase[3] = lif_bitmap[3];

    _regbase[4] = (uif0 & NET_MREGS_DHS_FWD_MAP_04_UIF0_MASK_FIELD) |
      ((uif1 & NET_MREGS_DHS_FWD_MAP_04_UIF1_MASK_FIELD) <<
        NET_MREGS_DHS_FWD_MAP_04_UIF1_SHIFT) |
      ((ig_bcn_drop & NET_MREGS_DHS_FWD_MAP_04_IG_BCN_DROP_MASK_FIELD) <<
        NET_MREGS_DHS_FWD_MAP_04_IG_BCN_DROP_SHIFT);
}

static inline void
net_mregs_fwd_map_get(
  volatile u_int32_t *_regbase,
  u_int32_t *lif_bitmap,
  u_int8_t *uif0,
  u_int8_t *uif1,
  u_int8_t *ig_bcn_drop)
{
    lif_bitmap[0] = _regbase[0];
    lif_bitmap[1] = _regbase[1];
    lif_bitmap[2] = _regbase[2];
    lif_bitmap[3] = _regbase[3];

    *uif0 = _regbase[4] & NET_MREGS_DHS_FWD_MAP_04_UIF0_MASK_FIELD;
    *uif1 = (_regbase[4] >> NET_MREGS_DHS_FWD_MAP_04_UIF1_SHIFT) &
      NET_MREGS_DHS_FWD_MAP_04_UIF1_MASK_FIELD;
    *ig_bcn_drop = (_regbase[4] >>
      NET_MREGS_DHS_FWD_MAP_04_IG_BCN_DROP_SHIFT) &
      NET_MREGS_DHS_FWD_MAP_04_IG_BCN_DROP_MASK_FIELD;
}

#elif defined CONFIG_BASE_SERENO || defined CONFIG_BASE_CRUZ

static inline void
net_mregs_fwd_map_make(
  volatile u_int32_t *_regbase,
  const u_int32_t *lif_bitmap,
  u_int8_t uif0,
  u_int8_t uif1)
{
    _regbase[0] = lif_bitmap[0];
    _regbase[1] = lif_bitmap[1];
    _regbase[2] = lif_bitmap[2];
    _regbase[3] = lif_bitmap[3];
    _regbase[4] = lif_bitmap[4];
    _regbase[5] = lif_bitmap[5];
    _regbase[6] = lif_bitmap[6];
    _regbase[7] = lif_bitmap[7];

    _regbase[8] = (uif0 | (uif1 << 1));
}

static inline void
net_mregs_fwd_map_get(
  volatile u_int32_t *_regbase,
  u_int32_t *lif_bitmap,
  u_int8_t *uif0,
  u_int8_t *uif1)
{
    lif_bitmap[0] = _regbase[0];
    lif_bitmap[1] = _regbase[1];
    lif_bitmap[2] = _regbase[2];
    lif_bitmap[3] = _regbase[3];
    lif_bitmap[4] = _regbase[4];
    lif_bitmap[5] = _regbase[5];
    lif_bitmap[6] = _regbase[6];
    lif_bitmap[7] = _regbase[7];

    *uif0 = _regbase[8] & 1;
    *uif1 = (_regbase[8] >> 1) & 1;
}

#endif /* CONFIG_BASE */

#if defined CONFIG_BASE_PALO
static inline void
net_mregs_cfg_uif_cfg_lfwd_en_get(
    volatile u_int32_t *_regbase,
    u_int32_t *lfwd_en)
{
    *lfwd_en =
        !!(_regbase[0] & NET_MREGS_CFG_UIF1_CFG_01_LOCAL_FWD_ENABLE);
}

static inline void
net_mregs_cfg_uif_cfg_lfwd_en_make(
    volatile u_int32_t *_regbase,
    u_int32_t lfwd_en)
{
    u_int32_t mask_val =
        lfwd_en << NET_MREGS_CFG_UIF1_CFG_01_LOCAL_FWD_ENABLE_SHIFT;

    _regbase[0] = (_regbase[0] & !NET_MREGS_CFG_UIF1_CFG_01_LOCAL_FWD_ENABLE);
    _regbase[0] |= mask_val;
}
#endif /* CONFIG_BASE */

#define NET_MREGS_ACL_FLAGS_MASK    0x0f
#define NET_MREGS_ACL_FLAGS_IPV6    0x01
#define NET_MREGS_ACL_FLAGS_IPV4    0x02
#define NET_MREGS_ACL_FLAGS_FCOE    0x04
#define NET_MREGS_ACL_FLAGS_IPFRAG  0x08

static inline void
net_mregs_acl_get(
    volatile u_int32_t *_regbase,
    u_int64_t *mac_da,
    u_int64_t *mac_sa,
    u_int16_t *tag,
    u_int16_t *type,
    u_int64_t *ip_da_lo,
    u_int64_t *ip_da_hi,
    u_int64_t *ip_sa_lo,
    u_int64_t *ip_sa_hi,
    u_int16_t *l4_dp,
    u_int16_t *l4_sp,
    u_int8_t *ip_proto,
    u_int8_t *flags,
    u_int8_t *tcp_flag,
    u_int16_t *eg_lif)
{
    *mac_da = (((u_int64_t) _regbase[1] & 0xffff) << 32) | _regbase[0];
    *mac_sa = ((u_int64_t) _regbase[2] << 16) | (_regbase[1] >> 16);
    *tag = _regbase[3] & 0xffff;
    *type = (_regbase[3] & 0xffff0000) >> 16;
    *ip_da_lo = ((u_int64_t) _regbase[5] << 32) | _regbase[4];
    *ip_da_hi = ((u_int64_t) _regbase[7] << 32) | _regbase[6];
    *ip_sa_lo = ((u_int64_t) _regbase[9] << 32) | _regbase[8];
    *ip_sa_hi = ((u_int64_t) _regbase[11] << 32) | _regbase[10];
    *l4_dp = _regbase[12] & 0xffff;
    *l4_sp = (_regbase[12] & 0xffff0000) >> 16;
    *ip_proto = _regbase[13] & 0xff;
    *flags = (_regbase[13] & 0xff00) >> 8;
    *tcp_flag = (_regbase[13] & 0xff0000) >> 16;

    /*
     * Sereno and Cruz also specify the LIF in the egress (only) ACL.
     * For Sereno, it's an 8-bit field, and for Cruz it's 11 bits.
     * Sereno sets the upper bits to zero, and Palo sets all the bits to
     * zero, so this expression will work on all ASICs.
     */
    *eg_lif = ((_regbase[14] & 0x7) << 8) | ((_regbase[13] & 0xff000000) >> 24);
}

static inline void
net_mregs_acl_make(
    volatile u_int32_t *_regbase,
    u_int64_t mac_da,
    u_int64_t mac_sa,
    u_int16_t tag,
    u_int16_t type,
    u_int64_t ip_da_lo,
    u_int64_t ip_da_hi,
    u_int64_t ip_sa_lo,
    u_int64_t ip_sa_hi,
    u_int16_t l4_dp,
    u_int16_t l4_sp,
    u_int8_t ip_proto,
    u_int8_t flags,
    u_int8_t tcp_flag,
    u_int16_t eg_lif)
{
    _regbase[0] = mac_da & 0xffffffff;
    _regbase[1] = ((mac_da >> 32) & 0xffff) | ((mac_sa & 0xffff) << 16);
    _regbase[2] = (mac_sa >> 16) & 0xffffffff;
    _regbase[3] = ((u_int32_t) type << 16) | tag;
    _regbase[4] = ip_da_lo & 0xffffffff;
    _regbase[5] = (ip_da_lo >> 32) & 0xffffffff;
    _regbase[6] = ip_da_hi & 0xffffffff;
    _regbase[7] = (ip_da_hi >> 32) & 0xffffffff;
    _regbase[8] = ip_sa_lo & 0xffffffff;
    _regbase[9] = (ip_sa_lo >> 32) & 0xffffffff;
    _regbase[10] = ip_sa_hi & 0xffffffff;
    _regbase[11] = (ip_sa_hi >> 32) & 0xffffffff;
    _regbase[12] = ((u_int32_t) l4_sp << 16) | l4_dp;
    _regbase[13] = ((u_int32_t)eg_lif << 24) | ((u_int32_t) tcp_flag << 16) |
                   (((u_int32_t) flags & NET_MREGS_ACL_FLAGS_MASK) << 8) |
                   ip_proto;
    _regbase[14] = eg_lif >> 8;
    _regbase[15] = 0;
}

#endif /* _NET_REGDEFS_H_ */
