/*
 * Copyright (c) 2007 Nuova Systems, Inc.  All rights reserved.
 */
#ident "$Id: regs_macros.h 61294 2010-10-12 20:31:01Z orosz $"

// N.B. Many of these defines are duplicated in
// sa/src/palo/fw/common/lib/libpalostand/src/regs_macros.h
// and sa/src/sims/model/tests/lib/libhwiotest/include/hwio_setup.h

#ifndef __REGS_MACROS_H__
#define __REGS_MACROS_H__

#ifndef __KERNEL__
#include <stdint.h>
#endif // __KERNEL__

#include "net_mregs.h"

#ifdef CONFIG_BASE_CRUZ
#include "wq_mregs_sched.h"
#endif

#define WQ_HREGS_SIZE           WQ_HREGS_RING_BASE_BLKSZ
#define WQ_HREGS_MAXCNT         WQ_HREGS_RING_BASE_MAXCNT
#define WQ_HREGS_ADDR(i)        (WQ_HREGS_BASE + (WQ_HREGS_SIZE * (i)))
#define WQ_HREGS_(reg, i)       (WQ_HREGS_ADDR(i) + WQ_HREGS_OFFSET(reg))

#define WQ_MREGS_SIZE           WQ_MREGS_WQ_TYPE_BLKSZ
#define WQ_MREGS_MAXCNT         WQ_MREGS_WQ_TYPE_MAXCNT
#define WQ_MREGS_ADDR(i)        (WQ_MREGS_BASE + (WQ_MREGS_SIZE * (i)))
#define WQ_MREGS_(reg, i)       (WQ_MREGS_ADDR(i) + WQ_MREGS_OFFSET(reg))

#define WQ_MREGS_VNIC_ADDR(i)   \
    (WQ_MREGS_VNIC_BASE__ + (WQ_MREGS_VNIC_BLKSZ__ * (i)))
#define WQ_MREGS_VNIC_REG(reg, i)  \
    (WQ_MREGS_VNIC_ADDR(i) + WQ_MREGS_VNIC_OFFSET_GEN(reg))

#if defined CONFIG_BASE_PALO

#define LIF_CFG_NWORDS 3
#define WQ_MREGS_SCHED_VNIC_STATE(vnicid, uif, cos) \
    WQ_MREGS_SCHED_DHS_VNIC_STATE + WQ_MREGS_SCHED_DHS_VNIC_STATE_BLKSZ * \
    ((uif) * 128 + (vnicid))

#elif defined(CONFIG_BASE_SERENO) || defined(CONFIG_BASE_CRUZ)

#define LIF_CFG_NWORDS NET_MREGS_DHS_LIF_CFG_NWORDS
#define WQ_MREGS_SCHED_VNIC_STATE(vnicid, uif, cos) \
    (WQ_MREGS_SCHED_DHS_VNIC_STATE + WQ_MREGS_SCHED_DHS_VNIC_STATE_BLKSZ * \
    (((vnicid) << 4) | ((uif) << 3) | ((cos) & 7)))

#endif /* CONFIG_BASE */

#if defined(CONFIG_BASE_PALO) || defined(CONFIG_BASE_SERENO)

#define WQ_MREGS_SCHED_RLMT_STATE(vnicid) \
    (WQ_MREGS_SCHED_DHS_RLMT_STATE + \
    WQ_MREGS_SCHED_DHS_RLMT_STATE_BLKSZ * (vnicid))

#define WQ_MREGS_SCHED_CIR_STATE(vnicid) \
    (WQ_MREGS_SCHED_DHS_CIR_STATE + \
    WQ_MREGS_SCHED_DHS_CIR_STATE_BLKSZ * (vnicid))

#define RLMT_STATE_MAXCNT WQ_MREGS_SCHED_DHS_RLMT_STATE_MAXCNT
#define RLMT_MAX_DEPTH (RLMT_STATE_MAXCNT * \
        WQ_MREGS_SCHED_DHS_RLMT_STATE_BLKSZ)
#define rlmt_state_base(i, j) \
    (WQ_MREGS_SCHED_DHS_RLMT_STATE + (i * RLMT_MAX_DEPTH) + \
    (j * WQ_MREGS_SCHED_DHS_RLMT_STATE_BLKSZ))
#define MAX_RLMT 1

#elif defined(CONFIG_BASE_CRUZ)

#define RLMT_STATE_MAXCNT WQ_MREGS_SCHED_DHS_RLMT_STATE_0_MAXCNT
#define RLMT_MAX_DEPTH (RLMT_STATE_MAXCNT * \
        WQ_MREGS_SCHED_DHS_RLMT_STATE_0_BLKSZ)
#define rlmt_state_base(i, j) \
    (WQ_MREGS_SCHED_DHS_RLMT_STATE_0 + (i * RLMT_MAX_DEPTH) + \
    (j * WQ_MREGS_SCHED_DHS_RLMT_STATE_0_BLKSZ))
#define MAX_RLMT 8

static inline void
wq_mregs_sched_dhs_rlmt_state_make(
  volatile u_int32_t *_regbase,
  u_int32_t  cur_credit,
  u_int32_t  cfg_max_credit,
  u_int8_t   rsvd,
  u_int32_t  cfg_rate)
{
    wq_mregs_sched_dhs_rlmt_state_0_make(_regbase,
            cur_credit, cfg_max_credit, rsvd, cfg_rate);
}

static inline void
wq_mregs_sched_dhs_rlmt_state_get(
  volatile u_int32_t *_regbase,
  u_int32_t *cur_credit,
  u_int32_t *cfg_max_credit,
  u_int8_t  *rsvd,
  u_int32_t *cfg_rate)
{
    wq_mregs_sched_dhs_rlmt_state_0_get(_regbase,
            cur_credit, cfg_max_credit, rsvd, cfg_rate);
}

/*
 * There are actually 8 sets of 256 rlmt state registers in Cruz, which are
 * indexed by vnicid. Rate limit for vnics 0-255 can programmed using state_0,
 * 256-512 using state_1 and so on. As the sets are contiguous and symmetric
 * we are using state_0 as base and indexing using vnicid.
 */
#define WQ_MREGS_SCHED_RLMT_STATE(vnicid) \
    (WQ_MREGS_SCHED_DHS_RLMT_STATE_0 + \
    WQ_MREGS_SCHED_DHS_RLMT_STATE_0_BLKSZ * (vnicid))

static inline void
wq_mregs_sched_dhs_cir_state_make(
  volatile u_int32_t *_regbase,
  u_int32_t  cur_credit,
  u_int32_t  cfg_max_credit,
  u_int8_t   rsvd,
  u_int32_t  cfg_rate)
{
    wq_mregs_sched_dhs_cir_state_0_make(_regbase,
            cur_credit, cfg_max_credit, rsvd, cfg_rate);
}

static inline void
wq_mregs_sched_dhs_cir_state_get(
  volatile u_int32_t *_regbase,
  u_int32_t *cur_credit,
  u_int32_t *cfg_max_credit,
  u_int8_t  *rsvd,
  u_int32_t *cfg_rate)
{
    wq_mregs_sched_dhs_cir_state_0_get(_regbase,
            cur_credit, cfg_max_credit, rsvd, cfg_rate);
}

/*
 * There are actually 8 sets of 256 cir state registers in Cruz, which are
 * indexed by vnicid. CIR for vnics 0-255 can programmed using state_0,
 * 256-512 using state_1 and so on. As the sets are contiguous and symmetric
 * we are using state_0 as base and indexing using vnicid.
 */
#define WQ_MREGS_SCHED_CIR_STATE(vnicid) \
    (WQ_MREGS_SCHED_DHS_CIR_STATE_0 + \
    WQ_MREGS_SCHED_DHS_CIR_STATE_0_BLKSZ * (vnicid))

#endif /* CONFIG_BASE */

#define WQ_MREGS_SCHED_DHS_COS_CREDIT(uif) \
    (WQ_MREGS_SCHED_DHS_P0_COS_CREDIT + \
    (WQ_MREGS_SCHED_DHS_P1_COS_CREDIT - \
     WQ_MREGS_SCHED_DHS_P0_COS_CREDIT) * (uif))

#define WQ_MREGS_SCHED_CFG_COS_CREDIT_ALLOWED(uif) \
    (WQ_MREGS_SCHED_CFG_P0_COS_CREDIT_ALLOWED + \
    (WQ_MREGS_SCHED_CFG_P1_COS_CREDIT_ALLOWED - \
     WQ_MREGS_SCHED_CFG_P0_COS_CREDIT_ALLOWED) * (uif))

#define RQ_HREGS_SIZE           RQ_HREGS_RING_BASE_BLKSZ
#define RQ_HREGS_MAXCNT         RQ_HREGS_RING_BASE_MAXCNT
#define RQ_HREGS_ADDR(i)        (RQ_HREGS_BASE + (RQ_HREGS_SIZE * (i)))
#define RQ_HREGS_(reg, i)       (RQ_HREGS_ADDR(i) + RQ_HREGS_OFFSET(reg))

#define RQ_MREGS_SIZE           RQ_MREGS_MASTER_ENABLE_BLKSZ
#define RQ_MREGS_MAXCNT         RQ_MREGS_MASTER_ENABLE_MAXCNT
#define RQ_MREGS_ADDR(i)        (RQ_MREGS_BASE + (RQ_MREGS_SIZE * (i)))
#define RQ_MREGS_(reg, i)       (RQ_MREGS_ADDR(i) + RQ_MREGS_OFFSET(reg))

#define RQ_MREGS_VNIC_ADDR(i)   \
    (RQ_MREGS_VNIC_BASE__ + (RQ_MREGS_VNIC_BLKSZ__ * (i)))
#define RQ_MREGS_VNIC_REG(reg, i) \
    (RQ_MREGS_VNIC_ADDR(i) + RQ_MREGS_VNIC_OFFSET_GEN(reg))

#define CQ_HREGS_SIZE           CQ_HREGS_CQ_RING_BASE_BLKSZ
#define CQ_HREGS_MAXCNT         CQ_HREGS_CQ_RING_BASE_MAXCNT
#define CQ_HREGS_ADDR(i)        (CQ_HREGS_BASE + (CQ_HREGS_SIZE * (i)))
#define CQ_HREGS_(reg, i)       (CQ_HREGS_ADDR(i) + CQ_HREGS_OFFSET(reg))

#define CQ_MREGS_SIZE           CQ_MREGS_CQ_VNIC_BLKSZ
#define CQ_MREGS_MAXCNT         CQ_MREGS_CQ_VNIC_MAXCNT
#define CQ_MREGS_ADDR(i)        (CQ_MREGS_BASE + (CQ_MREGS_SIZE * (i)))
#define CQ_MREGS_(reg, i)       (CQ_MREGS_ADDR(i) + CQ_MREGS_OFFSET(reg))

#define CQ_MREGS_VNIC_ADDR(i)   \
    (CQ_MREGS_VNIC_BASE + (CQ_MREGS_VNIC_BLKSZ * (i)))
#define CQ_MREGS_VNIC_REG(reg, i) \
    (CQ_MREGS_VNIC_ADDR(i) + CQ_MREGS_VNIC_OFFSET(reg))

#define EDMA_SIZE(reg)          (EDMA_MREGS_##reg##1_OFFSET - \
                                 EDMA_MREGS_##reg##0_OFFSET)
#define EDMA_BASE               EDMA_MREGS_BASE_ADDR
#define EDMA_ADDR(reg, i)       (EDMA_BASE + (EDMA_SIZE(reg) * (i)))
#define EDMA_OFFSET(reg)        (EDMA_MREGS_##reg##0_OFFSET)
#define EDMA_(reg, i)           (EDMA_ADDR(reg, i) + EDMA_OFFSET(reg))

#define IOMMU_MREGS_(reg, i)    (IOMMU_MREGS_##reg + \
    IOMMU_MREGS_##reg##_BLKSZ * (i))

#define INTRCTRL_HREGS_ADDR(i)  \
    (INTRCTRL_HREGS_BASE_ADDR + (INTRCTRL_HREGS_UNITSZ * (i)))
#define INTRCTRL_HREGS_(reg, i) \
    ((u_int32_t)(uintptr_t)INTR_HREGS_##reg##_ADDR_COMPUTE(INTR_HREGS_BASE_ADDR, i))

#define INTRCTRL_MREGS_ADDR(i)  \
    (INTRCTRL_MREGS_BASE_ADDR + (INTRCTRL_MREGS_UNITSZ * (i)))
#define INTRCTRL_MREGS_(reg, i) \
    (INTRCTRL_MREGS_ADDR(i) + INTR_MREGS_OFFSET(reg))

#define INTRMSI_HREGS_ADDR(i)  \
    (INTRMSI_HREGS_BASE_ADDR + (INTRMSI_HREGS_UNITSZ * (i)))

#define RSS_KEY_HREGS_ADDR(i)  \
    (NET_HREGS_DHS_RSS_KEY + (RSS_KEY_HREGS_SIZE * (i)))
#define RSS_KEY_HREGS_SIZE     NET_HREGS_DHS_RSS_KEY_BLKSZ
#define RSS_KEY_HREGS_MAXCNT   NET_HREGS_DHS_RSS_KEY_MAXCNT
#if defined CONFIG_BASE_PALO
#define NET_HREGS_DHS_RSS_KEY_NWORDS 3
#endif /* CONFIG_BASE_PALO */
#define RSS_KEY_HREGS_NBYTES   (NET_HREGS_DHS_RSS_KEY_NWORDS * 4)

#define RSS_CPU_HREGS_ADDR(i)  \
    (NET_HREGS_DHS_RSS_CPU + (RSS_CPU_HREGS_SIZE * (i)))
#define RSS_CPU_HREGS_SIZE     NET_HREGS_DHS_RSS_CPU_BLKSZ
#define RSS_CPU_HREGS_MAXCNT   NET_HREGS_DHS_RSS_CPU_MAXCNT
#if defined CONFIG_BASE_PALO
#define NET_HREGS_DHS_RSS_CPU_NWORDS 3
#endif /* CONFIG_BASE_PALO */
#define RSS_CPU_HREGS_NBYTES   (NET_HREGS_DHS_RSS_CPU_NWORDS * 4)

#define NIC_CFG_HREGS_ADDR(i)  \
    (NET_HREGS_DHS_NIC_CFG + (NIC_CFG_HREGS_SIZE * (i)))
#define NIC_CFG_HREGS_SIZE     NET_HREGS_DHS_NIC_CFG_BLKSZ
#define NIC_CFG_HREGS_MAXCNT   NET_HREGS_DHS_NIC_CFG_MAXCNT

#define TX_STATS_HREGS_ADDR(i)  \
    (NET_HREGS_DHS_TX_STATS + (TX_STATS_HREGS_SIZE * (i)))
#define TX_STATS_HREGS_SIZE     NET_HREGS_DHS_TX_STATS_BLKSZ
#define TX_STATS_HREGS_MAXCNT   NET_HREGS_DHS_TX_STATS_MAXCNT

#define RX_STATS_HREGS_ADDR(i)  \
    (NET_HREGS_DHS_RX_STATS + (RX_STATS_HREGS_SIZE * (i)))
#define RX_STATS_HREGS_SIZE     NET_HREGS_DHS_RX_STATS_BLKSZ
#define RX_STATS_HREGS_MAXCNT   NET_HREGS_DHS_RX_STATS_MAXCNT

#if defined CONFIG_BASE_PALO || defined CONFIG_BASE_SERENO

#define FWD_MAP(i, o) \
    ((uintptr_t)NET_MREGS_DHS_FWD_MAP_ADDR_COMPUTE( \
    NET_MREGS_BASE_ADDR, i, o))

#elif defined CONFIG_BASE_CRUZ

#define FWD_MAP(i) \
    ((uintptr_t)NET_MREGS_DHS_FWD_MAP_ADDR_COMPUTE( \
    NET_MREGS_BASE_ADDR, i))

/*
 * External multicast lists are packed arrays of 12-bit LIFs.  A block
 * of three words can hold 8 LIFs, as follows:
 *
 *          +------------24+-------------------12+--------------------0+
 * mem[0] = | lif[2][7:0]  |    lif[1][11:0]     |    lif[0][11:0]     |
 *          +--------------+---------------------+---------------------+
 *          +-----------28+------------16+-------------4+-------------0+
 * mem[1] = | lif[5][3:0] | lif[4][11:0] | lif[3][11:0] | lif[2][11:8] |
 *          +-------------+--------------+--------------+--------------+
 *          +------------------20+--------------------8+--------------0+
 * mem[2] = |   lif[7][11:0]     |    lif[6][11:0]     | lif[5][11:4]  |
 *          +--------------------+---------------------+---------------+
 */
static inline void
net_mcast_list_make(u_int32_t *mem, const u_int16_t lif[8])
{
    mem[0] = lif[0] | ((u_int32_t)lif[1] << 12) | ((u_int32_t)lif[2] << 24);
    mem[1] = (lif[2] >> 8) | (lif[3] << 4) |
             ((u_int32_t)lif[4] << 16) | ((u_int32_t)lif[5] << 28);
    mem[2] = (lif[5] >> 4) | ((u_int32_t)lif[6] << 8) |
             ((u_int32_t)lif[7] << 20);
}

static inline void
net_mcast_list_get(const u_int32_t *mem, u_int16_t lif[8])
{
    lif[0] = mem[0] & 0xfff;
    lif[1] = (mem[0] >> 12) & 0xfff;
    lif[2] = ((mem[1] << 8) | (mem[0] >> 24)) & 0xfff;
    lif[3] = (mem[1] >> 4) & 0xfff;
    lif[4] = (mem[1] >> 16) & 0xfff;
    lif[5] = ((mem[2] << 4) | (mem[1] >> 28)) & 0xfff;
    lif[6] = (mem[2] >> 8) & 0xfff;
    lif[7] = (mem[2] >> 20) & 0xfff;
}

#endif

#if defined CONFIG_BASE_PALO

#define FWD_TCAM(i) \
    ((uintptr_t)NET_MREGS_DHS_FWD_TCAM_ADDR_COMPUTE( \
    NET_MREGS_BASE_ADDR, i))

#define FWD_TCAM_OP \
    ((uintptr_t)NET_MREGS_DHS_CPUIF_FWD_TCAM_OP_ADDR_COMPUTE( \
    NET_MREGS_BASE_ADDR))

#define FWD_TCAM_SDATA \
    ((uintptr_t)NET_MREGS_CFG_CPUIF_FWD_TCAM_SDATA_ADDR_COMPUTE( \
    NET_MREGS_BASE_ADDR))
#define FWD_TCAM_STA \
    ((uintptr_t)NET_MREGS_STA_CPUIF_FWD_TCAM_STA_ADDR_COMPUTE( \
    NET_MREGS_BASE_ADDR))

#elif defined(CONFIG_BASE_SERENO)

#define FWD_TCAM_X(i) \
    ((uintptr_t)NET_MREGS_DHS_FWD_TCAM_X_ADDR_COMPUTE( \
    NET_MREGS_BASE_ADDR, i))
#define FWD_TCAM_Y(i) \
    ((uintptr_t)NET_MREGS_DHS_FWD_TCAM_Y_ADDR_COMPUTE( \
    NET_MREGS_BASE_ADDR, i))

#define FWD_TCAM_OP \
    ((uintptr_t)NET_MREGS_DHS_FWD_TCAM_OP_ADDR_COMPUTE( \
    NET_MREGS_BASE_ADDR))

#define FWD_TCAM_SDATA \
    ((uintptr_t)NET_MREGS_CFG_FWD_TCAM_SDATA_ADDR_COMPUTE( \
    NET_MREGS_BASE_ADDR))
#define FWD_TCAM_STA \
    ((uintptr_t)NET_MREGS_STA_FWD_TCAM_STA_ADDR_COMPUTE( \
    NET_MREGS_BASE_ADDR))

#elif defined(CONFIG_BASE_CRUZ)

#define FWD_TCAM_X(i) \
    ((uintptr_t)NET_MREGS_DHS_FWD_TCAM_X_ADDR_COMPUTE( \
    NET_MREGS_BASE_ADDR, i, 0))
#define FWD_TCAM_Y(i) \
    ((uintptr_t)NET_MREGS_DHS_FWD_TCAM_Y_ADDR_COMPUTE( \
    NET_MREGS_BASE_ADDR, i, 0))

#define FWD_TCAM_OP \
    ((uintptr_t)NET_MREGS_DHS_FWD_TCAM_OP_ADDR_COMPUTE( \
    NET_MREGS_BASE_ADDR))

#define FWD_TCAM_SDATA \
    ((uintptr_t)NET_MREGS_CFG_FWD_TCAM_SDATA_ADDR_COMPUTE( \
    NET_MREGS_BASE_ADDR, 0))
#define FWD_TCAM_STA \
    ((uintptr_t)NET_MREGS_STA_FWD_TCAM_STA_ADDR_COMPUTE( \
    NET_MREGS_BASE_ADDR, 0))

#endif /* CONFIG_BASE */

#if defined CONFIG_BASE_PALO

#define UIF0_CFG(o) \
    ((uintptr_t)NET_MREGS_CFG_UIF0_CFG_ADDR_COMPUTE( \
    NET_MREGS_BASE_ADDR, o))
#define UIF1_CFG(o) \
    ((uintptr_t)NET_MREGS_CFG_UIF1_CFG_ADDR_COMPUTE( \
    NET_MREGS_BASE_ADDR, o))

#define UIF_CFG(uif, o) \
    ((uif) == 0 ? UIF0_CFG(o) : UIF1_CFG(o))

#elif defined(CONFIG_BASE_SERENO) || defined(CONFIG_BASE_CRUZ)

#define _UIF_CFG_UIF_BLKSZ \
    (NET_MREGS_CFG_UIF1_PORT0_CFG - NET_MREGS_CFG_UIF0_PORT0_CFG)
#define _UIF_CFG_PORT_BLKSZ \
    (NET_MREGS_CFG_UIF0_PORT1_CFG - NET_MREGS_CFG_UIF0_PORT0_CFG)
#define UIF_CFG(uif, port) \
    ((uintptr_t)(NET_MREGS_CFG_UIF0_PORT0_CFG + \
                 (((uif) * _UIF_CFG_UIF_BLKSZ) + \
                  ((port) * _UIF_CFG_PORT_BLKSZ))))

#define UIF_CFG_VNTAG_MODE(uif) \
    ((uif) == 0 ? \
     NET_MREGS_CFG_UIF0_CFG_VNTAG_MODE : NET_MREGS_CFG_UIF1_CFG_VNTAG_MODE)

#define UIF_CFG_EC(uif) \
    ((uif) == 0 ? \
     NET_MREGS_CFG_UIF0_EC : NET_MREGS_CFG_UIF1_EC)

#if defined CONFIG_BASE_CRUZ

/*
 * Each uif uses half the VIF_TBL.
 *  VIF_TBL_IDX(uif, i) is the index (0..32767) for entry 'i' of uif 'uif'.
 *  VIF_TBL(uif, i) is the physical address for entry 'i' of uif 'uif'.
 */
#define VIF_TBL_IDX(uif, i) \
    ((uif) * (NET_MREGS_DHS_VIF_TBL_MAXCNT / 2) + (i))

#define VIF_TBL(uif, i) \
    (NET_MREGS_DHS_VIF_TBL + \
     VIF_TBL_IDX((uif), (i)) * NET_MREGS_DHS_VIF_TBL_BLKSZ)

#endif

#if defined CONFIG_BASE_SERENO

#define VIF_TBL(idx) \
    (NET_MREGS_DHS_VIF_TBL + ((idx) * NET_MREGS_DHS_VIF_TBL_BLKSZ))

#define _UIF_CFG_IG_FWD_MASK_UIF_BLKSZ \
    (NET_MREGS_CFG_UIF1_PORT0_CFG_IG_FWD_MASK - \
     NET_MREGS_CFG_UIF0_PORT0_CFG_IG_FWD_MASK)
#define _UIF_CFG_IG_FWD_MASK_PORT_BLKSZ \
    (NET_MREGS_CFG_UIF0_PORT1_CFG_IG_FWD_MASK - \
     NET_MREGS_CFG_UIF0_PORT0_CFG_IG_FWD_MASK)

#define UIF_CFG_IG_FWD_MASK(uif, port) \
    ((uintptr_t)(NET_MREGS_CFG_UIF0_PORT0_CFG_IG_FWD_MASK + \
                 (((uif) * _UIF_CFG_IG_FWD_MASK_UIF_BLKSZ) + \
                  ((port) * _UIF_CFG_IG_FWD_MASK_PORT_BLKSZ))))
#endif

#endif /* CONFIG_BASE */

#define UIF_CFG_SEARCH(i) \
    ((uintptr_t)NET_MREGS_CFG_UIF##i##_SEARCH_ADDR_COMPUTE( \
    NET_MREGS_BASE_ADDR))
#define LIF_CFG(i) \
    ((uintptr_t)NET_MREGS_DHS_LIF_CFG_ADDR_COMPUTE( \
    NET_MREGS_BASE_ADDR, i))
#define CFG_IG_PFIFO_PTR(uif, cos) \
    ((uintptr_t)NET_MREGS_CFG_IG##uif##_PFIFO_PTR_ADDR_COMPUTE( \
    NET_MREGS_BASE_ADDR, cos))
#define CFG_IG_PFIFO_PAUSE(uif, cos) \
    ((uintptr_t)NET_MREGS_CFG_IG##uif##_PFIFO_PAUSE_ADDR_COMPUTE( \
    NET_MREGS_BASE_ADDR, cos))
#define CFG_IG_PFIFO_DROP(uif) \
    ((uintptr_t)NET_MREGS_CFG_IG##uif##_PFIFO_DROP_ADDR_COMPUTE( \
    NET_MREGS_BASE_ADDR, 0))
#define CFG_EG_PFIFO_PTR(uif, cos) \
    (NET_MREGS_CFG_EG0_PFIFO_PTR + (NET_MREGS_CFG_EG1_PFIFO_PTR - \
    NET_MREGS_CFG_EG0_PFIFO_PTR) * (uif) + 4 * (cos))
#define STA_EG_PFIFO_PTR(uif, cos) \
    (NET_MREGS_STA_EG0_PFIFO_PTR + (NET_MREGS_STA_EG1_PFIFO_PTR - \
    NET_MREGS_STA_EG0_PFIFO_PTR) * (uif) + 4 * (cos))
#define CFG_LOOP_PFIFO_PTR(uif, cos) \
    (NET_MREGS_CFG_LOOP0_PFIFO_PTR + (NET_MREGS_CFG_LOOP1_PFIFO_PTR - \
    NET_MREGS_CFG_LOOP0_PFIFO_PTR) * (uif) + 4 * (cos))
#define STA_LOOP_PFIFO_PTR(uif, cos) \
    (NET_MREGS_STA_LOOP0_PFIFO_PTR + (NET_MREGS_STA_LOOP1_PFIFO_PTR - \
    NET_MREGS_STA_LOOP0_PFIFO_PTR) * (uif) + 4 * (cos))
#define CFG_LOOP_PFIFO_PAUSE(uif) \
    (NET_MREGS_CFG_LOOP0_PFIFO_PAUSE + (NET_MREGS_CFG_LOOP1_PFIFO_PAUSE - \
    NET_MREGS_CFG_LOOP0_PFIFO_PAUSE) * (uif))
#define STA_IG_PFIFO_PTR(uif, cos) \
    (NET_MREGS_STA_IG0_PFIFO_PTR + (NET_MREGS_STA_IG1_PFIFO_PTR - \
    NET_MREGS_STA_IG0_PFIFO_PTR) * (uif) + 4 * (cos))
#define CFG_IG_PFIFO_NET_COS(uif) \
    (NET_MREGS_CFG_IG0_PFIFO_NET_COS + (NET_MREGS_CFG_IG1_PFIFO_NET_COS - \
    NET_MREGS_CFG_IG0_PFIFO_NET_COS) * (uif))
#define CFG_EG_PFIFO_NET_COS(uif) \
    (NET_MREGS_CFG_EG0_PFIFO_NET_COS + (NET_MREGS_CFG_EG1_PFIFO_NET_COS - \
    NET_MREGS_CFG_EG0_PFIFO_NET_COS) * (uif))
#define CFG_IG_N2I_COS_MAP(uif) \
    (NET_MREGS_CFG_UIF0_IG_N2I_COS_MAP + (NET_MREGS_CFG_UIF1_IG_N2I_COS_MAP - \
     NET_MREGS_CFG_UIF0_IG_N2I_COS_MAP) * (uif))
#define CFG_EG_I2N_COS_MAP(uif) \
    (NET_MREGS_CFG_UIF0_EG_I2N_COS_MAP + (NET_MREGS_CFG_UIF1_EG_I2N_COS_MAP - \
     NET_MREGS_CFG_UIF0_EG_I2N_COS_MAP) * (uif))
#define ACL_ENABLE \
    ((uintptr_t)NET_MREGS_CFG_ACL_ENABLE_ADDR_COMPUTE( \
    NET_MREGS_BASE_ADDR))
#define ACL_CFG(i) \
    ((uintptr_t)NET_MREGS_DHS_ACL_CFG_ADDR_COMPUTE( \
    NET_MREGS_BASE_ADDR, ((i) * 32)))
#define ACL_CFG_MASK(i) \
    ((uintptr_t)NET_MREGS_DHS_ACL_CFG_ADDR_COMPUTE( \
    NET_MREGS_BASE_ADDR, ((i) * 32 + 16)))

#if defined CONFIG_BASE_PALO || defined CONFIG_BASE_SERENO
#define ACL_RQ_NUM(i) \
    ((uintptr_t)NET_MREGS_CFG_ACL_RQ_NUM_ADDR_COMPUTE( \
    NET_MREGS_BASE_ADDR, (i / 2)))
#elif defined CONFIG_BASE_CRUZ
#define ACL_IG_RQ_NUM(i) \
    ((uintptr_t)NET_MREGS_CFG_ACL_IG_RQ_NUM_ADDR_COMPUTE( \
    NET_MREGS_BASE_ADDR, (i / 2)))
#define ACL_EG_RQ_NUM(i) \
    ((uintptr_t)NET_MREGS_CFG_ACL_EG_RQ_NUM_ADDR_COMPUTE( \
    NET_MREGS_BASE_ADDR, (i / 2)))
#endif
#if defined CONFIG_BASE_SERENO || defined CONFIG_BASE_CRUZ
#define ACL_IG_ACTION \
    ((uintptr_t)NET_MREGS_CFG_ACL_IG_ACTION_ADDR_COMPUTE( \
    NET_MREGS_BASE_ADDR))
#define ACL_EG_ACTION \
    ((uintptr_t)NET_MREGS_CFG_ACL_EG_ACTION_ADDR_COMPUTE( \
    NET_MREGS_BASE_ADDR))
#endif

#if defined CONFIG_BASE_SERENO
#define CLSF_TBL_MREGS_ADDR(i)        (CLSF_TBL_MREGS_BASE + \
                                       (CLSF_TBL_MREGS_BLKSZ * (i)))
#define CLSF_TBL_MREGS_(reg, i)       (CLSF_TBL_MREGS_ADDR(i) + \
                                       CLSF_TBL_MREGS_OFFSET(reg))

#define CLSF_TCAM_X_MREGS_ADDR(i)  (CLSF_TCAM_X_MREGS_BASE + \
                                        (CLSF_TCAM_MREGS_BLKSZ * (i)))
#define CLSF_TCAM_X_MREGS_(reg, i) (CLSF_TCAM_X_MREGS_ADDR(i) + \
                                        CLSF_TCAM_X_MREGS_OFFSET(reg))
#define CLSF_TCAM_Y_MREGS_ADDR(i)  (CLSF_TCAM_Y_MREGS_BASE + \
                                        (CLSF_TCAM_MREGS_BLKSZ * (i)))
#define CLSF_TCAM_Y_MREGS_(reg, i) (CLSF_TCAM_Y_MREGS_ADDR(i) + \
                                        CLSF_TCAM_MREGS_OFFSET(reg))

#define CLSF_SDATA_CONT_MREGS_ADDR(i)  (CLSF_SDATA_CONT_MREGS_BASE + \
                                        (CLSF_SDATA_MREGS_BLKSZ * (i)))
#define CLSF_SDATA_CONT_MREGS_(reg, i) (CLSF_SDATA_CONT_MREGS_ADDR(i) + \
                                        CLSF_SDATA_CONT_MREGS_OFFSET(reg))
#define CLSF_SDATA_NO_CONT_MREGS_ADDR(i)  (CLSF_SDATA_NO_CONT_MREGS_BASE + \
                                        (CLSF_SDATA_MREGS_BLKSZ * (i)))
#define CLSF_SDATA_NO_CONT_MREGS_(reg, i) (CLSF_SDATA_NO_CONT_MREGS_ADDR(i) + \
                                        CLSF_SDATA_NO_CONT_MREGS_OFFSET(reg))
#endif

#if defined CONFIG_BASE_SERENO || defined CONFIG_BASE_CRUZ
#define EC_BASE_ADDR(uif) \
    (EC_MREGS0_BASE_ADDR + (EC_MREGS1_BASE_ADDR - EC_MREGS0_BASE_ADDR) * (uif))

#define EC_CFG(uif) (EC_BASE_ADDR(uif) + EC_MREGS_CFG_OFFSET)

#define CFG_EC_IG_PFIFO_PTR(uif, cos) \
    (EC_MREGS_CFG_IG_PFIFO_PTR + \
     (EC_MREGS1_BASE_ADDR - EC_MREGS0_BASE_ADDR) * (uif) + ((cos) * 4))

#define CFG_EC_IG_PFIFO_PAUSE(uif) \
    (EC_MREGS_CFG_IG_PFIFO_PAUSE + \
     (EC_MREGS1_BASE_ADDR - EC_MREGS0_BASE_ADDR) * (uif))

#define CFG_EC_IG_PFIFO_PAUSE_UIFCOS(uif, cos) \
    (EC_MREGS_CFG_IG_PFIFO_PAUSE + \
     (EC_MREGS1_BASE_ADDR - EC_MREGS0_BASE_ADDR) * (uif) + ((cos) * 4))

#define CFG_EC_IG_PFIFO_DROP(uif) \
    (EC_MREGS_CFG_IG_PFIFO_DROP + \
     (EC_MREGS1_BASE_ADDR - EC_MREGS0_BASE_ADDR) * (uif))

#define CFG_EC_IG_PFIFO_DROP_MODE(uif) \
    (EC_MREGS_CFG_IG_PFIFO_DROP_MODE + \
     (EC_MREGS1_BASE_ADDR - EC_MREGS0_BASE_ADDR) * (uif))

#define CFG_EC_IG_PFIFO_NET_COS_SZ \
    (EC_MREGS_CFG_IG_XG1_PFIFO_NET_COS - EC_MREGS_CFG_IG_XG0_PFIFO_NET_COS)
#define CFG_EC_IG_PFIFO_NET_COS(uif, port) \
    (EC_MREGS_CFG_IG_XG0_PFIFO_NET_COS + \
     (EC_MREGS1_BASE_ADDR - EC_MREGS0_BASE_ADDR) * ((uif)) + \
     (CFG_EC_IG_PFIFO_NET_COS_SZ * (port)))

#define CFG_EC_EG_PPS(uif) (EC_BASE_ADDR(uif) + EC_MREGS_CFG_EG_PPS_OFFSET)
#define CFG_EC_PPS_DWRR(uif) (EC_BASE_ADDR(uif) + EC_MREGS_CFG_PPS_DWRR_OFFSET)

#ifdef _EC_MREGS_H_
/*
 * default_queue registers are not equal spaced between sub-ports.
 * This array is used to map the subport to the corresponding
 * default_queue register address.
 */
static const u_int32_t ec_cfg_ig_xgn_default_queue[PALO_MAX_UIFPORT] = {
    EC_MREGS_CFG_IG_XG0_DEFAULT_QUEUE,
    EC_MREGS_CFG_IG_XG1_DEFAULT_QUEUE,
    EC_MREGS_CFG_IG_XG2_DEFAULT_QUEUE,
    EC_MREGS_CFG_IG_XG3_DEFAULT_QUEUE
};
#define CFG_EC_IG_DEFAULT_QUEUE(uif, port) \
    (ec_cfg_ig_xgn_default_queue[(port)] + \
     (EC_MREGS1_BASE_ADDR - EC_MREGS0_BASE_ADDR) * (uif))
#endif /* _EC_MREGS_H_ */

#define CFG_EC_IG_COS2QUEUE_MAP(uif, port) \
    (EC_MREGS_CFG_IG_XG0_COS2QUEUE_MAP + \
     (EC_MREGS1_BASE_ADDR - EC_MREGS0_BASE_ADDR) * ((uif)) + \
     ((EC_MREGS_CFG_IG_XG1_COS2QUEUE_MAP - EC_MREGS_CFG_IG_XG0_COS2QUEUE_MAP) \
       * (port)))

#define CFG_EC_EG_PFIFO_PTR_SZ \
    (EC_MREGS_CFG_EG1_PFIFO_PTR - EC_MREGS_CFG_EG0_PFIFO_PTR)
#define CFG_EC_EG_PFIFO_PTR(uif, port, cos) \
    (EC_MREGS_CFG_EG0_PFIFO_PTR + \
     (EC_MREGS1_BASE_ADDR - EC_MREGS0_BASE_ADDR) * (uif) + \
     (CFG_EC_EG_PFIFO_PTR_SZ * (port)) + ((cos) * 4))

#define STA_EC_EG_PFIFO_PTR_SZ \
    (EC_MREGS_STA_EG1_PFIFO_PTR - EC_MREGS_STA_EG0_PFIFO_PTR)
#define STA_EC_EG_PFIFO_PTR(uif, port, cos) \
    (EC_MREGS_STA_EG0_PFIFO_PTR + \
     (EC_MREGS1_BASE_ADDR - EC_MREGS0_BASE_ADDR) * (uif) + \
     (STA_EC_EG_PFIFO_PTR_SZ * (port)) + ((cos) * 4))

#define CFG_EC_EG_PFIFO_NET_COS_SZ \
    (EC_MREGS_CFG_EG_XG1_PFIFO_NET_COS - EC_MREGS_CFG_EG_XG0_PFIFO_NET_COS)
#define CFG_EC_EG_PFIFO_NET_COS(uif, port) \
    (EC_MREGS_CFG_EG_XG0_PFIFO_NET_COS + \
     (EC_MREGS1_BASE_ADDR - EC_MREGS0_BASE_ADDR) * (uif) + \
     (CFG_EC_EG_PFIFO_NET_COS_SZ * (port)))

#define CFG_EC_EG_PFIFO_PAUSE(uif) \
    (((uif == 0) ? EC_MREGS0_BASE_ADDR : EC_MREGS1_BASE_ADDR) + \
     EC_MREGS_CFG_EG_PFIFO_PAUSE_OFFSET)

#define STA_EC_IG_PFIFO_PTR(uif, cos) \
    (EC_MREGS_STA_IG_PFIFO_PTR + \
     (EC_MREGS1_BASE_ADDR - EC_MREGS0_BASE_ADDR) * (uif) + ((cos) * 4))

#endif  /* CONFIG_BASE */

/*
 * Extract the valid field from an array of lif_cfg words
 */
static inline int
dhs_lif_cfg_get_valid(u_int32_t *lif_cfg)
{
    int valid;

    valid = (lif_cfg[1] >> NET_MREGS_DHS_LIF_CFG_01_VALID_SHIFT) &
      NET_MREGS_DHS_LIF_CFG_01_VALID_MASK_FIELD;

    return valid;
}

/*
 * Set/clear the valid field in an array of lif_cfg words
 */
static inline void
dhs_lif_cfg_set_valid(u_int32_t *lif_cfg, int valid)
{
    lif_cfg[1] &= ~(NET_MREGS_DHS_LIF_CFG_01_VALID_MASK_FIELD <<
          NET_MREGS_DHS_LIF_CFG_01_VALID_SHIFT);
    if (valid) {
        lif_cfg[1] |= (NET_MREGS_DHS_LIF_CFG_01_VALID_MASK_FIELD <<
          NET_MREGS_DHS_LIF_CFG_01_VALID_SHIFT);
    }
}

#endif // __REGS_MACROS_H__
