
// -------------------------------------------------------------------
//
//
//                   (C) COPYRIGHT 2002-2004 SYNOPSYS INC.
//                             ALL RIGHTS RESERVED
//
//  This software and the associated documentation are confidential and
//  proprietary to Synopsys, Inc.  Your use or disclosure of this software
//  is subject to the terms and conditions of a written license agreement
//  between you, or your company, and Synopsys, Inc.
//
//
// File    : DW_apb_i2c_defs.h
// Author  : Joe Mc Cann
// Created : Fri Aug  2 12:01:06 IST 2002
// Abstract: Memory map header file for the DW_apb_i2c peripheral
//
// -------------------------------------------------------------------
// Revision: $Id: DW_apb_i2c_defs.h,v 1.2 2006/07/18 06:49:21 scott Exp $
// -------------------------------------------------------------------

#ifndef _DW_I2C_H_
#define _DW_I2C_H_

#define DW_APB_I2C_BASE             0x17f20000
#define DW_APB_I2C_STRIDE           0x10000

#define IC_CON_OFFSET               0x00
#define IC_TAR_OFFSET               0x04
#define IC_SAR_OFFSET               0x08
#define IC_HS_MADDR_OFFSET          0x0c
#define IC_DATA_CMD_OFFSET          0x10
#define IC_SS_HCNT_OFFSET           0x14
#define IC_SS_LCNT_OFFSET           0x18
#define IC_FS_HCNT_OFFSET           0x1c
#define IC_FS_LCNT_OFFSET           0x20
#define IC_HS_HCNT_OFFSET           0x24
#define IC_HS_LCNT_OFFSET           0x28
#define IC_INTR_STAT_OFFSET         0x2c
#define IC_INTR_MASK_OFFSET         0x30
#define IC_RAW_INTR_STAT_OFFSET     0x34
#define IC_RX_TL_OFFSET             0x38
#define IC_TX_TL_OFFSET             0x3c
#define IC_CLR_INTR_OFFSET          0x40
#define IC_CLR_RX_UNDER_OFFSET      0x44
#define IC_CLR_RX_OVER_OFFSET       0x48
#define IC_CLR_TX_OVER_OFFSET       0x4c
#define IC_CLR_RD_REQ_OFFSET        0x50
#define IC_CLR_TX_ABRT_OFFSET       0x54
#define IC_CLR_RX_DONE_OFFSET       0x58
#define IC_CLR_ACTIVITY_OFFSET      0x5c
#define IC_CLR_STOP_DET_OFFSET      0x60
#define IC_CLR_START_DET_OFFSET     0x64
#define IC_CLR_GEN_CALL_OFFSET      0x68
#define IC_ENABLE_OFFSET            0x6c
#define IC_STATUS_OFFSET            0x70
#define IC_TXFLR_OFFSET             0x74
#define IC_RXFLR_OFFSET             0x78
#define IC_SRESET_OFFSET            0x7c
#define IC_TX_ABRT_SOURCE_OFFSET    0x80
#define IC_VERSION_ID_OFFSET        0xf8
#define IC_DMA_CR_OFFSET            0x88
#define IC_DMA_TDLR_OFFSET          0x8c
#define IC_DMA_RDLR_OFFSET          0x90
#define IC_COMP_PARAM_1_OFFSET      0xf4
#define IC_COMP_VERSION_OFFSET      0xf8
#define IC_COMP_TYPE_OFFSET         0xfc

#define IC_REG(d, r)                (DW_APB_I2C_BASE + \
                                     (d) * DW_APB_I2C_STRIDE + \
                                     IC_##r##_OFFSET)

#define IC_CON(d)                   IC_REG(d, CON)
#define IC_TAR(d)                   IC_REG(d, TAR)
#define IC_SAR(d)                   IC_REG(d, SAR)
#define IC_HS_MADDR(d)              IC_REG(d, HS_MADDR)
#define IC_DATA_CMD(d)              IC_REG(d, DATA_CMD)
#define IC_SS_HCNT(d)               IC_REG(d, SS_HCNT)
#define IC_SS_LCNT(d)               IC_REG(d, SS_LCNT)
#define IC_FS_HCNT(d)               IC_REG(d, FS_HCNT)
#define IC_FS_LCNT(d)               IC_REG(d, FS_LCNT)
#define IC_HS_HCNT(d)               IC_REG(d, HS_HCNT)
#define IC_HS_LCNT(d)               IC_REG(d, HS_LCNT)
#define IC_INTR_STAT(d)             IC_REG(d, INTR_STAT)
#define IC_INTR_MASK(d)             IC_REG(d, INTR_MASK)
#define IC_RAW_INTR_STAT(d)         IC_REG(d, RAW_INTR_STAT)
#define IC_RX_TL(d)                 IC_REG(d, RX_TL)
#define IC_TX_TL(d)                 IC_REG(d, TX_TL)
#define IC_CLR_INTR(d)              IC_REG(d, CLR_INTR)
#define IC_CLR_RX_UNDER(d)          IC_REG(d, CLR_RX_UNDER)
#define IC_CLR_RX_OVER(d)           IC_REG(d, CLR_RX_OVER)
#define IC_CLR_TX_OVER(d)           IC_REG(d, CLR_TX_OVER)
#define IC_CLR_RD_REQ(d)            IC_REG(d, CLR_RD_REQ)
#define IC_CLR_TX_ABRT(d)           IC_REG(d, CLR_TX_ABRT)
#define IC_CLR_RX_DONE(d)           IC_REG(d, CLR_RX_DONE)
#define IC_CLR_ACTIVITY(d)          IC_REG(d, CLR_ACTIVITY)
#define IC_CLR_STOP_DET(d)          IC_REG(d, CLR_STOP_DET)
#define IC_CLR_START_DET(d)         IC_REG(d, CLR_START_DET)
#define IC_CLR_GEN_CALL(d)          IC_REG(d, CLR_GEN_CALL)
#define IC_ENABLE(d)                IC_REG(d, ENABLE)
#define IC_STATUS(d)                IC_REG(d, STATUS)
#define IC_TXFLR(d)                 IC_REG(d, TXFLR)
#define IC_RXFLR(d)                 IC_REG(d, RXFLR)
#define IC_SRESET(d)                IC_REG(d, SRESET)
#define IC_TX_ABRT_SOURCE(d)        IC_REG(d, TX_ABRT_SOURCE)
#define IC_VERSION_ID(d)            IC_REG(d, VERSION_ID)
#define IC_DMA_CR(d)                IC_REG(d, DMA_CR)
#define IC_DMA_TDLR(d)              IC_REG(d, DMA_TDLR)
#define IC_DMA_RDLR(d)              IC_REG(d, DMA_RDLR)
#define IC_COMP_PARAM_1(d)          IC_REG(d, COMP_PARAM_1)
#define IC_COMP_VERSION(d)          IC_REG(d, COMP_VERSION)
#define IC_COMP_TYPE(d)             IC_REG(d, COMP_TYPE)

/*
 * IC_CON
 */
#define IC_CON_SLAVE_DISABLE        0x40
#define IC_CON_RESTART_EN           0x20
#define IC_CON_10BITADDR_MASTER     0x10
#define IC_CON_10BITADDR_SLAVE      0x08
#define IC_CON_SPEED_MASK           0x06
#define IC_CON_SPEED_HIGH           0x06
#define IC_CON_SPEED_FAST           0x04
#define IC_CON_SPEED_STANDARD       0x02
#define IC_CON_MASTER_MODE          0x01

/*
 * IC_STATUS
 */
#define IC_STATUS_SLV_ACTIVITY      0x40
#define IC_STATUS_MST_ACTIVITY      0x20
#define IC_STATUS_RFF               0x10
#define IC_STATUS_RFNE              0x08
#define IC_STATUS_TFE               0x04
#define IC_STATUS_TFNF              0x02
#define IC_STATUS_ACTIVITY          0x01

/*
 * IC_INTR_STAT, IC_INTR_MASK, IC_RAW_INTR_STAT
 */
#define IC_INTR_GEN_CALL            0x0800
#define IC_INTR_START_DET           0x0400
#define IC_INTR_STOP_DET            0x0200
#define IC_INTR_ACTIVITY            0x0100
#define IC_INTR_RX_DONE             0x0080
#define IC_INTR_TX_ABRT             0x0040
#define IC_INTR_RD_REQ              0x0020
#define IC_INTR_TX_EMPTY            0x0010
#define IC_INTR_TX_OVER             0x0008
#define IC_INTR_RX_FULL             0x0004
#define IC_INTR_RX_OVER             0x0002
#define IC_INTR_RX_UNDER            0x0001

/*
 * IC_DATA_CMD
 */
#define IC_DATA_CMD_READ            0x0100
#define IC_DATA_CMD_WRITE           0x0000

#endif
